@@ -74,7 +74,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -158,8 +158,8 @@ add_buffer( VipsArithmetic *arithmetic, VipsPel *out, VipsPel **in, int width )
  * match the case statement in add_buffer() above.
  */
 static const VipsBandFormat vips_add_format_table[10] = {
-/* UC  C   US  S   UI  I  F  X  D  DX */
-   US, S,  UI, I,  UI, I, F, X, D, DX
+	/* Band format:  UC  C  US  S  UI  I  F  X  D  DX */
+	/* Promotion: */ US, S, UI, I, UI, I, F, X, D, DX
 };
 
 static void
