@@ -67,7 +67,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -224,8 +224,8 @@ vips_boolean_buffer( VipsArithmetic *arithmetic,
 /* Type conversions for boolean. 
  */
 static const VipsBandFormat vips_boolean_format_table[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, C,  US, S,  UI, I,  I,  I,  I,  I,
+	/* Band format:  UC  C  US  S  UI  I  F  X  D  DX */
+	/* Promotion: */ UC, C, US, S, UI, I, I, I, I, I
 };
 
 static void
@@ -248,7 +248,7 @@ vips_boolean_class_init( VipsBooleanClass *class )
 
 	VIPS_ARG_ENUM( class, "boolean", 200, 
 		_( "Operation" ), 
-		_( "boolean to perform" ),
+		_( "Boolean to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsBoolean, operation ),
 		VIPS_TYPE_OPERATION_BOOLEAN, 
@@ -548,7 +548,7 @@ vips_boolean_const_class_init( VipsBooleanConstClass *class )
 
 	VIPS_ARG_ENUM( class, "boolean", 200, 
 		_( "Operation" ), 
-		_( "boolean to perform" ),
+		_( "Boolean to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsBooleanConst, operation ),
 		VIPS_TYPE_OPERATION_BOOLEAN, 
@@ -562,7 +562,7 @@ vips_boolean_const_init( VipsBooleanConst *boolean_const )
 
 static int
 vips_boolean_constv( VipsImage *in, VipsImage **out, 
-	VipsOperationBoolean operation, double *c, int n, va_list ap )
+	VipsOperationBoolean operation, const double *c, int n, va_list ap )
 {
 	VipsArea *area_c;
 	double *array; 
@@ -609,7 +609,7 @@ vips_boolean_constv( VipsImage *in, VipsImage **out,
  */
 int
 vips_boolean_const( VipsImage *in, VipsImage **out, 
-	VipsOperationBoolean boolean, double *c, int n, ... )
+	VipsOperationBoolean boolean, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -637,7 +637,8 @@ vips_boolean_const( VipsImage *in, VipsImage **out,
  * Returns: 0 on success, -1 on error
  */
 int
-vips_andimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_andimage_const( VipsImage *in, VipsImage **out, 
+	const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -666,7 +667,8 @@ vips_andimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_orimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_orimage_const( VipsImage *in, VipsImage **out, 
+	const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -695,7 +697,8 @@ vips_orimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_eorimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_eorimage_const( VipsImage *in, VipsImage **out, 
+	const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -724,7 +727,7 @@ vips_eorimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_lshift_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_lshift_const( VipsImage *in, VipsImage **out, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -753,7 +756,7 @@ vips_lshift_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_rshift_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_rshift_const( VipsImage *in, VipsImage **out, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
