@@ -59,7 +59,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -136,6 +136,8 @@ vips_complex_hypot( double a, double b )
 {
 	double d;
 
+	/* hypot() is less sensitive to overflow. Use it if we can.
+	 */
 #ifdef HAVE_HYPOT
 	d = hypot( a, b ); 
 #else
@@ -150,6 +152,9 @@ vips_complex_atan2( double a, double b )
 {
 	double h;
 
+	/* atan2() is very slow, but is better behaved when a is near 0. Use
+	 * it in preference when we can.
+	 */
 #ifdef HAVE_ATAN2
 	h = VIPS_DEG( atan2( b, a ) );
 	if( h < 0.0 )
@@ -229,8 +234,8 @@ vips_complex_buffer( VipsArithmetic *arithmetic,
 #define DX VIPS_FORMAT_DPCOMPLEX
 
 static const VipsBandFormat vips_complex_format_table[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   X,  X,  X,  X,  X,  X,  X,  X,  DX, DX 
+	/* Band format:  UC C  US S  UI I  F  X  D   DX */
+	/* Promotion: */ X, X, X, X, X, X, X, X, DX, DX
 };
 
 static void
@@ -253,7 +258,7 @@ vips_complex_class_init( VipsComplexClass *class )
 
 	VIPS_ARG_ENUM( class, "cmplx", 200, 
 		_( "Operation" ), 
-		_( "complex to perform" ),
+		_( "Complex to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsComplex, cmplx ),
 		VIPS_TYPE_OPERATION_COMPLEX, VIPS_OPERATION_COMPLEX_POLAR ); 
@@ -522,8 +527,8 @@ vips_complex2_buffer( VipsArithmetic *arithmetic,
 #define DX VIPS_FORMAT_DPCOMPLEX
 
 static const VipsBandFormat vips_complex2_format_table[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   X,  X,  X,  X,  X,  X,  X,  X,  DX, DX 
+	/* Band format:  UC C  US S  UI I  F  X  D   DX */
+	/* Promotion: */ X, X, X, X, X, X, X, X, DX, DX
 };
 
 static void
@@ -546,7 +551,7 @@ vips_complex2_class_init( VipsComplex2Class *class )
 
 	VIPS_ARG_ENUM( class, "cmplx", 200, 
 		_( "Operation" ), 
-		_( "binary complex operation to perform" ),
+		_( "Binary complex operation to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsComplex2, cmplx ),
 		VIPS_TYPE_OPERATION_COMPLEX2, 
@@ -649,7 +654,7 @@ vips_complexget_build( VipsObject *object )
 }
 
 #define GETLOOP( TYPE, OP ) { \
-	TYPE *p __attribute__ ((unused)) = (TYPE *) in[0]; \
+	TYPE *p G_GNUC_UNUSED = (TYPE *) in[0]; \
 	TYPE *q = (TYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
@@ -658,7 +663,7 @@ vips_complexget_build( VipsObject *object )
 }
 
 #define CGETLOOP( TYPE, OP ) { \
-	TYPE *p __attribute__ ((unused)) = (TYPE *) in[0]; \
+	TYPE *p G_GNUC_UNUSED = (TYPE *) in[0]; \
 	TYPE *q = (TYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
@@ -736,8 +741,8 @@ vips_complexget_buffer( VipsArithmetic *arithmetic,
 #define DX VIPS_FORMAT_DPCOMPLEX
 
 static const VipsBandFormat vips_complexget_format_table[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, C,  US, S,  UI, I,  F,  F,  D,  D
+	/* Band format:  UC  C  US  S  UI  I  F  X  D  DX */
+	/* Promotion: */ UC, C, US, S, UI, I, F, F, D, D
 };
 
 static void
@@ -761,7 +766,7 @@ vips_complexget_class_init( VipsComplexgetClass *class )
 
 	VIPS_ARG_ENUM( class, "get", 200, 
 		_( "Operation" ), 
-		_( "complex to perform" ),
+		_( "Complex to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsComplexget, get ),
 		VIPS_TYPE_OPERATION_COMPLEXGET, 
@@ -946,9 +951,9 @@ vips_complexform_buffer( VipsArithmetic *arithmetic,
 /* Type promotion for form complex. Sign and value preserving. Make sure 
  * these match the case statement in complexform_buffer() above.
  */
-static int vips_complexform_format_table[10] = {
-/* UC  C   US  S   UI  I  F  X  D  DX */
-   X,  X,  X,  X,  X,  X, X, X, DX,DX
+static VipsBandFormat vips_complexform_format_table[10] = {
+	/* Band format:  UC C  US S  UI I  F  X  D   DX */
+	/* Promotion: */ X, X, X, X, X, X, X, X, DX, DX
 };
 
 static void
