@@ -62,7 +62,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -206,9 +206,9 @@ vips_math2_buffer( VipsArithmetic *arithmetic,
 
 /* Type promotion for math2. Keep in sync with math2_buffer() above.
  */
-static int vips_math2_format_table[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   F,  F,  F,  F,  F,  F,  F,  X,  D,  DX 
+static const VipsBandFormat vips_math2_format_table[10] = {
+	/* Band format:  UC C  US S  UI I  F  X  D  DX */
+	/* Promotion: */ F, F, F, F, F, F, F, X, D, DX
 };
 
 static void
@@ -231,7 +231,7 @@ vips_math2_class_init( VipsMath2Class *class )
 
 	VIPS_ARG_ENUM( class, "math2", 200, 
 		_( "Operation" ), 
-		_( "math to perform" ),
+		_( "Math to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsMath2, math2 ),
 		VIPS_TYPE_OPERATION_MATH2, VIPS_OPERATION_MATH2_POW ); 
@@ -462,7 +462,7 @@ vips_math2_const_class_init( VipsMath2ConstClass *class )
 
 	VIPS_ARG_ENUM( class, "math2", 200, 
 		_( "Operation" ), 
-		_( "math to perform" ),
+		_( "Math to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsMath2Const, math2 ),
 		VIPS_TYPE_OPERATION_MATH2, VIPS_OPERATION_MATH2_POW ); 
