@@ -51,7 +51,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <math.h>
@@ -69,7 +69,7 @@
  */
 #define QUANT_ELEMENTS (100000)
 
-float cbrt_table[QUANT_ELEMENTS];
+static float cbrt_table[QUANT_ELEMENTS];
 
 typedef struct _VipsXYZ2Lab {
 	VipsColourTransform parent_instance;
@@ -177,9 +177,9 @@ vips_XYZ2Lab_line( VipsColour *colour, VipsPel *out, VipsPel **in, int width )
  * @X: Input CIE XYZ colour
  * @Y: Input CIE XYZ colour
  * @Z: Input CIE XYZ colour
- * @L: Return CIE Lab value
- * @a: Return CIE Lab value
- * @b: Return CIE Lab value
+ * @L: (out): Return CIE Lab value
+ * @a: (out): Return CIE Lab value
+ * @b: (out): Return CIE Lab value
  *
  * Calculate XYZ from Lab, D65.
  * 
