@@ -51,7 +51,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -896,7 +896,7 @@ vips_composite_base_blend3( VipsCompositeSequence *seq,
 			break;
 
 		case VIPS_BLEND_MODE_HARD_LIGHT:
-			f = A < 0.5 ? 
+			f = A <= 0.5 ? 
 				2 * A * B :
 				1 - 2 * (1 - A) * (1 - B);
 			break;
@@ -1785,8 +1785,10 @@ vips_composite2( VipsImage *base, VipsImage *overlay, VipsImage **out,
 	va_list ap;
 	int result;
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wvarargs"
+	/* Works for gcc and clang.
+	 */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wvarargs"
 
 	/* Triggers a clang compiler warning because mode might not be an int.
 	 * I think the warning is harmless for all platforms we care about.
@@ -1795,7 +1797,7 @@ vips_composite2( VipsImage *base, VipsImage *overlay, VipsImage **out,
 
 	g_assert( sizeof( mode ) == sizeof( int ) );
 
-#pragma clang diagnostic pop
+#pragma GCC diagnostic pop
 
 	result = vips_call_split( "composite2", ap, base, overlay, out, mode );
 	va_end( ap );
