@@ -72,7 +72,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -620,7 +620,7 @@ vips_tile_cache_gen( VipsRegion *or,
 
 	VIPS_GATE_START( "vips_tile_cache_gen: wait1" );
 
-	g_mutex_lock( cache->lock );
+	vips__worker_lock( cache->lock );
 
 	VIPS_GATE_STOP( "vips_tile_cache_gen: wait1" );
 
@@ -895,7 +895,7 @@ vips_line_cache_gen( VipsRegion *or,
 
 	VIPS_GATE_START( "vips_line_cache_gen: wait" );
 
-	g_mutex_lock( block_cache->lock );
+	vips__worker_lock( block_cache->lock );
 
 	VIPS_GATE_STOP( "vips_line_cache_gen: wait" );
 
