@@ -83,6 +83,14 @@ static int
 vips_black_gen( VipsRegion *or, void *seq, void *a, void *b,
 	gboolean *stop )
 {
+        /*
+        VipsRect *r = &or->valid;
+
+        printf( "vips_black_gen: "
+                "left = %d, top = %d, width = %d, height = %d\n",
+                r->left, r->top, r->width, r->height );
+         */
+
 	vips_region_black( or );
 
 	return( 0 );
