@@ -37,12 +37,10 @@
  */
 
 
-/** HEADERS **/
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdlib.h>
 
@@ -50,8 +48,6 @@
 #include <vips/vips7compat.h>
 
 
-/** LOCAL TYPES **/
-
 typedef struct {
   REGION *reg;
   int *region_xgrad;
@@ -61,9 +57,6 @@ typedef struct {
 }
 gradcor_seq_t;
 
-
-/** LOCAL FUNCTION DECLARATIONS **/
-
 static void *gradcor_start( IMAGE *out, void *vptr_large, void *unrequired );
 static int gradcor_stop( void *vptr_seq, void *unrequired, void *unreq2 );
 static int gradcor_gen( REGION *to_make, void *vptr_seq, void *unrequired, void *vptr_grads );
@@ -90,9 +83,6 @@ XGRAD_GEN_DECLARATION( double );
 YGRAD_GEN_DECLARATION( double );
 #endif
 
-
-/** EXPORTED FUNCTION DEFINITIONS **/
-
 int im_gradcor_raw( IMAGE *large, IMAGE *small, IMAGE *out ){
 #define FUNCTION_NAME "im_gradcor_raw"
 
@@ -335,9 +325,6 @@ int im_grad_y( IMAGE *in, IMAGE *out ){
 #undef FUNCTION_NAME
 }
 
-
-/** LOCAL FUNCTION DEFINITIONS **/
-
 static void *gradcor_start( IMAGE *out, void *vptr_large, void *unrequired ){
 
   gradcor_seq_t *seq= IM_NEW( NULL, gradcor_seq_t );
