@@ -30,6 +30,8 @@
  * 	- use Draw base class
  * 21/1/14
  * 	- redo as a class
+ * 5/4/20
+ * 	- could fail to complete for some very complex shapes
  */
 
 /*
@@ -62,7 +64,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -184,7 +186,7 @@ buffer_free( Buffer *buf )
 /* Add a scanline to a buffer, prepending a new buffer if necessary. Return
  * the new head buffer.
  */
-static Buffer * 
+static Buffer *
 buffer_add( Buffer *buf, Flood *flood, int x1, int x2, int y, int dir )
 {
 	/* Clip against image size.
@@ -197,19 +199,19 @@ buffer_add( Buffer *buf, Flood *flood, int x1, int x2, int y, int dir )
 	if( x2 - x1 < 0 )
 		return( buf );
 
-	buf->scan[buf->n].x1 = x1;
-	buf->scan[buf->n].x2 = x2;
-	buf->scan[buf->n].y = y;
-	buf->scan[buf->n].dir = dir;
-	buf->n += 1; 
-
 	if( buf->n == PBUFSIZE ) { 
 		Buffer *new;
 
 		new = buffer_build();
 		new->next = buf;
 		buf = new;
-	} 
+	}
+
+	buf->scan[buf->n].x1 = x1;
+	buf->scan[buf->n].x2 = x2;
+	buf->scan[buf->n].y = y;
+	buf->scan[buf->n].dir = dir;
+	buf->n += 1;
 
 	return( buf );
 }
@@ -596,21 +598,21 @@ vips_draw_flood_class_init( VipsDrawFloodClass *class )
 
 	VIPS_ARG_INT( class, "top", 8, 
 		_( "Top" ), 
-		_( "top edge of modified area" ),
+		_( "Top edge of modified area" ),
 		VIPS_ARGUMENT_OPTIONAL_OUTPUT,
 		G_STRUCT_OFFSET( VipsDrawFlood, top ),
 		0, 1000000000, 0 );
 
 	VIPS_ARG_INT( class, "width", 9, 
 		_( "Width" ), 
-		_( "width of modified area" ),
+		_( "Width of modified area" ),
 		VIPS_ARGUMENT_OPTIONAL_OUTPUT,
 		G_STRUCT_OFFSET( VipsDrawFlood, width ),
 		0, 1000000000, 0 );
 
 	VIPS_ARG_INT( class, "height", 10, 
 		_( "Height" ), 
-		_( "height of modified area" ),
+		_( "Height of modified area" ),
 		VIPS_ARGUMENT_OPTIONAL_OUTPUT,
 		G_STRUCT_OFFSET( VipsDrawFlood, height ),
 		0, 1000000000, 0 );
