@@ -176,6 +176,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 
 #include <vips/vips.h>
 #include <vips/internal.h>
@@ -304,6 +305,119 @@ gsf_output_target_new( VipsTarget *target )
         return( GSF_OUTPUT( output ) );
 }
 
+/* A GSF output object that can write to a directory.
+ */
+
+typedef struct _GsfOutputDir {
+	GsfOutfile output;
+
+	char *root;
+
+} GsfOutputDir;
+
+typedef struct {
+	GsfOutfileClass output_class;
+} GsfOutputDirClass;
+
+G_DEFINE_TYPE( GsfOutputDir, gsf_output_dir, GSF_OUTFILE_TYPE );
+
+static gboolean
+gsf_output_dir_close( GsfOutput *output )
+{
+	return( TRUE );
+}
+
+static void
+gsf_output_dir_finalize( GObject *obj )
+{
+	GsfOutputDir *output_dir = (GsfOutputDir *) obj;
+
+	g_free( output_dir->root );
+
+	G_OBJECT_CLASS( gsf_output_dir_parent_class )->finalize( obj );
+}
+
+static GsfOutfile *
+gsf_output_dir_new_valist( char const *root,
+	char const *first_property_name, va_list var_args )
+{
+	GsfOutputDir *output_dir;
+
+	if( g_mkdir( root, 0777 ) != 0 && errno != EEXIST ) {
+		int save_errno = errno;
+		char *utf8name = g_filename_display_name( root );
+		vips_error( "dzsave",
+			_( "unable to create directory \"%s\", %s" ),
+			utf8name, g_strerror( save_errno ) );
+		g_free( utf8name );
+		return( NULL );
+	}
+
+	output_dir = (GsfOutputDir *) g_object_new_valist( GSF_OUTFILE_STDIO_TYPE,
+		first_property_name, var_args );
+	output_dir->root = g_strdup( root );
+	gsf_output_set_name_from_filename( GSF_OUTPUT( output_dir ), root );
+
+	return( GSF_OUTFILE( output_dir ) );
+}
+
+static GsfOutput *
+gsf_output_dir_new_child( GsfOutfile *parent,
+	char const *name, gboolean is_dir,
+	char const *first_property_name,
+	va_list args )
+{
+	GsfOutputDir *output_dir = (GsfOutputDir *) parent;
+	GsfOutput *child;
+	char *path = g_build_filename( output_dir->root, name, NULL );
+
+	if( is_dir )
+		child = (GsfOutput *) gsf_output_dir_new_valist( path,
+			first_property_name, args );
+	else
+		child = gsf_output_stdio_new_valist( path, NULL,
+			first_property_name, args );
+	g_free( path );
+
+	return( child );
+}
+
+static void
+gsf_output_dir_init( GsfOutputDir *output )
+{
+	output->root = NULL;
+}
+
+static void
+gsf_output_dir_class_init( GsfOutputDirClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	GsfOutputClass *output_class = GSF_OUTPUT_CLASS( class );
+	GsfOutfileClass *outfile_class = GSF_OUTFILE_CLASS( class );
+
+	gobject_class->finalize = gsf_output_dir_finalize;
+
+	output_class->Close = gsf_output_dir_close;
+	output_class->Seek = NULL;
+	output_class->Write = NULL;
+	output_class->Vprintf = NULL;
+
+	outfile_class->new_child = gsf_output_dir_new_child;
+}
+
+static GsfOutput *
+gsf_output_dir_new( char const *root, ... )
+{
+	GsfOutfile *output;
+	va_list var_args;
+
+	va_start( var_args, root );
+	output = gsf_output_dir_new_valist( root, NULL, var_args );
+	va_end( var_args );
+
+	return( GSF_OUTPUT( output ) );
+}
+
 /* Simple wrapper around libgsf.
  *
  * We need to be able to do scattered writes to structured files. So while
@@ -466,12 +580,13 @@ vips_gsf_dir_new( VipsGsfDirectory *parent, const char *name )
 	return( dir ); 
 }
 
-/* Return a GsfOutput for writing to a path. Paths are object name first, then
- * path components with least-specific first, NULL-terminated. For example:
+/* Return a GsfOutput or %NULL for writing to a path. Paths are object name first,
+ * then path components with least-specific first, NULL-terminated. For example:
  *
  * GsfOutput *obj = vips_gsf_path( tree, "fred.jpg", "a", "b", NULL );
  *
- * Returns an obj you can use to write to a/b/fred.jpg. 
+ * Returns an obj you can use to write to a/b/fred.jpg, or %NULL when it exceeds
+ * the path limits.
  *
  * You must write, close and unref obj.
  */
@@ -767,8 +882,10 @@ write_image( VipsForeignSaveDz *dz,
 		gsf_output_close( out );
 		g_mutex_unlock( vips__global_lock );
 		g_free( buf );
-		vips_error( class->nickname,
-			"%s", gsf_output_error( out )->message );
+
+		if( gsf_output_error( out ) )
+			vips_error( class->nickname,
+				"%s", gsf_output_error( out )->message );
 
 		return( -1 );
 	}
@@ -998,7 +1115,8 @@ write_dzi( VipsForeignSaveDz *dz )
 	char *p;
 
 	vips_snprintf( buf, VIPS_PATH_MAX, "%s.dzi", dz->basename );
-	out = vips_gsf_path( dz->tree, buf, NULL ); 
+	if( !(out = vips_gsf_path( dz->tree, buf, NULL ) ))
+		return( -1 );
 
 	vips_snprintf( buf, VIPS_PATH_MAX, "%s", dz->suffix + 1 );
 	if( (p = (char *) vips__find_rightmost_brackets( buf )) )
@@ -1029,7 +1147,10 @@ write_properties( VipsForeignSaveDz *dz )
 {
 	GsfOutput *out;
 
-	out = vips_gsf_path( dz->tree, "ImageProperties.xml", NULL ); 
+	if( !(out = vips_gsf_path( dz->tree, 
+		"ImageProperties.xml", NULL ) )) {
+		return( -1 );
+	}
 
 	gsf_output_printf( out, "<IMAGE_PROPERTIES "
 		"WIDTH=\"%d\" HEIGHT=\"%d\" NUMTILES=\"%d\" "
@@ -1082,7 +1203,11 @@ write_blank( VipsForeignSaveDz *dz )
 	g_object_unref( x );
 	x = t;
 
-	out = vips_gsf_path( dz->tree, "blank.png", NULL ); 
+	if( !(out = vips_gsf_path( dz->tree, "blank.png", NULL ) )) {
+		g_object_unref( x );
+
+		return( -1 );
+	}
 
 	if( write_image( dz, out, x, ".png" ) ) {
 		g_object_unref( out );
@@ -1115,7 +1240,8 @@ write_json( VipsForeignSaveDz *dz )
 	GsfOutput *out;
 	int i;
 
-	out = vips_gsf_path( dz->tree, "info.json", NULL ); 
+	if( !(out = vips_gsf_path( dz->tree, "info.json", NULL ) ))
+		return( -1 );
 
 	if( dz->layout == VIPS_FOREIGN_DZ_LAYOUT_IIIF3 ) 
 		gsf_output_printf( out, 
@@ -1230,7 +1356,13 @@ write_vips_meta( VipsForeignSaveDz *dz )
 	else
 		out = vips_gsf_path( dz->tree, "vips-properties.xml", NULL );
 
-	gsf_output_write( out, strlen( dump ), (guchar *) dump ); 
+	if( out == NULL ) {
+		g_free( dump );
+
+		return( -1 );
+	}
+
+	(void) gsf_output_write( out, strlen( dump ), (guchar *) dump ); 
 	(void) gsf_output_close( out );
 	g_object_unref( out );
 
@@ -1340,9 +1472,15 @@ write_scan_properties( VipsForeignSaveDz *dz )
 
 	if( !(dump = build_scan_properties( save->ready )) )
                 return( -1 );
+	
+	if( !(out = vips_gsf_path( dz->tree, 
+		"scan-properties.xml", NULL ) )) {
+		g_free( dump );
 
-	out = vips_gsf_path( dz->tree, "scan-properties.xml", NULL );
-	gsf_output_write( out, strlen( dump ), (guchar *) dump );
+		return( -1 );
+	}
+
+	(void) gsf_output_write( out, strlen( dump ), (guchar *) dump );
 	(void) gsf_output_close( out );
 	g_object_unref( out );
 
@@ -1375,7 +1513,12 @@ write_associated_images( VipsImage *image,
 			return( image );
 
 		vips_snprintf( buf, VIPS_PATH_MAX, "%s.jpg", p );
-		out = vips_gsf_path( dz->tree, buf, "associated_images", NULL );
+		if( !(out = vips_gsf_path( dz->tree, buf, 
+			"associated_images", NULL ) )) {
+			g_object_unref( associated );
+
+			return( image );
+		}
 
 		if( write_image( dz, out, associated, ".jpg" ) ) {
 			g_object_unref( out );
@@ -1714,6 +1857,12 @@ strip_work( VipsThreadState *state, void *a )
 	printf( "strip_work\n" );
 #endif /*DEBUG_VERBOSE*/
 
+        /* killed is checked by sink_disc, but that's only once per strip, and
+         * they can be huge. Check per output tile as well.
+         */
+	if( vips_image_iskilled( save->in ) )
+		return( -1 );
+
 	/* If we are centering we may be outside the real pixels. Skip in 
 	 * this case, and the viewer will display blank.png for us. 
 	 */
@@ -1780,6 +1929,16 @@ strip_work( VipsThreadState *state, void *a )
 
 	g_mutex_unlock( vips__global_lock );
 
+	/* vips_gsf_path() can return NULL when it exceeds the path limits.
+	 */
+	if( out == NULL ) {
+		g_object_unref( x );
+
+		return( -1 );
+	}
+
+        vips_image_set_int( x, VIPS_META_CONCURRENCY, 1 );
+
 	if( write_image( dz, out, x, dz->suffix ) ) {
 		g_object_unref( out );
 		g_object_unref( x );
@@ -2379,7 +2538,6 @@ vips_foreign_save_dz_build( VipsObject *object )
 	case VIPS_FOREIGN_DZ_CONTAINER_FS:
 {
 		GsfOutput *out;
-		GError *error = NULL;
 		char name[VIPS_PATH_MAX];
 
 		/* For filesystem output of deepzoom, we write 
@@ -2394,8 +2552,7 @@ vips_foreign_save_dz_build( VipsObject *object )
 				"%s/%s", dz->dirname, dz->basename ); 
 
 		if( !(out = (GsfOutput *) 
-			gsf_outfile_stdio_new( name, &error )) ) {
-			vips_g_error( &error );
+			gsf_output_dir_new( name, NULL )) ) {
 			return( -1 );
 		}
 	
@@ -2524,9 +2681,9 @@ vips_foreign_save_dz_build( VipsObject *object )
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static int bandfmt_dz[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, C,  US, S,  UI, I,  F,  F,  D,  D
+static VipsBandFormat bandfmt_dz[10] = {
+	/* Band format:  UC  C  US  S  UI  I  F  X  D  DX */
+	/* Promotion: */ UC, C, US, S, UI, I, F, F, D, D
 };
 
 static const char *dz_suffs[] = { ".dz", ".szi", NULL };
