@@ -167,6 +167,10 @@ vips_exif_load_data_without_fix( const void *data, size_t length )
 	/* exif_data_load_data() only allows uint for length. Limit it to less
 	 * than that: 2**20 should be enough for anyone.
 	 */
+	if( length < 4 ) {
+		vips_error( "exif", "%s", _( "exif too small" ) );
+		return( NULL );
+	}
 	if( length > 1 << 20 ) {
 		vips_error( "exif", "%s", _( "exif too large" ) ); 
 		return( NULL );
@@ -178,7 +182,17 @@ vips_exif_load_data_without_fix( const void *data, size_t length )
 	}
 
 	exif_data_unset_option( ed, EXIF_DATA_OPTION_FOLLOW_SPECIFICATION );
-	exif_data_load_data( ed, data, length );
+	if( !vips_isprefix( "Exif", (char *) data ) ) {
+		/* Ensure "Exif" prefix as loaders may not provide it.
+		 */
+		void* data_with_prefix;
+		data_with_prefix = g_malloc0( length + 6 );
+		memcpy( data_with_prefix, "Exif\0\0", 6 );
+		memcpy( data_with_prefix + 6, data, length );
+		exif_data_load_data( ed, data_with_prefix, length + 6 );
+		g_free( data_with_prefix );
+	} else
+		exif_data_load_data( ed, data, length );
 
 	return( ed );
 }
@@ -788,6 +802,20 @@ tag_is_ascii( ExifTag tag )
 		|| tag == EXIF_TAG_OFFSET_TIME
 		|| tag == EXIF_TAG_OFFSET_TIME_ORIGINAL
 		|| tag == EXIF_TAG_OFFSET_TIME_DIGITIZED
+		|| tag == EXIF_TAG_GPS_LATITUDE_REF
+		|| tag == EXIF_TAG_GPS_LONGITUDE_REF
+		|| tag == EXIF_TAG_GPS_SATELLITES
+		|| tag == EXIF_TAG_GPS_STATUS
+		|| tag == EXIF_TAG_GPS_MEASURE_MODE
+		|| tag == EXIF_TAG_GPS_SPEED_REF
+		|| tag == EXIF_TAG_GPS_TRACK_REF
+		|| tag == EXIF_TAG_GPS_IMG_DIRECTION_REF
+		|| tag == EXIF_TAG_GPS_MAP_DATUM
+		|| tag == EXIF_TAG_GPS_DEST_LATITUDE_REF
+		|| tag == EXIF_TAG_GPS_DEST_LONGITUDE_REF
+		|| tag == EXIF_TAG_GPS_DEST_BEARING_REF
+		|| tag == EXIF_TAG_GPS_DEST_DISTANCE_REF
+		|| tag == EXIF_TAG_GPS_DATE_STAMP
 #endif
 		);
 }
@@ -1391,7 +1419,7 @@ vips__exif_update( VipsImage *image )
 			(void *) &data, &length ) )
 			return( -1 );
 
-		if( !(ed = exif_data_new_from_data( data, length )) )
+		if( !(ed = vips_exif_load_data_without_fix( data, length )) )
 			return( -1 );
 	}
 	else  {
@@ -1401,12 +1429,13 @@ vips__exif_update( VipsImage *image )
 			EXIF_DATA_OPTION_FOLLOW_SPECIFICATION );
 		exif_data_set_data_type( ed, EXIF_DATA_TYPE_COMPRESSED );
 		exif_data_set_byte_order( ed, EXIF_BYTE_ORDER_INTEL );
-	
-		/* Create the mandatory EXIF fields with default data.
-		 */
-		exif_data_fix( ed );
 	}
 
+	/* Make sure all required fields are there before we attach the vips
+	 * metadata.
+	 */
+	exif_data_fix( ed );
+
 	/* Update EXIF tags from the image metadata.
 	 */
 	vips_exif_update( ed, image );
