@@ -38,7 +38,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -173,6 +173,7 @@ vips_foreign_load_fits_class_init( VipsForeignLoadFitsClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsOperationClass *operation_class = VIPS_OPERATION_CLASS( class );
 	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
 	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
 
@@ -184,6 +185,11 @@ vips_foreign_load_fits_class_init( VipsForeignLoadFitsClass *class )
 	object_class->description = _( "FITS loader base class" );
 	object_class->build = vips_foreign_load_fits_build;
 
+	/* cfitsio has not been fuzzed, so should not be used with
+	 * untrusted input unless you are very careful.
+	 */
+	operation_class->flags |= VIPS_OPERATION_UNTRUSTED;
+
 	/* is_a() is not that quick ... lower the priority.
 	 */
 	foreign_class->priority = -50;
@@ -326,7 +332,7 @@ vips_foreign_load_fits_source_class_init(
 	object_class->description = _( "load FITS from a source" );
 	object_class->build = vips_foreign_load_fits_source_build;
 
-	operation_class->flags = VIPS_OPERATION_NOCACHE;
+	operation_class->flags |= VIPS_OPERATION_NOCACHE;
 
 	load_class->is_a_source = 
 		vips_foreign_load_fits_source_is_a_source;
