@@ -936,9 +936,9 @@ vips_foreign_load_iscompat( VipsImage *a, VipsImage *b )
 		a->Ysize != b->Ysize ||
 		a->Bands != b->Bands ||
 		a->Coding != b->Coding ||
-		a->BandFmt != b->BandFmt ) {
-		vips_error( "VipsForeignLoad",
-			"%s", _( "images do not match" ) ); 
+		a->BandFmt != b->BandFmt ) { 
+		vips_error( "VipsForeignLoad", "%s", 
+                        _( "images do not match between header and load" ) ); 
 		return( FALSE );
 	}
 
@@ -1044,7 +1044,6 @@ vips_foreign_load_build( VipsObject *object )
 	VipsForeignLoadClass *fclass = VIPS_FOREIGN_LOAD_GET_CLASS( object );
 
 	VipsForeignFlags flags;
-	gboolean sequential;
 
 #ifdef DEBUG
 	printf( "vips_foreign_load_build:\n" );
@@ -1064,14 +1063,12 @@ vips_foreign_load_build( VipsObject *object )
 
 	g_object_set( load, "flags", flags, NULL );
 
-	/* Seq access has been requested, or the loader requires seq.
+	/* We must block caching of seq loaders running in seq mode. A seq
+	 * loader in random mode is fine, since we'll read to ram or a temp
+	 * file.
 	 */
-	sequential = (load->flags & VIPS_FOREIGN_SEQUENTIAL) || 
-		load->access != VIPS_ACCESS_RANDOM;
-
-	/* We must block caching of seq loads.
-	 */
-	if( sequential )
+	if( (load->flags & VIPS_FOREIGN_SEQUENTIAL) && 
+		load->access != VIPS_ACCESS_RANDOM )
 		load->nocache = TRUE;
 
 	/* The deprecated "fail" field sets fail_on warning.
@@ -1134,9 +1131,9 @@ vips_foreign_load_build( VipsObject *object )
 			return( -1 );
 	}
 
-	/* Tell downstream if we are reading sequentially.
+	/* Tell downstream if seq mode was requested.
 	 */
-	if( sequential )
+	if( load->access != VIPS_ACCESS_RANDOM )
 		vips_image_set_int( load->out, VIPS_META_SEQUENTIAL, 1 );
 
 	return( 0 );
@@ -1751,9 +1748,9 @@ vips_foreign_save_build( VipsObject *object )
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static int vips_foreign_save_format_table[10] = {
-// UC  C   US  S   UI  I  F  X  D  DX 
-   UC, C,  US, S,  UI, I, F, X, D, DX
+static VipsBandFormat vips_foreign_save_format_table[10] = {
+	/* Band format:  UC  C  US  S  UI  I  F  X  D  DX */
+	/* Promotion: */ UC, C, US, S, UI, I, F, X, D, DX
 };
 
 static void
@@ -2253,6 +2250,7 @@ vips_heifload_source( VipsSource *source, VipsImage **out, ... )
  * * @compression: #VipsForeignHeifCompression, write with this compression
  * * @effort: %gint, encoding effort
  * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
+ * * @encoder: #VipsForeignHeifEncoder, select encoder to use
  *
  * Write a VIPS image to a file in HEIF format. 
  *
@@ -2261,7 +2259,7 @@ vips_heifload_source( VipsSource *source, VipsImage **out, ... )
  *
  * Set @lossless %TRUE to switch to lossless compression.
  *
- * Use @compression to set the encoder e.g. HEVC, AVC, AV1. It defaults to AV1
+ * Use @compression to set the compression format e.g. HEVC, AVC, AV1 to use. It defaults to AV1
  * if the target filename ends with ".avif", otherwise HEVC.
  *
  * Use @effort to control the CPU effort spent improving compression.
@@ -2274,6 +2272,8 @@ vips_heifload_source( VipsSource *source, VipsImage **out, ... )
  * Use @bitdepth to set the bitdepth of the output file. HEIC supports at
  * least 8, 10 and 12 bits; other codecs may support more or fewer options.
  *
+ * Use @encoder to set the encode library to use, e.g. aom, SVT-AV1, rav1e etc.
+ *
  * See also: vips_image_write_to_file(), vips_heifload().
  *
  * Returns: 0 on success, -1 on error.
@@ -2306,6 +2306,7 @@ vips_heifsave( VipsImage *in, const char *filename, ... )
  * * @compression: #VipsForeignHeifCompression, write with this compression
  * * @effort: %gint, encoding effort
  * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
+ * * @encoder: #VipsForeignHeifEncoder, select encoder to use
  *
  * As vips_heifsave(), but save to a memory buffer. 
  *
@@ -2359,6 +2360,7 @@ vips_heifsave_buffer( VipsImage *in, void **buf, size_t *len, ... )
  * * @compression: #VipsForeignHeifCompression, write with this compression
  * * @effort: %gint, encoding effort
  * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
+ * * @encoder: #VipsForeignHeifEncoder, select encoder to use
  *
  * As vips_heifsave(), but save to a target.
  *
@@ -2911,6 +2913,7 @@ vips_foreign_operation_init( void )
 	extern GType vips_foreign_save_webp_file_get_type( void ); 
 	extern GType vips_foreign_save_webp_buffer_get_type( void ); 
 	extern GType vips_foreign_save_webp_target_get_type( void ); 
+	extern GType vips_foreign_save_webp_mime_get_type( void );
 
 	extern GType vips_foreign_load_pdf_file_get_type( void ); 
 	extern GType vips_foreign_load_pdf_buffer_get_type( void ); 
@@ -3090,6 +3093,7 @@ vips_foreign_operation_init( void )
 	vips_foreign_save_webp_file_get_type(); 
 	vips_foreign_save_webp_buffer_get_type(); 
 	vips_foreign_save_webp_target_get_type(); 
+	vips_foreign_save_webp_mime_get_type();
 #endif /*HAVE_LIBWEBP*/
 
 #ifdef HAVE_TIFF
