@@ -306,18 +306,18 @@ static int
 vips_foreign_load_heif_is_a( const char *buf, int len )
 {
 	if( len >= 12 ) {
-		const guint32 chunk_len = 
-			(guint32) buf[0] << 24 |
-			(guint32) buf[1] << 16 |
-			(guint32) buf[2] << 8 |
-			(guint32) buf[3];
+                unsigned char *p = (unsigned char *) buf;
+		guint32 chunk_len = 
+			VIPS_LSHIFT_INT( p[0], 24 ) |
+			VIPS_LSHIFT_INT( p[1], 16 ) |
+			VIPS_LSHIFT_INT( p[2], 8 ) |
+			VIPS_LSHIFT_INT( p[3], 0 );
 
 		int i;
 
-                /* We've seen real files with 36 here, so 64 should be
-                 * plenty.
+                /* chunk_len can be pretty big for eg. animated AVIF.
                  */
-		if( chunk_len > 64 || 
+		if( chunk_len > 2048 || 
 			chunk_len % 4 != 0 )
 			return( 0 );
 
@@ -1027,7 +1027,7 @@ vips_foreign_load_heif_load( VipsForeignLoad *load )
 	if( vips_foreign_load_heif_set_header( heif, t[0] ) )
 		return( -1 );
 
-	/* CLose input immediately at end of read.
+	/* Close input immediately at end of read.
 	 */
 	g_signal_connect( t[0], "minimise", 
 		G_CALLBACK( vips_foreign_load_heif_minimise ), heif ); 
@@ -1065,14 +1065,14 @@ vips_foreign_load_heif_class_init( VipsForeignLoadHeifClass *class )
 
 	VIPS_ARG_INT( class, "page", 2,
 		_( "Page" ),
-		_( "Load this page from the file" ),
+		_( "First page to load" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadHeif, page ),
 		0, 100000, 0 );
 
 	VIPS_ARG_INT( class, "n", 3,
 		_( "n" ),
-		_( "Load this many pages" ),
+		_( "Number of pages to load, -1 for all" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadHeif, n ),
 		-1, 100000, 1 );
