@@ -14,6 +14,8 @@
  * 	- rename "speed" as "effort" for consistency with other savers
  * 22/12/21
  * 	- add >8 bit support
+ * 22/10/11
+ *      - improve rules for 16-bit write [johntrunc]
  */
 
 /*
@@ -48,21 +50,6 @@
 #define DEBUG
  */
 
-/*
- *
-
-TODO:
-
-	what about a 16-bit PNG saved with bitdepth=8? does this work?
-
-		no!
-
-	what about a 8-bit PNG saved with bitdepth=12? does this work?
-
- * 
- */
-
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
@@ -112,6 +99,10 @@ typedef struct _VipsForeignSaveHeif {
 	 */
 	VipsForeignSubsample subsample_mode;
 
+	/* Encoder to use. For instance: aom, svt etc.
+	 */
+	VipsForeignHeifEncoder selected_encoder;
+
 	/* The image we save. This is a copy of save->ready since we need to
 	 * be able to update the metadata.
 	 */
@@ -148,10 +139,10 @@ typedef struct _VipsForeignSaveHeif {
 
 typedef VipsForeignSaveClass VipsForeignSaveHeifClass;
 
-/* Defined in heif2vips.c
+/* Defined in heifload.c
  */
-void vips__heif_error( struct heif_error *error );
 void vips__heif_image_print( struct heif_image *img );
+void vips__heif_error( struct heif_error *error );
 
 G_DEFINE_ABSTRACT_TYPE( VipsForeignSaveHeif, vips_foreign_save_heif, 
 	VIPS_TYPE_FOREIGN_SAVE );
@@ -179,7 +170,7 @@ typedef struct heif_error (*libheif_metadata_fn)( struct heif_context *,
 /* String-based metadata fields we add.
  */
 typedef struct _VipsForeignSaveHeifMetadata {
-	const char *name;		/* as understood by libvips */
+	const char *name;		        /* as understood by libvips */
 	libheif_metadata_fn saver;		/* as understood by libheif */
 } VipsForeignSaveHeifMetadata;
 
@@ -263,11 +254,22 @@ vips_foreign_save_heif_write_page( VipsForeignSaveHeif *heif, int page )
 		options->save_alpha_channel = 1;
 
 #ifdef DEBUG
-	printf( "encoding ..\n" ); 
+{
+        GTimer *timer = g_timer_new();
+
+	printf( "calling heif_context_encode_image() ...\n" ); 
 #endif /*DEBUG*/
+
 	error = heif_context_encode_image( heif->ctx, 
 		heif->img, heif->encoder, options, &heif->handle );
 
+#ifdef DEBUG
+	printf( "... libheif took %.2g seconds\n", 
+                g_timer_elapsed( timer, NULL ) ); 
+        g_timer_destroy( timer );
+}
+#endif /*DEBUG*/
+
 	heif_encoding_options_free( options );
 
 	if( error.code ) {
@@ -309,7 +311,7 @@ vips_foreign_save_heif_pack( VipsForeignSaveHeif *heif,
 
 	if( heif->image->BandFmt == VIPS_FORMAT_UCHAR &&
 		heif->bitdepth == 8 )
-		/* Most common cases -- 8 bit to 8 bit.
+		/* Most common case -- 8 bit to 8 bit.
 		 */
 		memcpy( q, p, ne );
 	else if( heif->image->BandFmt == VIPS_FORMAT_UCHAR &&
@@ -330,8 +332,14 @@ vips_foreign_save_heif_pack( VipsForeignSaveHeif *heif,
 	else if( heif->image->BandFmt == VIPS_FORMAT_USHORT &&
                 heif->bitdepth <= 8 ) {
 		/* 16-bit native byte order source, 8 bit write.
-		 */
-		int shift = 16 - heif->bitdepth;
+                 *
+                 * Pick the high or low bits of the source.
+                 */
+                int vips_bitdepth = 
+                        heif->image->Type == VIPS_INTERPRETATION_RGB16 ||
+                        heif->image->Type == VIPS_INTERPRETATION_GREY16 ? 
+                                16 : 8;
+		int shift = vips_bitdepth - heif->bitdepth;
 
 		for( i = 0; i < ne; i++ ) {
 			guint16 v = *((gushort *) p) >> shift;
@@ -345,7 +353,11 @@ vips_foreign_save_heif_pack( VipsForeignSaveHeif *heif,
                 heif->bitdepth > 8 ) {
 		/* 16-bit native byte order source, 16 bit bigendian write.
 		 */
-		int shift = 16 - heif->bitdepth;
+                int vips_bitdepth = 
+                        heif->image->Type == VIPS_INTERPRETATION_RGB16 ||
+                        heif->image->Type == VIPS_INTERPRETATION_GREY16 ? 
+                                16 : 8;
+		int shift = vips_bitdepth - heif->bitdepth;
 
 		for( i = 0; i < ne; i++ ) {
 			guint16 v = *((gushort *) p) >> shift;
@@ -430,11 +442,18 @@ vips_foreign_save_heif_build( VipsObject *object )
 	struct heif_error error;
 	struct heif_writer writer;
 	char *chroma;
+	const struct heif_encoder_descriptor *out_encoder;
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_heif_parent_class )->
 		build( object ) )
 		return( -1 );
 
+	/* Make a copy of the image in case we modify the metadata eg. for
+	 * exif_update.
+	 */
+	if( vips_copy( save->ready, &heif->image, NULL ) ) 
+		return( -1 );
+
 	/* If the old, deprecated "speed" param is being used and the new
 	 * "effort" param is not, use speed to init effort.
 	 */
@@ -442,22 +461,41 @@ vips_foreign_save_heif_build( VipsObject *object )
 		!vips_object_argument_isset( object, "effort" ) )
 		heif->effort = 9 - heif->speed;
 
-	/* Default 12 bit save for ushort. HEIC (for example) implements 
+	/* Default 12 bit save for 16-bit images. HEIC (for example) implements 
 	 * 8 / 10 / 12.
 	 */
 	if( !vips_object_argument_isset( object, "bitdepth" ) ) 
-		heif->bitdepth = save->ready->BandFmt == VIPS_FORMAT_UCHAR ?
-			8 : 12;
+		heif->bitdepth = 
+                        heif->image->Type == VIPS_INTERPRETATION_RGB16 ||
+                        heif->image->Type == VIPS_INTERPRETATION_GREY16 ? 
+                                12 : 8;
 
-	/* Make a copy of the image in case we modify the metadata eg. for
-	 * exif_update.
+	/* Try to find the selected encoder.
 	 */
-	if( vips_copy( save->ready, &heif->image, NULL ) ) 
-		return( -1 );
+	if( heif->selected_encoder != VIPS_FOREIGN_HEIF_ENCODER_AUTO ) {
+		const int count = heif_context_get_encoder_descriptors(
+			heif->ctx,
+			(enum heif_compression_format) heif->compression,
+			vips_enum_nick( VIPS_TYPE_FOREIGN_HEIF_ENCODER,
+				heif->selected_encoder ),
+			&out_encoder, 1 );
+
+		if( count > 0 ) 
+			error = heif_context_get_encoder( heif->ctx,
+				out_encoder, &heif->encoder );
+		else 
+			g_warning( "heifsave: could not find %s",
+				vips_enum_nick( VIPS_TYPE_FOREIGN_HEIF_ENCODER,
+					heif->selected_encoder ) );
+	}
+
+	/* Fallback to default encoder.
+	 */
+	if( !heif->encoder )
+		error = heif_context_get_encoder_for_format( heif->ctx,
+			(enum heif_compression_format) heif->compression,
+			&heif->encoder );
 
-	error = heif_context_get_encoder_for_format( heif->ctx, 
-		(enum heif_compression_format) heif->compression, 
-		&heif->encoder );
 	if( error.code ) {
 		if( error.code == heif_error_Unsupported_filetype ) 
 			vips_error( "heifsave", 
@@ -567,9 +605,12 @@ vips_foreign_save_heif_build( VipsObject *object )
 #define UC VIPS_FORMAT_UCHAR
 #define US VIPS_FORMAT_USHORT
 
-static int vips_heif_bandfmt[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, UC, US, US, US, US, US, US, US, US
+/* Except for 8-bit inputs, we send everything else to 16. We decide on 8-bit
+ * vs. 12 bit save based on Type in_build(), see above.
+ */
+static VipsBandFormat vips_heif_bandfmt[10] = {
+	/* Band format:  UC  C   US  S   UI  I   F   X   D   DX */
+	/* Promotion: */ UC, UC, US, US, US, US, US, US, US, US
 };
 
 static void
@@ -641,6 +682,13 @@ vips_foreign_save_heif_class_init( VipsForeignSaveHeifClass *class )
 		G_STRUCT_OFFSET( VipsForeignSaveHeif, speed ),
 		0, 9, 5 );
 
+	VIPS_ARG_ENUM( class, "encoder", 18,
+		_( "Encoder" ),
+		_( "Select encoder to use" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveHeif, selected_encoder ),
+		VIPS_TYPE_FOREIGN_HEIF_ENCODER,
+		VIPS_FOREIGN_HEIF_ENCODER_AUTO );
 }
 
 static void
