@@ -2,6 +2,8 @@
  *
  * 24/11/11
  * 	- wrap a class around the jpeg writer
+ * 18/2/20 Elad-Laufer
+ * 	- add subsample_mode, deprecate no_subsample
  */
 
 /*
@@ -39,7 +41,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -73,10 +75,17 @@ typedef struct _VipsForeignSaveJpeg {
 	 */
 	gboolean interlace;
 
-	/* Disable chroma subsampling. 
+	/* Deprecated: Disable chroma subsampling. Use subsample_mode instead.
 	 */
 	gboolean no_subsample;
 
+	/* Select chroma subsampling mode:
+	 * auto will disable subsampling for Q >= 90
+	 * on will always enable subsampling
+	 * off will always disable subsampling
+	 */
+	VipsForeignSubsample subsample_mode;
+
 	/* Apply trellis quantisation to each 8x8 block.
 	 */
 	gboolean trellis_quant;
@@ -94,6 +103,10 @@ typedef struct _VipsForeignSaveJpeg {
 	 */
 	int quant_table;
 
+	/* Use an MCU restart interval.
+	 */
+	int restart_interval;
+
 } VipsForeignSaveJpeg;
 
 typedef VipsForeignSaveClass VipsForeignSaveJpegClass;
@@ -105,11 +118,31 @@ G_DEFINE_ABSTRACT_TYPE( VipsForeignSaveJpeg, vips_foreign_save_jpeg,
 
 /* Type promotion for save ... just always go to uchar.
  */
-static int bandfmt_jpeg[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, UC, UC, UC, UC, UC, UC, UC, UC, UC
+static VipsBandFormat bandfmt_jpeg[10] = {
+	/* Band format:  UC  C   US  S   UI  I   F   X   D   DX */
+	/* Promotion: */ UC, UC, UC, UC, UC, UC, UC, UC, UC, UC
 };
 
+static int
+vips_foreign_save_jpeg_build( VipsObject *object )
+{
+	VipsForeignSaveJpeg *jpeg = (VipsForeignSaveJpeg *) object;
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_save_jpeg_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	/* no_subsample is deprecated, but we retain backwards compatibility
+	 * new code should use subsample_mode
+	 */
+	if( vips_object_argument_isset( object, "no_subsample" ) )
+		jpeg->subsample_mode = jpeg->no_subsample ? 
+			VIPS_FOREIGN_SUBSAMPLE_OFF :
+			VIPS_FOREIGN_SUBSAMPLE_AUTO;
+
+	return( 0 );
+}
+
 static void
 vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 {
@@ -123,6 +156,7 @@ vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 
 	object_class->nickname = "jpegsave_base";
 	object_class->description = _( "save jpeg" );
+	object_class->build = vips_foreign_save_jpeg_build;
 
 	foreign_class->suffs = vips__jpeg_suffs;
 
@@ -162,7 +196,7 @@ vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 	VIPS_ARG_BOOL( class, "no_subsample", 14,
 		_( "No subsample" ),
 		_( "Disable chroma subsample" ),
-		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignSaveJpeg, no_subsample ),
 		FALSE );
 
@@ -194,12 +228,28 @@ vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 		G_STRUCT_OFFSET( VipsForeignSaveJpeg, quant_table ),
 		0, 8, 0 );
 
+	VIPS_ARG_ENUM( class, "subsample_mode", 19,
+		_( "Subsample mode" ),
+		_( "Select chroma subsample operation mode" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveJpeg, subsample_mode ),
+		VIPS_TYPE_FOREIGN_SUBSAMPLE,
+		VIPS_FOREIGN_SUBSAMPLE_AUTO );
+
+	VIPS_ARG_INT( class, "restart_interval", 20,
+		_( "Restart interval" ),
+		_( "Add restart markers every specified number of mcu" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveJpeg, restart_interval ),
+		0, INT_MAX, 0 );
+
 }
 
 static void
 vips_foreign_save_jpeg_init( VipsForeignSaveJpeg *jpeg )
 {
 	jpeg->Q = 75;
+	jpeg->subsample_mode = VIPS_FOREIGN_SUBSAMPLE_AUTO;
 }
 
 typedef struct _VipsForeignSaveJpegTarget {
@@ -228,9 +278,10 @@ vips_foreign_save_jpeg_target_build( VipsObject *object )
 
 	if( vips__jpeg_write_target( save->ready, target->target,
 		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
-		jpeg->interlace, save->strip, jpeg->no_subsample,
-		jpeg->trellis_quant, jpeg->overshoot_deringing,
-		jpeg->optimize_scans, jpeg->quant_table ) )
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode,
+		jpeg->restart_interval ) )
 		return( -1 );
 
 	return( 0 );
@@ -256,7 +307,6 @@ vips_foreign_save_jpeg_target_class_init(
 		VIPS_ARGUMENT_REQUIRED_INPUT, 
 		G_STRUCT_OFFSET( VipsForeignSaveJpegTarget, target ),
 		VIPS_TYPE_TARGET );
-
 }
 
 static void
@@ -295,9 +345,10 @@ vips_foreign_save_jpeg_file_build( VipsObject *object )
 		return( -1 );
 	if( vips__jpeg_write_target( save->ready, target,
 		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
-		jpeg->interlace, save->strip, jpeg->no_subsample,
-		jpeg->trellis_quant, jpeg->overshoot_deringing,
-		jpeg->optimize_scans, jpeg->quant_table ) ) {
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode,
+		jpeg->restart_interval ) ) {
 		VIPS_UNREF( target );
 		return( -1 );
 	}
@@ -351,7 +402,8 @@ vips_foreign_save_jpeg_buffer_build( VipsObject *object )
 {
 	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSaveJpeg *jpeg = (VipsForeignSaveJpeg *) object;
-	VipsForeignSaveJpegBuffer *file = (VipsForeignSaveJpegBuffer *) object;
+	VipsForeignSaveJpegBuffer *buffer = 
+		(VipsForeignSaveJpegBuffer *) object;
 
 	VipsTarget *target;
 	VipsBlob *blob;
@@ -365,15 +417,16 @@ vips_foreign_save_jpeg_buffer_build( VipsObject *object )
 
 	if( vips__jpeg_write_target( save->ready, target,
 		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
-		jpeg->interlace, save->strip, jpeg->no_subsample,
-		jpeg->trellis_quant, jpeg->overshoot_deringing,
-		jpeg->optimize_scans, jpeg->quant_table ) ) {
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode,
+		jpeg->restart_interval ) ) {
 		VIPS_UNREF( target );
 		return( -1 );
 	}
 
 	g_object_get( target, "blob", &blob, NULL );
-	g_object_set( file, "buffer", blob, NULL );
+	g_object_set( buffer, "buffer", blob, NULL );
 	vips_area_unref( VIPS_AREA( blob ) );
 
 	VIPS_UNREF( target );
@@ -438,9 +491,10 @@ vips_foreign_save_jpeg_mime_build( VipsObject *object )
 
 	if( vips__jpeg_write_target( save->ready, target,
 		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
-		jpeg->interlace, save->strip, jpeg->no_subsample,
-		jpeg->trellis_quant, jpeg->overshoot_deringing,
-		jpeg->optimize_scans, jpeg->quant_table ) ) {
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode,
+		jpeg->restart_interval ) ) {
 		VIPS_UNREF( target );
 		return( -1 );
 	}
@@ -488,15 +542,16 @@ vips_foreign_save_jpeg_mime_init( VipsForeignSaveJpegMime *mime )
  * Optional arguments:
  *
  * * @Q: %gint, quality factor
- * * @profile: filename of ICC profile to attach
+ * * @profile: %gchararray, filename of ICC profile to attach
  * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
  * * @interlace: %gboolean, write an interlaced (progressive) jpeg
  * * @strip: %gboolean, remove all metadata from image
- * * @no_subsample: %gboolean, disable chroma subsampling
+ * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
  * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
  * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
  * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
  * * @quant_table: %gint, quantization table index
+ * * @restart_interval: %gint, restart interval in mcu
  *
  * Write a VIPS image to a file as JPEG.
  *
@@ -518,10 +573,10 @@ vips_foreign_save_jpeg_mime_init( VipsForeignSaveJpegMime *mime )
  * conection, but need much more memory to encode and decode. 
  *
  * If @strip is set, no EXIF data, IPTC data, ICC profile or XMP metadata is 
- * written into the output file. 
+ * written into the output file.
  *
- * If @no_subsample is set, chrominance subsampling is disabled. This will 
- * improve quality at the cost of larger file size. Useful for high Q factors. 
+ * Chroma subsampling is normally automatically disabled for Q >= 90. You can
+ * force the subsampling mode with @subsample_mode.
  *
  * If @trellis_quant is set and the version of libjpeg supports it
  * (e.g. mozjpeg >= 3.0), apply trellis quantisation to each 8x8 block.
@@ -567,6 +622,12 @@ vips_foreign_save_jpeg_mime_init( VipsForeignSaveJpegMime *mime )
  * For maximum compression with mozjpeg, a useful set of options is `strip, 
  * optimize-coding, interlace, optimize-scans, trellis-quant, quant_table=3`.
  *
+ * By default, the output stream won't have restart markers.  If a non-zero
+ * restart_interval is specified, a restart marker will be added after each
+ * specified number of MCU blocks.  This makes the stream more recoverable
+ * if there are transmission errors, but also allows for some decoders to read
+ * part of the JPEG without decoding the whole stream.
+ *
  * The image is automatically converted to RGB, Monochrome or CMYK before 
  * saving. 
  *
@@ -604,15 +665,16 @@ vips_jpegsave( VipsImage *in, const char *filename, ... )
  * Optional arguments:
  *
  * * @Q: %gint, quality factor
- * * @profile: filename of ICC profile to attach
+ * * @profile: %gchararray, filename of ICC profile to attach
  * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
  * * @interlace: %gboolean, write an interlaced (progressive) jpeg
  * * @strip: %gboolean, remove all metadata from image
- * * @no_subsample: %gboolean, disable chroma subsampling
+ * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
  * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
  * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
  * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
  * * @quant_table: %gint, quantization table index
+ * * @restart_interval: %gint, restart interval in mcu
  *
  * As vips_jpegsave(), but save to a target.
  *
@@ -643,15 +705,16 @@ vips_jpegsave_target( VipsImage *in, VipsTarget *target, ... )
  * Optional arguments:
  *
  * * @Q: %gint, quality factor
- * * @profile: filename of ICC profile to attach
+ * * @profile: %gchararray, filename of ICC profile to attach
  * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
  * * @interlace: %gboolean, write an interlaced (progressive) jpeg
  * * @strip: %gboolean, remove all metadata from image
- * * @no_subsample: %gboolean, disable chroma subsampling
+ * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
  * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
  * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
  * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
  * * @quant_table: %gint, quantization table index
+ * * @restart_interval: %gint, restart interval in mcu
  *
  * As vips_jpegsave(), but save to a memory buffer. 
  *
@@ -699,15 +762,16 @@ vips_jpegsave_buffer( VipsImage *in, void **buf, size_t *len, ... )
  * Optional arguments:
  *
  * * @Q: %gint, quality factor
- * * @profile: filename of ICC profile to attach
+ * * @profile: %gchararray, filename of ICC profile to attach
  * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
  * * @interlace: %gboolean, write an interlaced (progressive) jpeg
  * * @strip: %gboolean, remove all metadata from image
- * * @no_subsample: %gboolean, disable chroma subsampling
+ * * @subsample_mode: #VipsForeignSubsample, chroma subsampling mode
  * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
  * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
  * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
  * * @quant_table: %gint, quantization table index
+ * * @restart_interval: %gint, restart interval in mcu
  *
  * As vips_jpegsave(), but save as a mime jpeg on stdout.
  *
