@@ -38,7 +38,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -82,6 +82,14 @@ magick_sniff( const unsigned char *bytes, size_t length )
 		bytes[3] == 0 )
 		return( "ICO" );
 
+	if( length >= 5 &&
+		bytes[0] == 0 && 
+		bytes[1] == 1 &&
+		bytes[2] == 0 &&
+		bytes[3] == 0 &&
+		bytes[4] == 0 )
+		return( "TTF" );
+
 	if( length >= 18 &&
 		(bytes[1] == 0 || 
 		 bytes[1] == 1) &&
@@ -233,7 +241,7 @@ magick_set_number_scenes( ImageInfo *image_info, int scene, int number_scenes )
 	/* Some IMs must have the string version set as well.
 	 */
 	vips_snprintf( page, 256, "%d-%d", scene, scene + number_scenes );
-	image_info->scenes = strdup( page );
+	image_info->scenes = g_strdup( page );
 }
 
 int
@@ -278,6 +286,19 @@ magick_ismagick( const unsigned char *bytes, size_t length )
 		GetImageMagick( bytes, length, format ) );
 }
 
+int
+magick_quantize_images( Image *images,
+	const size_t depth, ExceptionInfo *exception )
+{
+	QuantizeInfo info;
+
+	GetQuantizeInfo( &info );
+	info.number_colors = 1 << depth;
+	QuantizeImages( &info, images, exception );
+
+	return( exception->severity == UndefinedException );
+}
+
 #endif /*HAVE_MAGICK7*/
 
 #ifdef HAVE_MAGICK6
@@ -538,7 +559,7 @@ magick_set_number_scenes( ImageInfo *image_info, int scene, int number_scenes )
 	/* Some IMs must have the string version set as well.
 	 */
 	vips_snprintf( page, 256, "%d-%d", scene, scene + number_scenes );
-	image_info->scenes = strdup( page );
+	image_info->scenes = g_strdup( page );
 #else /*!HAVE_NUMBER_SCENES*/
 	/* This works with GM 1.2.31 and probably others.
 	 */
@@ -608,6 +629,17 @@ magick_ismagick( const unsigned char *bytes, size_t length )
 #endif
 }
 
+int
+magick_quantize_images( Image *images,
+	const size_t depth, ExceptionInfo *exception )
+{
+	QuantizeInfo info;
+
+	GetQuantizeInfo( &info );
+	info.number_colors = (1 << depth);
+	return QuantizeImages( &info, images );
+}
+
 #endif /*HAVE_MAGICK6*/
 
 #if defined(HAVE_MAGICK6) || defined(HAVE_MAGICK7)
@@ -750,13 +782,13 @@ magick_set_vips_profile_cb( Image *image,
 	char name_text[256];
 	VipsBuf vips_name = VIPS_BUF_STATIC( name_text );
 
-	if( strcasecmp( name, "XMP" ) == 0 )
+	if( g_ascii_strcasecmp( name, "XMP" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_XMP_NAME );
-	else if( strcasecmp( name, "IPTC" ) == 0 )
+	else if( g_ascii_strcasecmp( name, "IPTC" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_IPTC_NAME );
-	else if( strcasecmp( name, "ICC" ) == 0 )
+	else if( g_ascii_strcasecmp( name, "ICC" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_ICC_NAME );
-	else if( strcasecmp( name, "EXIF" ) == 0 )
+	else if( g_ascii_strcasecmp( name, "EXIF" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_EXIF_NAME );
 	else
 		vips_buf_appendf( &vips_name, "magickprofile-%s", name );
