@@ -66,6 +66,8 @@
  * 	- add profile (xmp, ipct, etc.) read
  * 12/11/21
  * 	- set "orientation"
+ * 26/8/22
+ *      - set "magick-format"
  */
 
 /*
@@ -485,6 +487,12 @@ parse_header( Read *read )
 }
 #endif 
 
+        /* Something like "BMP".
+         */
+        if( strlen( read->image->magick ) > 0 )
+                vips_image_set_string( im, "magick-format", 
+                        read->image->magick );
+
 	/* Do we have a set of equal-sized frames? Append them.
 
 	   	FIXME ... there must be an attribute somewhere from dicom read 
@@ -745,9 +753,11 @@ magick_fill_region( VipsRegion *out,
 
 		PixelPacket *pixels;
 
-		g_mutex_lock( read->lock );
+		vips__worker_lock( read->lock );
+
 		pixels = get_pixels( read->frames[frame], 
 			r->left, line, r->width, 1 );
+
 		g_mutex_unlock( read->lock );
 
 		if( !pixels ) {
