@@ -40,7 +40,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -422,7 +422,7 @@ static int
 vips_foreign_load_matrix_source_is_a_source( VipsSource *source )
 {
 	unsigned char *data;
-	size_t bytes_read;
+	gint64 bytes_read;
 	char line[80];
 	int width;
 	int height;
@@ -458,7 +458,7 @@ vips_foreign_load_matrix_source_class_init(
 	object_class->nickname = "matrixload_source";
 	object_class->build = vips_foreign_load_matrix_source_build;
 
-	operation_class->flags = VIPS_OPERATION_NOCACHE;
+	operation_class->flags |= VIPS_OPERATION_NOCACHE;
 
 	load_class->is_a_source = vips_foreign_load_matrix_source_is_a_source;
 
