@@ -55,7 +55,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -482,7 +482,10 @@ vips_foreign_load_nifti_set_header( VipsForeignLoadNifti *nifti,
 	printf( "get_vips_properties: fmt = %d\n", fmt );
 #endif /*DEBUG*/
 
-	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_SMALLTILE, NULL );
+        /* We load to memory then write to out, so we'll hint THINSTRIP.
+         */
+	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL );
+
 	vips_image_init_fields( out,
 		width, height, bands, fmt, 
 		VIPS_CODING_NONE, 
@@ -580,6 +583,7 @@ vips_foreign_load_nifti_class_init( VipsForeignLoadNiftiClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsOperationClass *operation_class = VIPS_OPERATION_CLASS( class );
 	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
 	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
 
@@ -591,6 +595,11 @@ vips_foreign_load_nifti_class_init( VipsForeignLoadNiftiClass *class )
 	object_class->description = _( "load a NIFTI image" );
 	object_class->build = vips_foreign_load_nifti_build;
 
+	/* nificlib has not been fuzzed, so should not be used with
+	 * untrusted input unless you are very careful.
+	 */
+	operation_class->flags |= VIPS_OPERATION_UNTRUSTED;
+
 	/* is_a() is not that quick ... lower the priority.
 	 */
 	foreign_class->priority = -50;
@@ -777,7 +786,7 @@ vips_foreign_load_nifti_source_class_init(
 	object_class->description = _( "load NIfTI volumes" );
 	object_class->build = vips_foreign_load_nifti_source_build;
 
-	operation_class->flags = VIPS_OPERATION_NOCACHE;
+	operation_class->flags |= VIPS_OPERATION_NOCACHE;
 
 	load_class->is_a_source = 
 		vips_foreign_load_nifti_source_is_a_source;
