@@ -3,9 +3,11 @@
  * 6/10/18
  * 	- from gifload.c
  * 3/3/22 tlsa
- *	- update libnsgif API
- *9/5/22
- 	- attach GIF palette as metadata
+ * 	- update libnsgif API
+ * 9/5/22
+ * 	- attach GIF palette as metadata
+ * 26/11/22 kleisauke
+ * 	- avoid minimise after mapping -- not reliable on Win32
  */
 
 /*
@@ -160,9 +162,7 @@ vips_foreign_load_nsgif_dispose( GObject *gobject )
 
 	VIPS_DEBUG_MSG( "vips_foreign_load_nsgif_dispose:\n" );
 
-	if( gif->anim ) {
-		nsgif_destroy( gif->anim );
-	}
+	VIPS_FREEF( nsgif_destroy, gif->anim );
 	VIPS_UNREF( gif->source );
 	VIPS_FREE( gif->delay );
 
@@ -336,8 +336,6 @@ vips_foreign_load_nsgif_set_header( VipsForeignLoadNsgif *gif,
  *
  * Don't flag any errors unless we have to: we want to work for corrupt or
  * malformed GIFs.
- *
- * Close as soon as we can to free up the fd.
  */
 static int
 vips_foreign_load_nsgif_header( VipsForeignLoad *load )
@@ -352,25 +350,49 @@ vips_foreign_load_nsgif_header( VipsForeignLoad *load )
 
 	VIPS_DEBUG_MSG( "vips_foreign_load_nsgif_header:\n" );
 
-	/* We map in the image, then minimise to close any underlying file
-	 * object. This won't unmap.
+	/* Map the whole source into memory.
 	 */
-	if( !(data = vips_source_map( gif->source, &size )) ) 
+	if( !(data = vips_source_map( gif->source, &size )) )
 		return( -1 );
-	vips_source_minimise( gif->source );
 
+	/* Treat errors from _scan() as warnings. If libnsgif really can't do
+	 * something it'll fail gracefully later when we try to read out 
+	 * frame data.
+	 */
 	result = nsgif_data_scan( gif->anim, size, (void *) data );
 	VIPS_DEBUG_MSG( "nsgif_data_scan() = %s\n", nsgif_strerror( result ) );
-	gif->info = nsgif_get_info(gif->anim);
+	switch( result ) {
+	case NSGIF_ERR_END_OF_DATA:
+		if( load->fail_on >= VIPS_FAIL_ON_TRUNCATED ) {
+			vips_foreign_load_nsgif_error( gif, result ); 
+			return( -1 );
+		}
+		else
+			g_warning( "%s", nsgif_strerror( result ) );
+		break;
+
+	case NSGIF_OK:
+		break;
+
+	default:
+		if( load->fail_on >= VIPS_FAIL_ON_WARNING ) {
+			vips_foreign_load_nsgif_error( gif, result ); 
+			return( -1 );
+		}
+		else
+			g_warning( "%s", nsgif_strerror( result ) );
+		break;
+	}
+
+	/* Tell libnsgif that that's all the data we have. This will let us
+	 * read out any truncated final frames.
+	 */
+	nsgif_data_complete( gif->anim );
+
+	gif->info = nsgif_get_info( gif->anim );
 #ifdef VERBOSE
 	print_animation( gif->anim, gif->info );
 #endif /*VERBOSE*/
-	if( result != NSGIF_OK &&
-		load->fail_on >= VIPS_FAIL_ON_WARNING ) {
-		vips_foreign_load_nsgif_error( gif, result ); 
-		return( -1 );
-	}
-
 	if( !gif->info->frame_count ) {
 		vips_error( class->nickname, "%s", _( "no frames in GIF" ) );
 		return( -1 );
@@ -487,12 +509,36 @@ vips_foreign_load_nsgif_generate( VipsRegion *or,
 	return( 0 );
 }
 
+int
+vips_foreign_load_nsgif_tile_height( VipsForeignLoadNsgif *gif ) 
+{
+	int height = gif->info->height;
+
+	int i;
+
+	/* First, check the perfect size.
+         */
+	if( height % 16 == 0 )
+		return( 16 );
+
+	/* Next, check larger and smaller sizes.
+         */
+	for( i = 1; i < 16; i++ ) {
+		if( height % (16 + i) == 0 )
+			return( 16 + i );
+		if( height % (16 - i) == 0 )
+			return( 16 - i );
+	}
+
+	return( 1 );
+}
+
 static int
 vips_foreign_load_nsgif_load( VipsForeignLoad *load )
 {
 	VipsForeignLoadNsgif *gif = (VipsForeignLoadNsgif *) load;
 	VipsImage **t = (VipsImage **)
-		vips_object_local_array( VIPS_OBJECT( load ), 4 );
+		vips_object_local_array( VIPS_OBJECT( load ), 2 );
 
 	VIPS_DEBUG_MSG( "vips_foreign_load_nsgif_load:\n" );
 
@@ -507,7 +553,8 @@ vips_foreign_load_nsgif_load( VipsForeignLoad *load )
 	if( vips_image_generate( t[0],
 		NULL, vips_foreign_load_nsgif_generate, NULL, gif, NULL ) ||
 		vips_sequential( t[0], &t[1],
-			"tile_height", VIPS__FATSTRIP_HEIGHT,
+			"tile_height", 
+				vips_foreign_load_nsgif_tile_height( gif ),
 			NULL ) ||
 		vips_image_write( t[1], load->real ) )
 		return( -1 );
@@ -542,14 +589,14 @@ vips_foreign_load_nsgif_class_init( VipsForeignLoadNsgifClass *class )
 
 	VIPS_ARG_INT( class, "page", 10,
 		_( "Page" ),
-		_( "Load this page from the file" ),
+		_( "First page to load" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadNsgif, page ),
 		0, 100000, 0 );
 
 	VIPS_ARG_INT( class, "n", 6,
 		_( "n" ),
-		_( "Load this many pages" ),
+		_( "Number of pages to load, -1 for all" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadNsgif, n ),
 		-1, 100000, 1 );
@@ -603,6 +650,7 @@ vips_foreign_load_nsgif_init( VipsForeignLoadNsgif *gif )
 		&gif->anim );
 	if (result != NSGIF_OK) {
 		VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( gif );
+
 		vips_error( class->nickname, "%s",
 			nsgif_strerror( result ) );
 		return;
