@@ -572,7 +572,7 @@ vips_foreign_load_pdf_generate( VipsRegion *or,
 		if( vips_foreign_load_pdf_get_page( pdf, pdf->page_no + i ) )
 			return( -1 ); 
 
-		g_mutex_lock( vips_pdfium_mutex );
+		vips__worker_lock( vips_pdfium_mutex );
 
 		/* 4 means RGBA.
 		 */
@@ -689,42 +689,42 @@ vips_foreign_load_pdf_class_init( VipsForeignLoadPdfClass *class )
 
 	VIPS_ARG_INT( class, "page", 10,
 		_( "Page" ),
-		_( "Load this page from the file" ),
+		_( "First page to load" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadPdf, page_no ),
 		0, 100000, 0 );
 
 	VIPS_ARG_INT( class, "n", 11,
 		_( "n" ),
-		_( "Load this many pages" ),
+		_( "Number of pages to load, -1 for all" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadPdf, n ),
 		-1, 100000, 1 );
 
 	VIPS_ARG_DOUBLE( class, "dpi", 12,
 		_( "DPI" ),
-		_( "Render at this DPI" ),
+		_( "DPI to render at" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadPdf, dpi ),
 		0.001, 100000.0, 72.0 );
 
 	VIPS_ARG_DOUBLE( class, "scale", 13,
 		_( "Scale" ),
-		_( "Scale output by this factor" ),
+		_( "Factor to scale by" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadPdf, scale ),
 		0.001, 100000.0, 1.0 );
 
 	VIPS_ARG_BOXED( class, "background", 14, 
 		_( "Background" ), 
-		_( "Background value" ),
+		_( "Background colour" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadPdf, background ),
 		VIPS_TYPE_ARRAY_DOUBLE );
 
 	VIPS_ARG_STRING( class, "password", 25, 
 		_( "Password" ), 
-		_( "Decrypt with this password" ),
+		_( "Password to decrypt with" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignLoadPdf, password ),
 		NULL );
