@@ -14,6 +14,8 @@
  * 	- don't add date with @strip [ewelot]
  * 28/10/21
  * 	- add @format, default type by filename
+ * 30/8/22
+ *      - add ".pnm", save as image format [ewelot]
  */
 
 /*
@@ -51,7 +53,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -93,8 +95,6 @@ vips_foreign_save_ppm_dispose( GObject *gobject )
 {
 	VipsForeignSavePpm *ppm = (VipsForeignSavePpm *) gobject;
 
-	if( ppm->target ) 
-		vips_target_finish( ppm->target );
 	VIPS_UNREF( ppm->target );
 
 	G_OBJECT_CLASS( vips_foreign_save_ppm_parent_class )->
@@ -241,6 +241,7 @@ vips_foreign_save_ppm_build( VipsObject *object )
 	 *   pgm ... 1 band many bit
 	 *   ppm ... 3 band many bit
 	 *   pfm ... 1 or 3 bands, 32 bit
+	 *   pnm ... pick from input
 	 */
 	switch( ppm->format ) {
 	case VIPS_FOREIGN_PPM_FORMAT_PBM:
@@ -267,8 +268,9 @@ vips_foreign_save_ppm_build( VipsObject *object )
 		target_format = VIPS_FORMAT_FLOAT;
 		break;
 
+	case VIPS_FOREIGN_PPM_FORMAT_PNM:
 	default:
-		/* Harmless.
+		/* Just use the input format and interpretation.
 		 */
 		break;
 	}
@@ -423,7 +425,8 @@ vips_foreign_save_ppm_build( VipsObject *object )
 	if( vips_sink_disc( image, vips_foreign_save_ppm_block, ppm ) )
 		return( -1 );
 
-	vips_target_finish( ppm->target );
+	if( vips_target_end( ppm->target ) )
+		return( -1 );
 
 	return( 0 );
 }
@@ -441,9 +444,9 @@ vips_foreign_save_ppm_build( VipsObject *object )
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static int bandfmt_ppm[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, UC, US, US, UI, UI, F,  F,  F,  F
+static VipsBandFormat bandfmt_ppm[10] = {
+	/* Band format:  UC  C   US  S   UI  I   F  X  D  DX */
+	/* Promotion: */ UC, UC, US, US, UI, UI, F, F, F, F
 };
 
 static void
@@ -474,21 +477,21 @@ vips_foreign_save_ppm_class_init( VipsForeignSavePpmClass *class )
 
 	VIPS_ARG_BOOL( class, "ascii", 10, 
 		_( "ASCII" ), 
-		_( "save as ascii" ),
+		_( "Save as ascii" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSavePpm, ascii ),
 		FALSE );
 
 	VIPS_ARG_INT( class, "bitdepth", 15,
-		_( "bitdepth" ),
-		_( "set to 1 to write as a 1 bit image" ),
+		_( "Bit depth" ),
+		_( "Set to 1 to write as a 1 bit image" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSavePpm, bitdepth ),
 		0, 1, 0 );
 
 	VIPS_ARG_BOOL( class, "squash", 11, 
 		_( "Squash" ), 
-		_( "save as one bit" ),
+		_( "Save as one bit" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignSavePpm, squash ),
 		FALSE );
@@ -528,6 +531,8 @@ vips_foreign_save_ppm_file_build( VipsObject *object )
 		ppm->format = VIPS_FOREIGN_PPM_FORMAT_PGM;
 	else if( vips_iscasepostfix( file->filename, ".pfm" ) )
 		ppm->format = VIPS_FOREIGN_PPM_FORMAT_PFM;
+	else if( vips_iscasepostfix( file->filename, ".pnm" ) )
+		ppm->format = VIPS_FOREIGN_PPM_FORMAT_PNM;
 
 	return( VIPS_OBJECT_CLASS( vips_foreign_save_ppm_file_parent_class )->
 		build( object ) );
@@ -642,7 +647,7 @@ vips_foreign_save_pbm_target_class_init(
 
 	/* Hide from UI.
 	 */
-	operation_class->flags = VIPS_OPERATION_DEPRECATED;
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
 }
 
 static void
@@ -674,7 +679,7 @@ vips_foreign_save_pgm_target_class_init(
 
 	/* Hide from UI.
 	 */
-	operation_class->flags = VIPS_OPERATION_DEPRECATED;
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
 }
 
 static void
@@ -706,7 +711,7 @@ vips_foreign_save_pfm_target_class_init(
 
 	/* Hide from UI.
 	 */
-	operation_class->flags = VIPS_OPERATION_DEPRECATED;
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
 }
 
 static void
@@ -717,6 +722,38 @@ vips_foreign_save_pfm_target_init( VipsForeignSavePfmTarget *target )
 	ppm->format = VIPS_FOREIGN_PPM_FORMAT_PFM;
 }
 
+typedef VipsForeignSavePpmTarget VipsForeignSavePnmTarget;
+typedef VipsForeignSavePpmTargetClass VipsForeignSavePnmTargetClass;
+
+G_DEFINE_TYPE( VipsForeignSavePnmTarget, vips_foreign_save_pnm_target, 
+	vips_foreign_save_ppm_target_get_type() );
+
+static void
+vips_foreign_save_pnm_target_class_init( 
+	VipsForeignSavePfmTargetClass *class )
+{
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
+	VipsOperationClass *operation_class = (VipsOperationClass *) class;
+
+	object_class->nickname = "pnmsave_target";
+	object_class->description = _( "save image in pnm format" );
+
+	foreign_class->suffs = vips__save_pnm_suffs;
+
+	/* Hide from UI.
+	 */
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
+}
+
+static void
+vips_foreign_save_pnm_target_init( VipsForeignSavePfmTarget *target )
+{
+	VipsForeignSavePpm *ppm = (VipsForeignSavePpm *) target;
+
+	ppm->format = VIPS_FOREIGN_PPM_FORMAT_PNM;
+}
+
 #endif /*HAVE_PPM*/
 
 /**
