@@ -12,6 +12,8 @@
  * 	- use libspng for save
  * 15/7/22 [lovell]
  * 	- default filter to none
+ * 17/11/22
+ * 	- add exif save
  */
 
 /*
@@ -56,6 +58,7 @@
 #include <string.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
 
 #include "pforeign.h"
 #include "quantise.h"
@@ -217,7 +220,7 @@ vips_foreign_save_spng_metadata( VipsForeignSaveSpng *spng, VipsImage *in )
 			"of ICC profile\n", length );
 #endif /*DEBUG*/
 
-		vips_strncpy( iccp.profile_name, "", 
+		vips_strncpy( iccp.profile_name, "icc",
 			sizeof( iccp.profile_name ) );
 		iccp.profile_len = length;
 		iccp.profile = (void *) data;
@@ -244,6 +247,25 @@ vips_foreign_save_spng_metadata( VipsForeignSaveSpng *spng, VipsImage *in )
 		g_free( str );
 	}
 
+	if( vips_image_get_typeof( in, VIPS_META_EXIF_NAME ) ) {
+		struct spng_exif exif;
+
+		if( vips__exif_update( in ) ||
+			vips_image_get_blob( in, VIPS_META_EXIF_NAME, 
+				(const void **) &exif.data, &exif.length ) )
+			return( -1 );
+
+		/* libspng does not want the JFIF "Exif\0\0" prefix.
+		 */
+		if( exif.length >= 6 &&
+			vips_isprefix( "Exif", exif.data ) ) {
+			exif.data += 6;
+			exif.length -= 6;
+		}
+
+		spng_set_exif( spng->ctx, &exif );
+	}
+
 	if( vips_image_map( in, vips_foreign_save_spng_comment, spng ) )
 		return( -1 );
 
@@ -270,6 +292,7 @@ vips_foreign_save_spng_pack( VipsForeignSaveSpng *spng,
 	VipsPel *q, VipsPel *p, size_t n )
 {
         int pixel_mask = 8 / spng->bitdepth - 1;
+	int shift = spng->palette ? 0 : 8 - spng->bitdepth;
 
         VipsPel bits;
         size_t x;
@@ -277,7 +300,7 @@ vips_foreign_save_spng_pack( VipsForeignSaveSpng *spng,
         bits = 0;
         for( x = 0; x < n; x++ ) {
                 bits <<= spng->bitdepth;
-                bits |= p[x];
+		bits |= p[x] >> shift;
 
                 if( (x & pixel_mask) == pixel_mask )
                         *q++ = bits;
@@ -361,8 +384,6 @@ vips_foreign_save_spng_write( VipsForeignSaveSpng *spng, VipsImage *in )
 	int error;
 	struct spng_ihdr ihdr;
 	struct spng_phys phys;
-	struct spng_plte plte = { 0 };
-	struct spng_trns trns = { 0 };
 	int fmt;
 	enum spng_encode_flags encode_flags;
 
@@ -374,8 +395,76 @@ vips_foreign_save_spng_write( VipsForeignSaveSpng *spng, VipsImage *in )
 		return( -1 );
 	}
 
+	ihdr.width = in->Xsize;
+	ihdr.height = in->Ysize;
+	ihdr.bit_depth = spng->bitdepth;
+
+	switch( in->Bands ) {
+	case 1:
+		ihdr.color_type = SPNG_COLOR_TYPE_GRAYSCALE;
+		break;
+
+	case 2:
+		ihdr.color_type = SPNG_COLOR_TYPE_GRAYSCALE_ALPHA; 
+		break;
+
+	case 3:
+		ihdr.color_type = SPNG_COLOR_TYPE_TRUECOLOR;
+		break;
+
+	case 4: 
+		ihdr.color_type = SPNG_COLOR_TYPE_TRUECOLOR_ALPHA; 
+		break;
+
+	default:
+		vips_error( class->nickname, "%s", _( "bad bands" ) );
+		return( -1 );
+	}
+
+#ifdef HAVE_QUANTIZATION
+	/* Enable image quantisation to paletted 8bpp PNG if palette is set.
+	 */
+	if( spng->palette )
+		ihdr.color_type = SPNG_COLOR_TYPE_INDEXED;
+#else
+	if( spng->palette )
+		g_warning( "%s",
+			_( "ignoring palette (no quantisation support)" ) );
+#endif /*HAVE_QUANTIZATION*/
+
+	ihdr.compression_method = 0;
+	ihdr.filter_method = 0;
+	ihdr.interlace_method = spng->interlace ? 1 : 0;
+	if( (error = spng_set_ihdr( spng->ctx, &ihdr )) ) {
+		vips_error( class->nickname, "%s", spng_strerror( error ) ); 
+		return( -1 );
+	}
+
+	spng_set_option( spng->ctx, 
+		SPNG_IMG_COMPRESSION_LEVEL, spng->compression );
+	spng_set_option( spng->ctx, 
+		SPNG_TEXT_COMPRESSION_LEVEL, spng->compression );
+	spng_set_option( spng->ctx, 
+		SPNG_FILTER_CHOICE, spng->filter );
+
+	/* Set resolution. spng uses pixels per meter.
+	 */
+	phys.unit_specifier = 1;
+	phys.ppu_x = VIPS_RINT( in->Xres * 1000.0 );
+	phys.ppu_y = VIPS_RINT( in->Xres * 1000.0 );
+	spng_set_phys( spng->ctx, &phys );
+
+	/* Metadata.
+	 */
+	if( !save->strip &&
+		vips_foreign_save_spng_metadata( spng, in ) )
+		return( -1 );
+
 #ifdef HAVE_QUANTIZATION
 	if( spng->palette ) {
+		struct spng_plte plte = { 0 };
+		struct spng_trns trns = { 0 };
+
 		VipsImage *im_index;
 		VipsImage *im_palette;
 		int palette_count;
@@ -405,14 +494,9 @@ vips_foreign_save_spng_write( VipsForeignSaveSpng *spng, VipsImage *in )
 			entry->blue = p[2];
 			plte.n_entries += 1;
 
-			/* Quantizr and libimagequant sort the pallette
-			 * by transparency, so trns.type3_alpha[] and
-			 * plte.entries[] will use the same indexing.
-			 */
-			g_assert( i == 0 || p[3] >= p[-1] );
+			trns.type3_alpha[i] = p[3];
 			if( p[3] != 255 ) {
-				trns.type3_alpha[trns.n_type3_entries] = p[3];
-				trns.n_type3_entries += 1;
+				trns.n_type3_entries = i+1;
 			}
 		}
 
@@ -425,14 +509,14 @@ vips_foreign_save_spng_write( VipsForeignSaveSpng *spng, VipsImage *in )
 
 		VIPS_UNREF( im_palette );
 
+		spng_set_plte( spng->ctx, &plte );
+		if( trns.n_type3_entries ) 
+			spng_set_trns( spng->ctx, &trns );
+
 		in = spng->memory = im_index;
 	}
 #endif /*HAVE_QUANTIZATION*/
 
-	ihdr.width = in->Xsize;
-	ihdr.height = in->Ysize;
-	ihdr.bit_depth = spng->bitdepth;
-
 	/* Low-bitdepth write needs an extra buffer for packing pixels.
 	 */
 	if( spng->bitdepth < 8 ) {
@@ -444,67 +528,6 @@ vips_foreign_save_spng_write( VipsForeignSaveSpng *spng, VipsImage *in )
 			return( -1 );
 	}
 
-	switch( in->Bands ) {
-	case 1:
-		if( spng->palette )
-			ihdr.color_type = SPNG_COLOR_TYPE_INDEXED; 
-		else
-			ihdr.color_type = SPNG_COLOR_TYPE_GRAYSCALE;
-		break;
-
-	case 2:
-		ihdr.color_type = SPNG_COLOR_TYPE_GRAYSCALE_ALPHA; 
-		break;
-
-	case 3:
-		ihdr.color_type = SPNG_COLOR_TYPE_TRUECOLOR;
-		break;
-
-	case 4: 
-		ihdr.color_type = SPNG_COLOR_TYPE_TRUECOLOR_ALPHA; 
-		break;
-
-	default:
-		vips_error( class->nickname, "%s", _( "bad bands" ) );
-		return( -1 );
-	}
-
-	ihdr.compression_method = 0;
-	ihdr.filter_method = 0;
-	ihdr.interlace_method = spng->interlace ? 1 : 0;
-	if( (error = spng_set_ihdr( spng->ctx, &ihdr )) ) {
-		vips_error( class->nickname, "%s", spng_strerror( error ) ); 
-		return( -1 );
-	}
-
-	spng_set_option( spng->ctx, 
-		SPNG_IMG_COMPRESSION_LEVEL, spng->compression );
-	spng_set_option( spng->ctx, 
-		SPNG_TEXT_COMPRESSION_LEVEL, spng->compression );
-	spng_set_option( spng->ctx, 
-		SPNG_FILTER_CHOICE, spng->filter );
-
-	/* Set resolution. libpng uses pixels per meter.
-	 */
-	phys.unit_specifier = 1;
-	phys.ppu_x = VIPS_RINT( in->Xres * 1000.0 );
-	phys.ppu_y = VIPS_RINT( in->Xres * 1000.0 );
-	spng_set_phys( spng->ctx, &phys );
-
-	/* Metadata.
-	 */
-	if( !save->strip &&
-		vips_foreign_save_spng_metadata( spng, in ) )
-		return( -1 );
-
-#ifdef HAVE_QUANTIZATION
-	if( spng->palette ) {
-		spng_set_plte( spng->ctx, &plte );
-		if( trns.n_type3_entries ) 
-			spng_set_trns( spng->ctx, &trns );
-	}
-#endif /*HAVE_QUANTIZATION*/
-
 	/* SPNG_FMT_PNG is a special value that matches the format in ihdr 
 	 */
 	fmt = SPNG_FMT_PNG;
@@ -579,12 +602,22 @@ vips_foreign_save_spng_build( VipsObject *object )
 	in = save->ready;
 	g_object_ref( in );
 
-	/* in will have been converted to uint16 for high-bitdepth
-	 * formats (eg. float) ... we need to check Type to see if we want 
-	 * to save as 8 or 16-bits. Eg. imagine a float image tagged as sRGB.
+	/* If no output bitdepth has been specified, use input Type to pick.
 	 */
-	if( in->Type == VIPS_INTERPRETATION_sRGB ||
-		in->Type == VIPS_INTERPRETATION_B_W ) {
+        if( !vips_object_argument_isset( object, "bitdepth" ) ) 
+		spng->bitdepth = 
+                        in->Type == VIPS_INTERPRETATION_RGB16 ||
+                        in->Type == VIPS_INTERPRETATION_GREY16 ? 16 : 8;
+
+	/* Deprecated "colours" arg just sets bitdepth large enough to hold
+	 * that many colours.
+	 */
+        if( vips_object_argument_isset( object, "colours" ) ) 
+		spng->bitdepth = ceil( log2( spng->colours ) );
+
+	/* Cast in down to 8 bit if we can.
+	 */
+	if( spng->bitdepth <= 8 ) { 
 		VipsImage *x;
 
 		if( vips_cast( in, &x, VIPS_FORMAT_UCHAR, NULL ) ) {
@@ -595,19 +628,6 @@ vips_foreign_save_spng_build( VipsObject *object )
 		in = x;
 	}
 
-	/* If no output bitdepth has been specified, use input Type to pick.
-	 * We only go for 16 bits for the types where we know there's a
-	 * 0-65535 range.
-	 */
-        if( !vips_object_argument_isset( object, "bitdepth" ) ) 
-		spng->bitdepth = in->BandFmt == VIPS_FORMAT_UCHAR ? 8 : 16;
-
-	/* Deprecated "colours" arg just sets bitdepth large enough to hold
-	 * that many colours.
-	 */
-        if( vips_object_argument_isset( object, "colours" ) ) 
-		spng->bitdepth = ceil( log2( spng->colours ) );
-
 	/* If this is a RGB or RGBA image and a low bit depth has been
 	 * requested, enable palettisation.
 	 */
@@ -615,6 +635,11 @@ vips_foreign_save_spng_build( VipsObject *object )
 		spng->bitdepth < 8 )
 		spng->palette = TRUE;
 
+	/* Disable palettization for >8 bit save.
+	 */
+	if( spng->bitdepth > 8 )
+		spng->palette = FALSE;
+
 	if( vips_foreign_save_spng_write( spng, in ) ) {
 		g_object_unref( in );
 		return( -1 );
@@ -625,14 +650,15 @@ vips_foreign_save_spng_build( VipsObject *object )
 	return( 0 );
 }
 
-/* Except for 8-bit inputs, we send everything else to 16. We decide on spng8
- * vs. spng16 based on Type in_build(), see above.
- */
 #define UC VIPS_FORMAT_UCHAR
 #define US VIPS_FORMAT_USHORT
-static int bandfmt_spng[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, UC, US, US, US, US, US, US, US, US
+
+/* Except for 8-bit inputs, we send everything else to 16. We decide on png8
+ * vs. png16 based on Type in_build(), see above.
+ */
+static VipsBandFormat bandfmt_spng[10] = {
+	/* Band format:  UC  C   US  S   UI  I   F   X   D   DX */
+	/* Promotion: */ UC, UC, US, US, US, US, US, US, US, US
 };
 
 static void
