@@ -19,6 +19,15 @@
  * 	- requires us to use the gio API to librsvg
  * 11/9/19
  * 	- rework as a sequential loader to reduce overcomputation
+ * 11/6/21
+ * 	- switch to rsvg_handle_render_document()
+ * 	- librsvg can no longer render very large images :( 
+ * 14/10/21
+ * 	- allow utf-8 headers for svg detection
+ * 28/4/22
+ * 	- support rsvg_handle_get_intrinsic_size_in_pixels()
+ * 5/6/22
+ * 	- allow random access
  */
 
 /*
@@ -55,7 +64,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -67,316 +76,20 @@
 #include <vips/internal.h>
 #include <vips/debug.h>
 
-/* We need GIO for GInputStream.
- */
-
-#if defined(HAVE_RSVG) && defined(HAVE_GIO)
-
-#include <gio/gio.h>
-
-/* A GInputStream that's actually a VipsSource under the hood. This lets us
- * hook librsvg up to libvips using the GInputStream interface.
- */
-
-#define VIPS_TYPE_G_INPUT_STREAM (vips_g_input_stream_get_type())
-#define VIPS_G_INPUT_STREAM( obj ) \
-	(G_TYPE_CHECK_INSTANCE_CAST( (obj), \
-	VIPS_TYPE_G_INPUT_STREAM, VipsGInputStream ))
-#define VIPS_G_INPUT_STREAM_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_CAST( (klass), \
-	VIPS_TYPE_G_INPUT_STREAM, VipsGInputStreamClass))
-#define VIPS_IS_G_INPUT_STREAM( obj ) \
-	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), VIPS_TYPE_G_INPUT_STREAM ))
-#define VIPS_IS_G_INPUT_STREAM_CLASS( klass ) \
-	(G_TYPE_CHECK_CLASS_TYPE( (klass), VIPS_TYPE_G_INPUT_STREAM ))
-#define VIPS_G_INPUT_STREAM_GET_CLASS( obj ) \
-	(G_TYPE_INSTANCE_GET_CLASS( (obj), \
-	VIPS_TYPE_G_INPUT_STREAM, VipsGInputStreamClass ))
-
-/* GInputStream <--> VipsSource
- */
-typedef struct _VipsGInputStream {
-	GInputStream parent_instance;
-
-	/*< private >*/
-
-	/* The VipsSource we wrap.
-	 */
-	VipsSource *source;
-
-} VipsGInputStream;
-
-typedef struct _VipsGInputStreamClass {
-	GInputStreamClass parent_class;
-
-} VipsGInputStreamClass;
-
-static void vips_g_input_stream_seekable_iface_init( GSeekableIface *iface );
-
-G_DEFINE_TYPE_WITH_CODE( VipsGInputStream, vips_g_input_stream, 
-	G_TYPE_INPUT_STREAM, G_IMPLEMENT_INTERFACE( G_TYPE_SEEKABLE,
-		vips_g_input_stream_seekable_iface_init ) )
-
-enum {
-	PROP_0,
-	PROP_STREAM
-};
-
-static void
-vips_g_input_stream_get_property( GObject *object, guint prop_id,
-	GValue *value, GParamSpec *pspec )
-{
-	VipsGInputStream *gstream = VIPS_G_INPUT_STREAM( object );
-
-	switch( prop_id ) {
-	case PROP_STREAM:
-		g_value_set_object( value, gstream->source );
-		break;
-
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID( object, prop_id, pspec );
-	}
-}
-
-static void
-vips_g_input_stream_set_property( GObject *object, guint prop_id,
-	const GValue *value, GParamSpec *pspec )
-{
-	VipsGInputStream *gstream = VIPS_G_INPUT_STREAM( object );
-
-	switch( prop_id ) {
-	case PROP_STREAM:
-		gstream->source = g_value_dup_object( value );
-		break;
-
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID( object, prop_id, pspec );
-	}
-}
-
-static void
-vips_g_input_stream_finalize( GObject *object )
-{
-	VipsGInputStream *gstream = VIPS_G_INPUT_STREAM( object );
-
-	VIPS_FREEF( g_object_unref, gstream->source );
-
-	G_OBJECT_CLASS( vips_g_input_stream_parent_class )->finalize( object );
-}
-
-static goffset
-vips_g_input_stream_tell( GSeekable *seekable )
-{
-	VipsSource *source = VIPS_G_INPUT_STREAM( seekable )->source;
-
-	goffset pos;
-
-	VIPS_DEBUG_MSG( "vips_g_input_stream_tell:\n" );
-
-	pos = vips_source_seek( source, 0, SEEK_CUR );
-	if( pos == -1 )
-		return( 0 );
-
-	return( pos );
-}
-
-static gboolean
-vips_g_input_stream_can_seek( GSeekable *seekable )
-{
-	VipsSource *source = VIPS_G_INPUT_STREAM( seekable )->source;
-
-	VIPS_DEBUG_MSG( "vips_g_input_stream_can_seek: %d\n", 
-		!source->is_pipe );
-
-	return( !source->is_pipe );
-}
-
-static int
-seek_type_to_lseek( GSeekType type )
-{
-	switch( type ) {
-	default:
-	case G_SEEK_CUR:
-		return( SEEK_CUR );
-	case G_SEEK_SET:
-		return( SEEK_SET );
-	case G_SEEK_END:
-		return( SEEK_END );
-	}
-}
-
-static gboolean
-vips_g_input_stream_seek( GSeekable *seekable, goffset offset,
-	GSeekType type, GCancellable *cancellable, GError **error )
-{
-	VipsSource *source = VIPS_G_INPUT_STREAM( seekable )->source;
-
-	VIPS_DEBUG_MSG( "vips_g_input_stream_seek: offset = %" G_GINT64_FORMAT
-		", type = %d\n", offset, type );
-
-	if( vips_source_seek( source, offset, 
-		seek_type_to_lseek( type ) ) == -1 ) {
-		g_set_error( error, G_IO_ERROR,
-			G_IO_ERROR_FAILED,
-			_( "Error while seeking: %s" ),
-			vips_error_buffer() );
-		return( FALSE );
-	}
-
-
-	return( TRUE );
-}
-
-static gboolean
-vips_g_input_stream_can_truncate( GSeekable *seekable )
-{
-	return( FALSE );
-}
-
-static gboolean
-vips_g_input_stream_truncate( GSeekable *seekable, goffset offset,
-	GCancellable *cancellable, GError **error )
-{
-	g_set_error_literal( error,
-		G_IO_ERROR,
-		G_IO_ERROR_NOT_SUPPORTED,
-		_( "Cannot truncate VipsGInputStream" ) );
-
-	return( FALSE );
-}
-
-static gssize
-vips_g_input_stream_read( GInputStream *stream, void *buffer, gsize count,
-	GCancellable *cancellable, GError **error )
-{
-	VipsSource *source;
-	gssize res;
-
-	source = VIPS_G_INPUT_STREAM( stream )->source;
-
-	VIPS_DEBUG_MSG( "vips_g_input_stream_read: count: %zd\n", count );
-
-	if( g_cancellable_set_error_if_cancelled( cancellable, error ) )
-		return( -1 );
-
-	if( (res = vips_source_read( source, buffer, count )) == -1 )
-		g_set_error( error, G_IO_ERROR,
-			G_IO_ERROR_FAILED,
-			_( "Error while reading: %s" ),
-			vips_error_buffer() );
-
-	return( res );
-}
-
-static gssize
-vips_g_input_stream_skip( GInputStream *stream, gsize count,
-	GCancellable *cancellable, GError **error )
-{
-	VipsSource *source;
-	gssize position;
-
-	source = VIPS_G_INPUT_STREAM( stream )->source;
-
-	VIPS_DEBUG_MSG( "vips_g_input_stream_skip: count: %zd\n", count );
-
-	if( g_cancellable_set_error_if_cancelled( cancellable, error ) )
-		return( -1 );
-
-	position = vips_source_seek( source, count, SEEK_CUR );
-	if( position == -1 ) {
-		g_set_error( error, G_IO_ERROR,
-			G_IO_ERROR_FAILED,
-			_( "Error while seeking: %s" ),
-			vips_error_buffer() );
-		return( -1 );
-	}
-
-	return( position );
-}
-
-static gboolean
-vips_g_input_stream_close( GInputStream *stream,
-	GCancellable *cancellable, GError **error )
-{
-	VipsGInputStream *gstream = VIPS_G_INPUT_STREAM( stream );
-
-	vips_source_minimise( gstream->source );
-
-	return( TRUE );
-}
-
-static void
-vips_g_input_stream_seekable_iface_init( GSeekableIface *iface )
-{
-	iface->tell = vips_g_input_stream_tell;
-	iface->can_seek = vips_g_input_stream_can_seek;
-	iface->seek = vips_g_input_stream_seek;
-	iface->can_truncate = vips_g_input_stream_can_truncate;
-	iface->truncate_fn = vips_g_input_stream_truncate;
-}
-
-static void
-vips_g_input_stream_class_init( VipsGInputStreamClass *class )
-{
-	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
-	GInputStreamClass *istream_class = G_INPUT_STREAM_CLASS( class );
-
-	gobject_class->finalize = vips_g_input_stream_finalize;
-	gobject_class->get_property = vips_g_input_stream_get_property;
-	gobject_class->set_property = vips_g_input_stream_set_property;
-
-	istream_class->read_fn = vips_g_input_stream_read;
-	istream_class->skip = vips_g_input_stream_skip;
-	istream_class->close_fn = vips_g_input_stream_close;
-
-	g_object_class_install_property( gobject_class, PROP_STREAM,
-		g_param_spec_object( "input",
-			_( "Input" ),
-			_( "Stream to wrap" ),
-			VIPS_TYPE_SOURCE, G_PARAM_CONSTRUCT_ONLY | 
-				G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS ) );
-
-}
-
-static void
-vips_g_input_stream_init( VipsGInputStream *gstream )
-{
-}
-
-/**
- * g_input_stream_new_from_vips:
- * @source: stream to wrap
- *
- * Create a new #GInputStream wrapping a #VipsSource.
- *
- * Returns: a new #GInputStream
- */
-static GInputStream *
-g_input_stream_new_from_vips( VipsSource *source )
-{
-	return( g_object_new( VIPS_TYPE_G_INPUT_STREAM,
-		"input", source,
-		NULL ) );
-}
+#if defined(HAVE_RSVG)
 
 #include <cairo.h>
 #include <librsvg/rsvg.h>
 
+/* Render SVGs with tiles this size. They need to be pretty big to limit 
+ * overcomputation.
+ */
+#define TILE_SIZE (2000)
+
 /* The <svg tag must appear within this many bytes of the start of the file.
  */
 #define SVG_HEADER_SIZE (1000)
 
-/* The maximum pixel width librsvg is able to render.
- */
-#define RSVG_MAX_WIDTH (32767)
-
-/* Old librsvg versions don't include librsvg-features.h by default.
- * Newer versions deprecate direct inclusion.
- */
-#ifndef LIBRSVG_FEATURES_H
-#include <librsvg/librsvg-features.h>
-#endif
-
 /* A handy #define for we-will-handle-svgz.
  */
 #if LIBRSVG_CHECK_FEATURE(SVGZ) && defined(HAVE_ZLIB)
@@ -429,6 +142,85 @@ vips_foreign_load_svg_zfree( void *opaque, void *ptr )
 }
 #endif /*HANDLE_SVGZ*/
 
+/* Find a utf-8 substring within the first len_bytes (not characters). 
+ *
+ *   - case-insensitive
+ *   - needle must be zero-terminated, but hackstack need not be
+ *   - haystack can be null-terminated
+ *   - if haystack is shorter than len bytes, that'll end the search 
+ *   - if we hit invalid utf-8, we return NULL
+ */
+static const char *
+vips_utf8_strcasestr( const char *haystack_start, const char *needle_start, 
+	int len_bytes )
+{
+        int needle_len = g_utf8_strlen( needle_start, -1 );
+        int needle_len_bytes = strlen( needle_start );
+
+	const char *haystack;
+
+	for( haystack = haystack_start; 
+		haystack - haystack_start <= len_bytes - needle_len_bytes; 
+		haystack = g_utf8_find_next_char( haystack, NULL ) ) {
+                const char *needle_char;
+                const char *haystack_char;
+		int i;
+
+                haystack_char = haystack;
+                needle_char = needle_start;
+                for( i = 0; i < needle_len; i++ ) {
+			/* Haystack isn't necessarily null-terminated and
+			 * might end half-way through a utf-8 character, so we
+			 * need to be careful not to run off the end.
+			 */
+                        gunichar a = 
+				g_utf8_get_char_validated( haystack_char, 
+					haystack_start + len_bytes - haystack );
+                        gunichar b = 
+				g_utf8_get_char_validated( needle_char, -1 );
+
+                        /* Invalid utf8? 
+			 *
+			 * gunichar is a uint32, so we can't compare < 0, we 
+			 * have to look for -1 and -2 (the two possible error 
+			 * values).
+                         */
+                        if( a == (gunichar) -1 ||
+				a == (gunichar) -2 ||
+				b == (gunichar) -1 ||
+				b == (gunichar) -2 )
+                                return( NULL );
+
+                        /* End of haystack. There can't be a complete needle
+                         * anywhere.
+                         */
+                        if( a == (gunichar) 0 )
+                                return( NULL );
+
+                        /* Mismatch.
+                         */
+                        if( g_unichar_tolower( a ) != g_unichar_tolower( b ) )
+                                break;
+
+                        haystack_char = 
+				g_utf8_find_next_char( haystack_char, 
+					haystack_start + len_bytes );
+                        needle_char = 
+				g_utf8_find_next_char( needle_char, NULL );
+                }
+
+                if( i == needle_len )
+			/* Walked the whole of needle, so we must have found a 
+			 * complete match.
+			 */
+                        return( haystack );
+        }
+
+        /* Walked the whole of haystack without finding a match.
+         */
+        return( NULL );
+}
+
 /* This is used by both the file and buffer subclasses.
  */
 static gboolean
@@ -443,8 +235,6 @@ vips_foreign_load_svg_is_a( const void *buf, size_t len )
 	char obuf[SVG_HEADER_SIZE];
 #endif /*HANDLE_SVGZ*/
 
-	int i;
-
 	/* Start with str pointing at the argument buffer, swap to it pointing
 	 * into obuf if we see zip data.
 	 */
@@ -498,23 +288,17 @@ vips_foreign_load_svg_is_a( const void *buf, size_t len )
 	 *
 	 * But there can be a doctype in there too. And case and whitespace can
 	 * vary a lot. And the <?xml can be missing. And you can have a comment
-	 * before the <svg line.
+	 * before the <svg line. And it can be utf-8, so non ASCII characters.
 	 *
-	 * Simple rules:
-	 * - first 24 chars are plain ascii
-	 * - first SVG_HEADER_SIZE chars contain "<svg", upper or lower case.
+	 * All we do is look for "<svg", any case, within the first
+	 * SVG_HEADER_SIZE bytes, where the bytestream up to the "<svg" is
+	 * valid utf-8.
 	 *
 	 * We could rsvg_handle_new_from_data() on the buffer, but that can be
 	 * horribly slow for large documents. 
 	 */
-	if( len < 24 )
-		return( 0 );
-	for( i = 0; i < 24; i++ )
-		if( !isascii( str[i] ) )
-			return( FALSE );
-	for( i = 0; i < SVG_HEADER_SIZE && i < len - 5; i++ )
-		if( g_ascii_strncasecmp( str + i, "<svg", 4 ) == 0 )
-			return( TRUE );
+	if( vips_utf8_strcasestr( str, "<svg", len ) )
+		return( TRUE );
 
 	return( FALSE );
 }
@@ -533,57 +317,223 @@ vips_foreign_load_svg_dispose( GObject *gobject )
 static VipsForeignFlags
 vips_foreign_load_svg_get_flags_filename( const char *filename )
 {
-	return( VIPS_FOREIGN_SEQUENTIAL );
+	/* We can render any part of the page on demand.
+	 */
+	return( VIPS_FOREIGN_PARTIAL );
 }
 
 static VipsForeignFlags
 vips_foreign_load_svg_get_flags( VipsForeignLoad *load )
 {
-	return( VIPS_FOREIGN_SEQUENTIAL );
+	return( VIPS_FOREIGN_PARTIAL );
 }
 
-static void
-vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
+#if LIBRSVG_CHECK_VERSION( 2, 52, 0 )
+/* Derived from `CssLength::to_user` in librsvg.
+ * https://gitlab.gnome.org/GNOME/librsvg/-/blob/e6607c9ae8d8409d4efff6b12993717400b3356e/src/length.rs#L368
+ */
+static double
+svg_css_length_to_pixels( RsvgLength length, double dpi )
+{
+	double value = length.length;
+
+	/* The following implies that our default font size is 12, which
+	 * matches the default in librsvg.
+	 */
+	double font_size = 12.0;
+
+	switch( length.unit ) {
+		case RSVG_UNIT_PX:
+			/* Already a pixel value.
+			 */
+			break;
+		case RSVG_UNIT_EM:
+			value *= font_size;
+			break;
+		case RSVG_UNIT_EX:
+			value *= font_size / 2.0;
+			break;
+		case RSVG_UNIT_IN:
+			value *= dpi;
+			break;
+		case RSVG_UNIT_CM:
+			/* 2.54 cm in an inch.
+			 */
+			value = dpi * value / 2.54;
+			break;
+		case RSVG_UNIT_MM:
+			/* 25.4 mm in an inch.
+			 */
+			value = dpi * value / 25.4;
+			break;
+		case RSVG_UNIT_PT:
+			/* 72 points in an inch.
+			 */
+			value = dpi * value / 72;
+			break;
+		case RSVG_UNIT_PC:
+			/* 6 picas in an inch.
+			 */
+			value = dpi * value / 6;
+			break;
+		default:
+			/* Probably RSVG_UNIT_PERCENT. We can't know what the 
+			 * pixel value is without more information.
+			 */
+			value = 0;
+	}
+
+	return value;
+}
+#endif
+
+static int
+vips_foreign_load_svg_get_natural_size( VipsForeignLoadSvg *svg, 
+	double *out_width, double *out_height )
+{
+	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( svg );
+
+	double width;
+	double height;
+
+#if LIBRSVG_CHECK_VERSION( 2, 52, 0 )
+
+	if( !rsvg_handle_get_intrinsic_size_in_pixels( svg->page, 
+		&width, &height ) ) {
+		RsvgRectangle viewbox;
+
+		/* Try the intrinsic dimensions first.
+		 */
+		gboolean has_width, has_height;
+		RsvgLength iwidth, iheight;
+		gboolean has_viewbox;
+
+		rsvg_handle_get_intrinsic_dimensions( svg->page,
+			&has_width, &iwidth,
+			&has_height, &iheight,
+			&has_viewbox, &viewbox );
+
+#if LIBRSVG_CHECK_VERSION( 2, 54, 0 )
+		/* After librsvg 2.54.0, the `has_width` and `has_height` 
+		 * arguments always returns `TRUE`, since with SVG2 all 
+		 * documents *have* a default width and height of `100%`.
+		 */
+		width = svg_css_length_to_pixels( iwidth, svg->dpi );
+		height = svg_css_length_to_pixels( iheight, svg->dpi );
+
+		has_width = width > 0.0;
+		has_height = height > 0.0;
+
+		if( has_width && has_height ) {
+			/* Success! Taking the viewbox into account is not 
+			 * needed.
+			 */
+		}
+		else if( has_width && has_viewbox ) {
+			height = width * viewbox.height / viewbox.width;
+		}
+		else if( has_height && has_viewbox ) {
+			width = height * viewbox.width / viewbox.height;
+		}
+		else if( has_viewbox ) {
+			width = viewbox.width;
+			height = viewbox.height;
+		}
+#else /*!LIBRSVG_CHECK_VERSION( 2, 54, 0 )*/
+		if( has_width && has_height ) {
+			/* We can use these values directly.
+			 */
+			width = svg_css_length_to_pixels( iwidth, svg->dpi );
+			height = svg_css_length_to_pixels( iheight, svg->dpi );
+		}
+		else if( has_width && has_viewbox ) {
+			width = svg_css_length_to_pixels( iwidth, svg->dpi );
+			height = width * viewbox.height / viewbox.width;
+		}
+		else if( has_height && has_viewbox ) {
+			height = svg_css_length_to_pixels( iheight, svg->dpi );
+			width = height * viewbox.width / viewbox.height;
+		}
+		else if( has_viewbox ) {
+			width = viewbox.width;
+			height = viewbox.height;
+		}
+#endif /*!LIBRSVG_CHECK_VERSION( 2, 54, 0 )*/
+
+		if( width <= 0.0 ||
+			height <= 0.0 ) {
+			/* We haven't found a usable set of sizes, so try 
+			 * working out the visible area.
+			 */
+			rsvg_handle_get_geometry_for_element( svg->page, NULL,
+				&viewbox, NULL, NULL );
+			width = viewbox.x + viewbox.width;
+			height = viewbox.y + viewbox.height;
+		}
+	}
+
+#else /*!LIBRSVG_CHECK_VERSION( 2, 52, 0 )*/
+
 {
 	RsvgDimensionData dimensions;
-	int width;
-	int height;
-	double scale;
-	double res;
 
-	/* Calculate dimensions at default dpi/scale.
-	 */
-	rsvg_handle_set_dpi( svg->page, 72.0 );
 	rsvg_handle_get_dimensions( svg->page, &dimensions );
 	width = dimensions.width;
 	height = dimensions.height;
+}
 
-	/* Calculate dimensions at required dpi/scale.
+#endif /*LIBRSVG_CHECK_VERSION( 2, 52, 0 )*/
+
+	/* width or height below 0.5 can't be rounded to 1.
 	 */
-	scale = svg->scale * svg->dpi / 72.0;
-	if( scale != 1.0 ) {
-		rsvg_handle_set_dpi( svg->page, svg->dpi * svg->scale );
-		rsvg_handle_get_dimensions( svg->page, &dimensions );
-
-		if( width == dimensions.width && 
-			height == dimensions.height ) {
-			/* SVG without width and height always reports the same 
-			 * dimensions regardless of dpi. Apply dpi/scale using 
-			 * cairo instead.
-			 */
-			svg->cairo_scale = scale;
-			width = VIPS_ROUND_UINT( width * scale );
-			height = VIPS_ROUND_UINT( height * scale );
-		} 
-		else {
-			/* SVG with width and height reports correctly scaled 
-			 * dimensions.
-			 */
-			width = dimensions.width;
-			height = dimensions.height;
-		}
+	if( width < 0.5 || 
+		height < 0.5 ) {
+		vips_error( class->nickname, "%s", _( "bad dimensions" ) );
+		return( -1 );
 	}
 
+	*out_width = width;
+	*out_height = height;
+
+	return( 0 );
+}
+
+static int
+vips_foreign_load_svg_get_scaled_size( VipsForeignLoadSvg *svg, 
+	int *out_width, int *out_height )
+{
+	double width;
+	double height;
+
+	/* Get dimensions with the default dpi.
+	 */
+	rsvg_handle_set_dpi( svg->page, 72.0 );
+	if( vips_foreign_load_svg_get_natural_size( svg, &width, &height ) )
+		return( -1 );
+
+	/* We scale up with cairo --- scaling with rsvg_handle_set_dpi() will
+	 * fail for SVGs with absolute sizes.
+	 */
+	svg->cairo_scale = svg->scale * svg->dpi / 72.0;
+	width *= svg->cairo_scale;
+	height *= svg->cairo_scale;
+
+	*out_width = VIPS_ROUND_UINT( width );
+	*out_height = VIPS_ROUND_UINT( height );
+
+	return ( 0 );
+}
+
+static int
+vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
+{
+	int width;
+	int height;
+	double res;
+
+	if( vips_foreign_load_svg_get_scaled_size( svg, &width, &height ) )
+		return( -1 );
+
 	/* We need pixels/mm for vips.
 	 */
 	res = svg->dpi / 25.4;
@@ -593,10 +543,12 @@ vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
 		4, VIPS_FORMAT_UCHAR,
 		VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, res, res );
 
-	/* We render to a linecache, so fat strips work well.
+	/* We use a tilecache, so it's smalltile.
 	 */
-        vips_image_pipelinev( out, VIPS_DEMAND_STYLE_FATSTRIP, NULL );
+        if( vips_image_pipelinev( out, VIPS_DEMAND_STYLE_SMALLTILE, NULL ) )
+		return( -1 );
 
+	return( 0 );
 }
 
 static int
@@ -604,26 +556,25 @@ vips_foreign_load_svg_header( VipsForeignLoad *load )
 {
 	VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) load;
 
-	vips_foreign_load_svg_parse( svg, load->out ); 
-
-	return( 0 );
+	return vips_foreign_load_svg_parse( svg, load->out );
 }
 
 static int
 vips_foreign_load_svg_generate( VipsRegion *or, 
 	void *seq, void *a, void *b, gboolean *stop )
 {
-	VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) a;
-	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( svg );
-	VipsRect *r = &or->valid;
+	const VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) a;
+	const VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( svg );
+	const VipsRect *r = &or->valid;
 
 	cairo_surface_t *surface;
 	cairo_t *cr;
 	int y;
 
 #ifdef DEBUG
-	printf( "vips_foreign_load_svg_generate:\n     "
+	printf( "vips_foreign_load_svg_generate: %p \n     "
 		"left = %d, top = %d, width = %d, height = %d\n", 
+		svg,
 		r->left, r->top, r->width, r->height ); 
 #endif /*DEBUG*/
 
@@ -639,26 +590,58 @@ vips_foreign_load_svg_generate( VipsRegion *or,
 	cr = cairo_create( surface );
 	cairo_surface_destroy( surface );
 
+	/* rsvg is single-threaded, but we don't need to lock since we're
+	 * running inside a non-threaded tilecache.
+	 */
+#if LIBRSVG_CHECK_VERSION( 2, 46, 0 )
+
+{
+	RsvgRectangle viewport;
+	GError *error = NULL;
+
+	/* No need to scale -- we always set the viewport to the
+	 * whole image, and set the region to draw on the surface.
+	 */
+	cairo_translate( cr, -r->left, -r->top );
+	viewport.x = 0;
+	viewport.y = 0;
+	viewport.width = or->im->Xsize;
+	viewport.height = or->im->Ysize;
+
+	if( !rsvg_handle_render_document( svg->page, cr, &viewport, &error ) ) {
+		cairo_destroy( cr );
+		vips_operation_invalidate( VIPS_OPERATION( svg ) );
+		vips_error( class->nickname, 
+			"%s", _( "SVG rendering failed" ) );
+		vips_g_error( &error );
+		return( -1 );
+	}
+
+	cairo_destroy( cr );
+}
+
+#else /*!LIBRSVG_CHECK_VERSION( 2, 46, 0 )*/
+
 	cairo_scale( cr, svg->cairo_scale, svg->cairo_scale );
 	cairo_translate( cr, -r->left / svg->cairo_scale,
 		-r->top / svg->cairo_scale );
 
-	/* rsvg is single-threaded, but we don't need to lock since we're
-	 * running inside a non-threaded tilecache.
-	 */
 	if( !rsvg_handle_render_cairo( svg->page, cr ) ) {
+		cairo_destroy( cr );
 		vips_operation_invalidate( VIPS_OPERATION( svg ) );
-		vips_error( class->nickname, 
+		vips_error( class->nickname,
 			"%s", _( "SVG rendering failed" ) );
 		return( -1 );
 	}
 
 	cairo_destroy( cr );
 
+#endif /*LIBRSVG_CHECK_VERSION( 2, 46, 0 )*/
+
 	/* Cairo makes pre-multipled BRGA -- we must byteswap and unpremultiply.
 	 */
 	for( y = 0; y < r->height; y++ ) 
-		vips__cairo2rgba( 
+                vips__premultiplied_bgra2rgba( 
 			(guint32 *) VIPS_REGION_ADDR( or, r->left, r->top + y ),
 			r->width ); 
 
@@ -672,28 +655,18 @@ vips_foreign_load_svg_load( VipsForeignLoad *load )
 	VipsImage **t = (VipsImage **) 
 		vips_object_local_array( (VipsObject *) load, 3 );
 
-	/* librsvg starts to fail if any axis in a single render call is over
-	 * RSVG_MAX_WIDTH pixels. vips_sequential() will use a tilecache where 
-	 * each tile is a strip the width of the image and (in this case) 
-	 * 2000 pixels high. To be able to render images wider than 
-	 * RSVG_MAX_WIDTH pixels, we need to chop these strips up.
-	 * We use a small tilecache between the seq and our gen to do this.
-	 *
-	 * Make tiles 2000 pixels high to limit overcomputation. 
+	/* Enough tiles for two complete rows.
 	 */
 	t[0] = vips_image_new(); 
-	vips_foreign_load_svg_parse( svg, t[0] ); 
-	if( vips_image_generate( t[0], 
-		NULL, vips_foreign_load_svg_generate, NULL, svg, NULL ) ||
+	if( vips_foreign_load_svg_parse( svg, t[0] ) ||
+		vips_image_generate( t[0], NULL,
+			vips_foreign_load_svg_generate, NULL, svg, NULL ) ||
 		vips_tilecache( t[0], &t[1],
-			"tile_width", VIPS_MIN( t[0]->Xsize, RSVG_MAX_WIDTH ),
-			"tile_height", 2000,
-			"max_tiles", 1,
+			"tile_width", TILE_SIZE,
+			"tile_height", TILE_SIZE,
+			"max_tiles", 2 * (1 + t[0]->Xsize / TILE_SIZE),
 			NULL ) ||
-		vips_sequential( t[1], &t[2], 
-			"tile_height", 2000, 
-			NULL ) ||
-		vips_image_write( t[2], load->real ) ) 
+		vips_image_write( t[1], load->real ) ) 
 		return( -1 );
 
 	return( 0 );
@@ -704,6 +677,7 @@ vips_foreign_load_svg_class_init( VipsForeignLoadSvgClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsOperationClass *operation_class = VIPS_OPERATION_CLASS( class );
 	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
 	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
 
@@ -711,9 +685,14 @@ vips_foreign_load_svg_class_init( VipsForeignLoadSvgClass *class )
 	gobject_class->set_property = vips_object_set_property;
 	gobject_class->get_property = vips_object_get_property;
 
-	object_class->nickname = "svgload";
+	object_class->nickname = "svgload_base";
 	object_class->description = _( "load SVG with rsvg" );
 
+	/* librsvg has not been fuzzed, so should not be used with
+	 * untrusted input unless you are very careful.
+	 */
+	operation_class->flags |= VIPS_OPERATION_UNTRUSTED;
+
 	/* is_a() is not that quick ... lower the priority.
 	 */
 	foreign_class->priority = -5;
@@ -772,7 +751,7 @@ gboolean
 vips_foreign_load_svg_source_is_a_source( VipsSource *source )
 {
 	unsigned char *data;
-	size_t bytes_read;
+	gint64 bytes_read;
 
 	if( (bytes_read = vips_source_sniff_at_most( source, 
 		&data, SVG_HEADER_SIZE )) <= 0 )
@@ -796,7 +775,7 @@ vips_foreign_load_svg_source_header( VipsForeignLoad *load )
 	if( vips_source_rewind( source->source ) )
 		return( -1 );
 
-	gstream = g_input_stream_new_from_vips( source->source );
+	gstream = vips_g_input_stream_new_from_source( source->source );
 	if( !(svg->page = rsvg_handle_new_from_stream_sync( 
 		gstream, NULL, flags, NULL, &error )) ) {
 		g_object_unref( gstream );
@@ -826,6 +805,7 @@ vips_foreign_load_svg_source_class_init( VipsForeignLoadSvgSourceClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsOperationClass *operation_class = VIPS_OPERATION_CLASS( class );
 	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
 
 	gobject_class->set_property = vips_object_set_property;
@@ -834,6 +814,8 @@ vips_foreign_load_svg_source_class_init( VipsForeignLoadSvgSourceClass *class )
 	object_class->nickname = "svgload_source";
 	object_class->description = _( "load svg from source" );
 
+	operation_class->flags |= VIPS_OPERATION_NOCACHE;
+
 	load_class->is_a_source = vips_foreign_load_svg_source_is_a_source;
 	load_class->header = vips_foreign_load_svg_source_header;
 	load_class->load = vips_foreign_load_svg_source_load;
@@ -1015,7 +997,7 @@ vips_foreign_load_svg_buffer_init( VipsForeignLoadSvgBuffer *buffer )
 {
 }
 
-#endif /*HAVE_RSVG && HAVE_GIO*/
+#endif /*HAVE_RSVG*/
 
 /**
  * vips_svgload:
@@ -1101,6 +1083,45 @@ vips_svgload_buffer( void *buf, size_t len, VipsImage **out, ... )
 	return( result );
 }
 
+/**
+ * vips_svgload_string:
+ * @str: string to load
+ * @out: (out): image to write
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Optional arguments:
+ *
+ * * @dpi: %gdouble, render at this DPI
+ * * @scale: %gdouble, scale render by this factor
+ * * @unlimited: %gboolean, allow SVGs of any size
+ *
+ * Exactly as vips_svgload(), but read from a string. This function takes a
+ * copy of the string.
+ *
+ * See also: vips_svgload().
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_svgload_string( const char *str, VipsImage **out, ... )
+{
+	va_list ap;
+	VipsBlob *blob;
+	int result;
+
+	/* Copy the string.
+	 */
+	blob = vips_blob_copy( (const void *) str, strlen( str ) );
+
+	va_start( ap, out );
+	result = vips_call_split( "svgload_buffer", ap, blob, out );
+	va_end( ap );
+
+	vips_area_unref( VIPS_AREA( blob ) );
+
+	return( result );
+}
+
 /**
  * vips_svgload_source:
  * @source: source to load from
