@@ -144,7 +144,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <setjmp.h>
 #include <math.h>
 
 #include <vips/vips.h>
@@ -173,12 +172,6 @@ vips__new_output_message( j_common_ptr cinfo )
 #ifdef DEBUG
 	printf( "vips__new_output_message: \"%s\"\n", buffer );
 #endif /*DEBUG*/
-
-	/* This is run for things like file truncated. Signal invalidate to
-	 * force this op out of cache. 
-	 */
-        if( cinfo->client_data )
-		vips_foreign_load_invalidate( VIPS_IMAGE( cinfo->client_data ) );
 }
 
 /* New error_exit handler.
