@@ -14,6 +14,8 @@
  * 	- support GIF optimization
  * 21/4/21 kleisauke
  * 	- include GObject part from magicksave.c
+ * 9/12/21 [erik-frontify]
+ * 	- add gif save subclass
  */
 
 /*
@@ -46,7 +48,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -67,6 +69,7 @@ typedef struct _VipsForeignSaveMagick {
 	char *filename;		/* NULL during buffer output */
 	char *format;
 	int quality;
+	int bitdepth;
 	gboolean optimize_gif_frames;
 	gboolean optimize_gif_transparency;
 
@@ -404,6 +407,21 @@ vips_foreign_save_magick_build( VipsObject *object )
 			return( -1 );
 		}
 	}
+	
+	/* Bitdepth <= 8 requested? Quantize/Dither images.
+	 * ImageMagick then selects the appropriate bit depth when writing
+	 * the actual image (e.g. BMP or GIF).
+	 */
+	if( magick->bitdepth ) {
+		if ( !magick_quantize_images( magick->images,
+			magick->bitdepth, magick->exception ) ) {
+			magick_inherit_exception( magick->exception,
+				magick->images );
+			magick_vips_error( class->nickname, magick->exception );
+
+			return( -1 );
+		}
+	}
 
 	return( 0 );
 }
@@ -417,6 +435,7 @@ vips_foreign_save_magick_build( VipsObject *object )
  */
 static const char *vips__save_magick_suffs[] = { NULL };
 static const char *vips__save_magick_bmp_suffs[] = { ".bmp", NULL };
+static const char *vips__save_magick_gif_suffs[] = { ".gif", NULL };
 
 /* Save a bit of typing.
  */
@@ -426,9 +445,9 @@ static const char *vips__save_magick_bmp_suffs[] = { ".bmp", NULL };
 #define F VIPS_FORMAT_FLOAT
 #define D VIPS_FORMAT_DOUBLE
 
-static int bandfmt_magick[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, UC, US, US, UI, UI, F,  F,  D,  D
+static VipsBandFormat bandfmt_magick[10] = {
+	/* Band format:  UC  C   US  S   UI  I   F  X  D  DX */
+	/* Promotion: */ UC, UC, US, US, UI, UI, F, F, D, D
 };
 
 static void
@@ -436,6 +455,7 @@ vips_foreign_save_magick_class_init( VipsForeignSaveMagickClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsOperationClass *operation_class = (VipsOperationClass *) class;
 	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
 	VipsForeignSaveClass *save_class = (VipsForeignSaveClass *) class;
 
@@ -447,6 +467,11 @@ vips_foreign_save_magick_class_init( VipsForeignSaveMagickClass *class )
 	object_class->description = _( "save with ImageMagick" );
 	object_class->build = vips_foreign_save_magick_build;
 
+	/* *magick is fuzzed, but it's such a huge thing it's safer to
+	 * disable it.
+	 */
+	operation_class->flags |= VIPS_OPERATION_UNTRUSTED;
+
 	/* We need to be well to the back of the queue since vips's
 	 * dedicated savers are usually preferable.
 	 */
@@ -484,6 +509,13 @@ vips_foreign_save_magick_class_init( VipsForeignSaveMagickClass *class )
 		G_STRUCT_OFFSET( VipsForeignSaveMagick, 
 			optimize_gif_transparency ),
 		FALSE );
+		
+	VIPS_ARG_INT( class, "bitdepth", 6,
+		_( "Bit depth" ),
+		_( "Number of bits per pixel" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveMagick, bitdepth ),
+		0, 8, 0);
 }
 
 static void
@@ -493,6 +525,7 @@ vips_foreign_save_magick_init( VipsForeignSaveMagick *magick )
 	 * int array later.
 	 */
 	g_value_init( &magick->delay_gvalue, G_TYPE_INT );
+	magick->bitdepth = 0;
 }
 
 typedef struct _VipsForeignSaveMagickFile {
@@ -559,38 +592,6 @@ vips_foreign_save_magick_file_init( VipsForeignSaveMagickFile *file )
 {
 }
 
-typedef VipsForeignSaveMagickFile VipsForeignSaveMagickBmpFile;
-typedef VipsForeignSaveMagickFileClass VipsForeignSaveMagickBmpFileClass;
-
-G_DEFINE_TYPE( VipsForeignSaveMagickBmpFile, vips_foreign_save_magick_bmp_file, 
-	vips_foreign_save_magick_file_get_type() );
-
-static void
-vips_foreign_save_magick_bmp_file_class_init( 
-	VipsForeignSaveMagickBmpFileClass *class )
-{
-	VipsObjectClass *object_class = (VipsObjectClass *) class;
-	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
-	VipsOperationClass *operation_class = (VipsOperationClass *) class;
-
-	object_class->nickname = "magicksave_bmp";
-	object_class->description = _( "save bmp image with ImageMagick" );
-
-	foreign_class->suffs = vips__save_magick_bmp_suffs;
-
-	/* Hide from UI.
-	 */
-	operation_class->flags = VIPS_OPERATION_DEPRECATED;
-}
-
-static void
-vips_foreign_save_magick_bmp_file_init( VipsForeignSaveMagickBmpFile *file )
-{
-	VipsForeignSaveMagick *magick = (VipsForeignSaveMagick *) file;
-
-	VIPS_SETSTR( magick->format, "bmp" );
-}
-
 typedef struct _VipsForeignSaveMagickBuffer {
 	VipsForeignSaveMagick parent_object;
 
@@ -664,6 +665,38 @@ vips_foreign_save_magick_buffer_init( VipsForeignSaveMagickBuffer *buffer )
 {
 }
 
+typedef VipsForeignSaveMagickFile VipsForeignSaveMagickBmpFile;
+typedef VipsForeignSaveMagickFileClass VipsForeignSaveMagickBmpFileClass;
+
+G_DEFINE_TYPE( VipsForeignSaveMagickBmpFile, vips_foreign_save_magick_bmp_file, 
+	vips_foreign_save_magick_file_get_type() );
+
+static void
+vips_foreign_save_magick_bmp_file_class_init( 
+	VipsForeignSaveMagickBmpFileClass *class )
+{
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
+	VipsOperationClass *operation_class = (VipsOperationClass *) class;
+
+	object_class->nickname = "magicksave_bmp";
+	object_class->description = _( "save bmp image with ImageMagick" );
+
+	foreign_class->suffs = vips__save_magick_bmp_suffs;
+
+	/* Hide from UI.
+	 */
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
+}
+
+static void
+vips_foreign_save_magick_bmp_file_init( VipsForeignSaveMagickBmpFile *file )
+{
+	VipsForeignSaveMagick *magick = (VipsForeignSaveMagick *) file;
+
+	VIPS_SETSTR( magick->format, "bmp" );
+}
+
 typedef VipsForeignSaveMagickBuffer VipsForeignSaveMagickBmpBuffer;
 typedef VipsForeignSaveMagickBufferClass VipsForeignSaveMagickBmpBufferClass;
 
@@ -686,7 +719,7 @@ vips_foreign_save_magick_bmp_buffer_class_init(
 
 	/* Hide from UI.
 	 */
-	operation_class->flags = VIPS_OPERATION_DEPRECATED;
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
 }
 
 static void
@@ -698,4 +731,70 @@ vips_foreign_save_magick_bmp_buffer_init(
 	VIPS_SETSTR( magick->format, "bmp" );
 }
 
+typedef VipsForeignSaveMagickFile VipsForeignSaveMagickGifFile;
+typedef VipsForeignSaveMagickFileClass VipsForeignSaveMagickGifFileClass;
+
+G_DEFINE_TYPE( VipsForeignSaveMagickGifFile, vips_foreign_save_magick_gif_file, 
+	vips_foreign_save_magick_file_get_type() );
+
+static void
+vips_foreign_save_magick_gif_file_class_init( 
+	VipsForeignSaveMagickGifFileClass *class )
+{
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
+	VipsOperationClass *operation_class = (VipsOperationClass *) class;
+
+	object_class->nickname = "magicksave_gif";
+	object_class->description = _( "save gif image with ImageMagick" );
+
+	foreign_class->suffs = vips__save_magick_gif_suffs;
+
+	/* Hide from UI.
+	 */
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
+}
+
+static void
+vips_foreign_save_magick_gif_file_init( VipsForeignSaveMagickGifFile *file )
+{
+	VipsForeignSaveMagick *magick = (VipsForeignSaveMagick *) file;
+
+	VIPS_SETSTR( magick->format, "gif" );
+}
+
+typedef VipsForeignSaveMagickBuffer VipsForeignSaveMagickGifBuffer;
+typedef VipsForeignSaveMagickBufferClass VipsForeignSaveMagickGifBufferClass;
+
+G_DEFINE_TYPE( VipsForeignSaveMagickGifBuffer, 
+	vips_foreign_save_magick_gif_buffer, 
+	vips_foreign_save_magick_buffer_get_type() );
+
+static void
+vips_foreign_save_magick_gif_buffer_class_init(
+	VipsForeignSaveMagickGifBufferClass *class )
+{
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
+	VipsOperationClass *operation_class = (VipsOperationClass *) class;
+
+	object_class->nickname = "magicksave_gif_buffer";
+	object_class->description = _( "save gif image to magick buffer" );
+
+	foreign_class->suffs = vips__save_magick_gif_suffs;
+
+	/* Hide from UI.
+	 */
+	operation_class->flags |= VIPS_OPERATION_DEPRECATED;
+}
+
+static void
+vips_foreign_save_magick_gif_buffer_init( 
+	VipsForeignSaveMagickGifBuffer *buffer )
+{
+	VipsForeignSaveMagick *magick = (VipsForeignSaveMagick *) buffer;
+
+	VIPS_SETSTR( magick->format, "gif" );
+}
+
 #endif /*ENABLE_MAGICKSAVE*/
