@@ -85,6 +85,8 @@
  *	-  add "unlimited" flag to png load
  * 13/1/22
  * 	- raise libpng pixel size limit to VIPS_MAX_COORD 
+ * 17/11/22
+ * 	- add exif read/write
  */
 
 /*
@@ -393,7 +395,6 @@ png2vips_header( Read *read, VipsImage *out )
 	png_uint_32 width, height;
 	int bitdepth, color_type;
 	int interlace_type;
-	VipsDemandStyle hint;
 
 	png_uint_32 res_x, res_y;
 	int unit_type;
@@ -524,17 +525,7 @@ png2vips_header( Read *read, VipsImage *out )
 	VIPS_SETSTR( out->filename, 
 		vips_connection_filename( VIPS_CONNECTION( read->source ) ) );
 
-	/* Uninterlaced images will be read in seq mode. Interlaced images are
-	 * read via a huge memory buffer.
-	 */
-	if( interlace_type == PNG_INTERLACE_NONE ) 
-		/* Sequential mode needs thinstrip to work with things like
-		 * vips_shrink().
-		 */
-		hint = VIPS_DEMAND_STYLE_THINSTRIP;
-	else 
-		hint = VIPS_DEMAND_STYLE_ANY;
-	if( vips_image_pipelinev( out, hint, NULL ) )
+	if( vips_image_pipelinev( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL ) )
 		return( -1 );
 
 	/* Fetch the ICC profile. @name is useless, something like "icc" or
@@ -649,6 +640,17 @@ png2vips_header( Read *read, VipsImage *out )
 }
 #endif /*PNG_bKGD_SUPPORTED*/
 
+#ifdef PNG_eXIf_SUPPORTED
+{
+	png_uint_32 num_exif;
+	png_bytep exif;
+
+	if( png_get_eXIf_1( read->pPng, read->pInfo, &num_exif, &exif ) )
+		vips_image_set_blob_copy( out, VIPS_META_EXIF_NAME, 
+			exif, num_exif );
+}
+#endif /*PNG_eXIf_SUPPORTED*/
+
 	return( 0 );
 }
 
@@ -1077,7 +1079,7 @@ write_vips( Write *write,
 	}
 
 #ifdef HAVE_QUANTIZATION
-	/* Enable image quantisation to paletted 8bpp PNG if colours is set.
+	/* Enable image quantisation to paletted 8bpp PNG if palette is set.
 	 */
 	if( palette ) 
 		color_type = PNG_COLOR_TYPE_PALETTE;
@@ -1184,8 +1186,30 @@ write_vips( Write *write,
 			g_free( str );
 		}
 
-		if( vips_image_map( in,
-			write_png_comment, write ) )
+#ifdef PNG_eXIf_SUPPORTED
+		if( vips_image_get_typeof( in, VIPS_META_EXIF_NAME ) ) {
+			const void *data;
+			size_t length;
+
+			if( vips__exif_update( in ) ||
+				vips_image_get_blob( in, VIPS_META_EXIF_NAME, 
+					&data, &length ) )
+				return( -1 );
+
+			/* libpng does not want the JFIF "Exif\0\0" prefix.
+			 */
+			if( length >= 6 &&
+				vips_isprefix( "Exif", (char *) data ) ) {
+				data += 6;
+				length -= 6;
+			}
+
+			png_set_eXIf_1( write->pPng, write->pInfo,
+				length, (png_bytep) data );
+		}
+#endif /*PNG_eXIf_SUPPORTED*/
+
+		if( vips_image_map( in, write_png_comment, write ) )
 			return( -1 );
 	}
 
