@@ -30,6 +30,10 @@
  * 	- hide warnings is VIPS_WARNING is set
  * 20/4/19
  * 	- set the min stack, if we can
+ * 17/9/21
+ * 	- don't use atexit for cleanup, it's too unreliable ... users should
+ * 	  call vips_shutdown explicitly if they want a clean exit, though a
+ * 	  dirty exit is fine
  */
 
 /*
@@ -70,7 +74,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #ifdef HAVE_PTHREAD_DEFAULT_NP
 #include <pthread.h>
@@ -90,10 +94,17 @@
 #include <limits.h>
 #include <string.h>
 
+/* Disable deprecation warnings from gsf. There are loads, and still not
+ * patched as of 12/2020.
+ */
 #ifdef HAVE_GSF
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #include <gsf/gsf.h>
+#pragma GCC diagnostic pop
 #endif /*HAVE_GSF*/
 
+#define VIPS_DISABLE_DEPRECATION_WARNINGS
 #include <vips/vips.h>
 #include <vips/thread.h>
 #include <vips/internal.h>
@@ -120,6 +131,10 @@ GTimer *vips__global_timer = NULL;
  */
 static char *vips__argv0 = NULL;
 
+/* Keep a copy of the last component of argv0 here.
+ */
+static char *vips__prgname = NULL;
+
 /* Leak check on exit.
  */
 int vips__leak = 0;
@@ -137,7 +152,8 @@ static gint64 vips_pipe_read_limit = 1024 * 1024 * 1024;
  *
  * See also: VIPS_INIT().
  *
- * Returns: a pointer to an internal copy of the argv0 string passed to
+ * Returns: (transfer none): a pointer to an internal copy of the 
+ * argv0 string passed to
  * VIPS_INIT(). Do not free this value
  */
 const char *
@@ -146,13 +162,31 @@ vips_get_argv0( void )
 	return( vips__argv0 );
 }
 
+/**
+ * vips_get_prgname:
+ *
+ * Return the program name. This can be useful for the user tio see,.
+ *
+ * See also: VIPS_INIT().
+ *
+ * Returns: (transfer none): a pointer to an internal copy of the program 
+ * name. Do not free this value
+ */
+const char *
+vips_get_prgname( void )
+{
+	const char *prgname;
+
+	if( (prgname = g_get_prgname()) )
+		return( prgname );
+	else
+		return( vips__prgname );
+}
+
 /**
  * VIPS_INIT:
  * @ARGV0: name of application
  *
- * gtk-doc mistakenly tags this macro as deprecated for unknown reasons. It is
- * *NOT* deprecated, please ignore the warning above. 
- *
  * VIPS_INIT() starts up the world of VIPS. You should call this on
  * program startup before using any other VIPS operations. If you do not call
  * VIPS_INIT(), VIPS will call it for you when you use your first VIPS 
@@ -198,9 +232,6 @@ vips_get_argv0( void )
  * + loads any plugins from $libdir/vips-x.y/, where x and y are the
  *   major and minor version numbers for this VIPS.
  *
- * + if your platform supports atexit(), VIPS_INIT() will ask for
- *   vips_shutdown() to be called on program exit
- *
  * Example:
  *
  * |[
@@ -221,8 +252,9 @@ vips_get_argv0( void )
  * Returns: 0 on success, -1 otherwise
  */
 
-/* Load all plugins in a directory ... look for '.plg' suffix. Error if we had
- * any probs.
+#ifdef ENABLE_MODULES
+/* Load all plugins in a directory ... look for '.<G_MODULE_SUFFIX>' or
+ * '.plg' (deprecated) suffix. Error if we had any probs.
  */
 static int
 vips_load_plugins( const char *fmt, ... )
@@ -242,9 +274,7 @@ vips_load_plugins( const char *fmt, ... )
         (void) vips_vsnprintf( dir_name, VIPS_PATH_MAX - 1, fmt, ap );
         va_end( ap );
 
-#ifdef DEBUG
-	printf( "vips_load_plugins: searching \"%s\"\n", dir_name );
-#endif /*DEBUG*/
+	g_info( "searching \"%s\"", dir_name );
 
         if( !(dir = g_dir_open( dir_name, 0, NULL )) ) 
 		/* Silent success for dir not there.
@@ -253,16 +283,18 @@ vips_load_plugins( const char *fmt, ... )
 
         result = 0;
         while( (name = g_dir_read_name( dir )) )
-                if( vips_ispostfix( name, ".plg" ) ) { 
+                if( vips_ispostfix( name, "." G_MODULE_SUFFIX )
+#if ENABLE_DEPRECATED
+				|| vips_ispostfix( name, ".plg" ) 
+#endif
+			) { 
 			char path[VIPS_PATH_MAX];
 			GModule *module;
 
 			vips_snprintf( path, VIPS_PATH_MAX - 1, 
 				"%s" G_DIR_SEPARATOR_S "%s", dir_name, name );
 
-#ifdef DEBUG
-			printf( "vips_load_plugins: loading \"%s\"\n", path );
-#endif /*DEBUG*/
+			g_info( "loading \"%s\"", path );
 
 			module = g_module_open( path, G_MODULE_BIND_LAZY );
 			if( !module ) {
@@ -270,11 +302,17 @@ vips_load_plugins( const char *fmt, ... )
 					path, g_module_error() ); 
 				result = -1;
 			}
+
+			/* Modules will almost certainly create new types, so
+			 * they can't be unloaded.
+			 */
+			g_module_make_resident( module );
                 }
         g_dir_close( dir );
 
 	return( result );
 }
+#endif /*ENABLE_MODULES*/
 
 /* Install this log handler to hide warning messages.
  */
@@ -292,7 +330,7 @@ set_stacksize( guint64 size )
 {
 #ifdef HAVE_PTHREAD_DEFAULT_NP
 	pthread_attr_t attr;
-	guint64 cur_stack_size;
+	size_t cur_stack_size;
 
 	/* Don't allow stacks less than 2mb.
 	 */
@@ -318,17 +356,12 @@ set_stacksize( guint64 size )
 static void
 vips_verbose( void ) 
 {
-	/* Older glibs were showing G_LOG_LEVEL_{INFO,DEBUG} messages
-	 * by default
-	 */
-#if GLIB_CHECK_VERSION ( 2, 31, 0 )
 	const char *old;
 
 	old = g_getenv( "G_MESSAGES_DEBUG" );
 
-	if( !old ) {
+	if( !old ) 
 		g_setenv( "G_MESSAGES_DEBUG", G_LOG_DOMAIN, TRUE );
-	}
 	else if( !g_str_equal( old, "all" ) &&
 		!g_strrstr( old, G_LOG_DOMAIN ) ) {
 		char *new;
@@ -338,7 +371,51 @@ vips_verbose( void )
 
 		g_free( new );
 	}
-#endif /*GLIB_CHECK_VERSION( 2, 31, 0 )*/
+}
+
+static int
+vips_leak( void ) 
+{
+	char txt[1024];
+	VipsBuf buf = VIPS_BUF_STATIC( txt );
+	int n_leaks;
+
+	n_leaks = 0;
+
+	n_leaks += vips__object_leak();
+	n_leaks += vips__type_leak();
+	n_leaks += vips_tracked_get_allocs();
+	n_leaks += vips_tracked_get_mem();
+	n_leaks += vips_tracked_get_files();
+
+	if( vips_tracked_get_allocs() || 
+		vips_tracked_get_mem() ||
+		vips_tracked_get_files() ) {
+		vips_buf_appendf( &buf, "memory: %d allocations, %zd bytes\n",
+			vips_tracked_get_allocs(), vips_tracked_get_mem() );
+		vips_buf_appendf( &buf, "files: %d open\n",
+			vips_tracked_get_files() );
+	}
+
+	vips_buf_appendf( &buf, "memory: high-water mark " );
+	vips_buf_append_size( &buf, vips_tracked_get_mem_highwater() );
+	vips_buf_appends( &buf, "\n" );
+
+	if( strlen( vips_error_buffer() ) > 0 ) {
+		vips_buf_appendf( &buf, "error buffer: %s", 
+			vips_error_buffer() );
+		n_leaks += strlen( vips_error_buffer() );
+	}
+
+	fprintf( stderr, "%s", vips_buf_all( &buf ) );
+
+	n_leaks += vips__print_renders();
+
+#ifdef DEBUG
+	vips_buffer_dump_all();
+#endif /*DEBUG*/
+
+	return( n_leaks );
 }
 
 /**
@@ -370,9 +447,12 @@ vips_init( const char *argv0 )
 	static gboolean started = FALSE;
 	static gboolean done = FALSE;
 	const char *vips_min_stack_size;
+	gint64 min_stack_size;
 	const char *prefix;
 	const char *libdir;
+#ifdef ENABLE_NLS
 	char *locale;
+#endif /* ENABLE_NLS */
 
 	/* Two stage done handling: 'done' means we've completed, 'started'
 	 * means we're currently initialising. Use this to prevent recursive
@@ -389,7 +469,33 @@ vips_init( const char *argv0 )
 		return( 0 );
 	started = TRUE;
 
-#ifdef OS_WIN32
+	/* Try to set a minimum stacksize, default 2mb. We need to do this
+	 * before any threads start.
+	 */
+	min_stack_size = 2 * 1024 * 1024;
+        if( (vips_min_stack_size = g_getenv( "VIPS_MIN_STACK_SIZE" )) )
+		min_stack_size = vips__parse_size( vips_min_stack_size );
+	(void) set_stacksize( min_stack_size );
+
+	if( g_getenv( "VIPS_INFO" )
+#if ENABLE_DEPRECATED
+		|| g_getenv( "IM_INFO" )
+#endif
+	)
+		vips_verbose();
+	if( g_getenv( "VIPS_PROFILE" ) )
+		vips_profile_set( TRUE );
+	if( g_getenv( "VIPS_LEAK" ) )
+		vips_leak_set( TRUE );
+	if( g_getenv( "VIPS_TRACE" ) )
+		vips_cache_set_trace( TRUE );
+	if( g_getenv( "VIPS_PIPE_READ_LIMIT" ) ) 
+		vips_pipe_read_limit = 
+			g_ascii_strtoll( g_getenv( "VIPS_PIPE_READ_LIMIT" ),
+				NULL, 10 );
+	vips_pipe_read_limit_set( vips_pipe_read_limit );
+
+#ifdef G_OS_WIN32
 	/* Windows has a limit of 512 files open at once for the fopen() family
 	 * of functions, and 2048 for the _open() family. This raises the limit
 	 * of fopen() to the same level as _open().
@@ -397,21 +503,9 @@ vips_init( const char *argv0 )
 	 * It will not go any higher than this, unfortunately.  
 	 */
 	(void) _setmaxstdio( 2048 );
-#endif /*OS_WIN32*/
-
-#ifdef HAVE_TYPE_INIT
-	/* Before glib 2.36 you have to call this on startup.
-	 */
-	g_type_init();
-#endif /*HAVE_TYPE_INIT*/
-
-	/* Older glibs need this.
-	 */
-#ifndef HAVE_THREAD_NEW
-	if( !g_thread_supported() ) 
-		g_thread_init( NULL );
-#endif /*HAVE_THREAD_NEW*/
+#endif /*G_OS_WIN32*/
 
+        vips__thread_init();
 	vips__threadpool_init();
 	vips__buffer_init();
 	vips__meta_init();
@@ -431,14 +525,7 @@ vips_init( const char *argv0 )
 		vips__global_timer = g_timer_new();
 
 	VIPS_SETSTR( vips__argv0, argv0 );
-
-	if( argv0 ) {
-		char *prgname;
-
-		prgname = g_path_get_basename( argv0 );
-		g_set_prgname( prgname );
-		g_free( prgname );
-	}
+	vips__prgname = g_path_get_basename( argv0 );
 
 	vips__thread_profile_attach( "main" );
 
@@ -451,35 +538,27 @@ vips_init( const char *argv0 )
 
 	/* Try to discover our prefix. 
 	 */
+        if( (prefix = g_getenv( "VIPSHOME" )) )
+		g_info( "VIPSHOME = %s", prefix );
 	if( !(prefix = vips_guess_prefix( argv0, "VIPSHOME" )) || 
 		!(libdir = vips_guess_libdir( argv0, "VIPSHOME" )) ) 
 		return( -1 );
 
+	g_info( "VIPS_PREFIX = %s", VIPS_PREFIX );
+	g_info( "VIPS_LIBDIR = %s", VIPS_LIBDIR );
+	g_info( "prefix = %s", prefix );
+	g_info( "libdir = %s", libdir );
+
 	/* Get i18n .mo files from $VIPSHOME/share/locale/.
 	 */
+#ifdef ENABLE_NLS
 	locale = g_build_filename( prefix, "share", "locale", NULL );
 	bindtextdomain( GETTEXT_PACKAGE, locale );
 	g_free( locale );
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
 	bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
-
-#if ENABLE_DEPRECATED
-	if( g_getenv( "VIPS_INFO" ) || 
-		g_getenv( "IM_INFO" ) ) 
-#else
-	if( g_getenv( "VIPS_INFO" ) ) 
-#endif
-		vips_verbose();
-	if( g_getenv( "VIPS_PROFILE" ) )
-		vips_profile_set( TRUE );
-	if( g_getenv( "VIPS_LEAK" ) )
-		vips_leak_set( TRUE );
-	if( g_getenv( "VIPS_TRACE" ) )
-		vips_cache_set_trace( TRUE );
-	if( g_getenv( "VIPS_PIPE_READ_LIMIT" ) ) 
-		vips_pipe_read_limit = 
-			g_ascii_strtoll( g_getenv( "VIPS_PIPE_READ_LIMIT" ),
-				NULL, 10 );
-	vips_pipe_read_limit_set( vips_pipe_read_limit );
+#endif /* HAVE_BIND_TEXTDOMAIN_CODESET */
+#endif /* ENABLE_NLS */
 
 	/* Register base vips types.
 	 */
@@ -524,13 +603,23 @@ vips_init( const char *argv0 )
 	vips_mosaicing_operation_init();
 	vips_g_input_stream_get_type(); 
 
-	/* Load any vips8 plugins from the vips libdir. Keep going, even if
-	 * some plugins fail to load. 
+#ifdef ENABLE_MODULES
+	/* Load any vips8 modules from the vips libdir. Keep going, even if
+	 * some modules fail to load. 
+	 *
+	 * Only do this if we have been built as a set of loadable
+	 * modules, or we might try loading an operation into a library that
+	 * already has that operation built in.
+	 */
+	(void) vips_load_plugins( "%s/vips-modules-%d.%d", 
+		libdir, VIPS_MAJOR_VERSION, VIPS_MINOR_VERSION );
+
+#if ENABLE_DEPRECATED
+	/* Load any vips8 plugins from the vips libdir.
 	 */
 	(void) vips_load_plugins( "%s/vips-plugins-%d.%d", 
 		libdir, VIPS_MAJOR_VERSION, VIPS_MINOR_VERSION );
 
-#if ENABLE_DEPRECATED
 	/* Load up any vips7 plugins in the vips libdir. We don't error on 
 	 * failure, it's too annoying to have VIPS refuse to start because of 
 	 * a broken plugin.
@@ -548,7 +637,8 @@ vips_init( const char *argv0 )
 		g_warning( "%s", vips_error_buffer() );
 		vips_error_clear();
 	}
-#endif
+#endif /*ENABLE_DEPRECATED*/
+#endif /*ENABLE_MODULES*/
 
 	/* Get the run-time compiler going.
 	 */
@@ -560,20 +650,11 @@ vips_init( const char *argv0 )
 	gsf_init();
 #endif /*HAVE_GSF*/
 
-	/* Register vips_shutdown(). This may well not get called and many
-	 * platforms don't support it anyway.
-	 */
-#ifdef HAVE_ATEXIT
-	atexit( vips_shutdown );
-#endif /*HAVE_ATEXIT*/
-
 #ifdef DEBUG_LEAK
 	vips__image_pixels_quark = 
 		g_quark_from_static_string( "vips-image-pixels" ); 
 #endif /*DEBUG_LEAK*/
 
-	done = TRUE;
-
 	/* If VIPS_WARNING is defined, suppress all warning messages from vips.
 	 *
 	 * Libraries should not call g_log_set_handler(), it is
@@ -581,19 +662,20 @@ vips_init( const char *argv0 )
 	 * set up if you are using libvips from something like Ruby. Allow this
 	 * env var hack as a workaround. 
 	 */
+	if( g_getenv( "VIPS_WARNING" )
 #if ENABLE_DEPRECATED
-	if( g_getenv( "VIPS_WARNING" ) ||
-		g_getenv( "IM_WARNING" ) ) 
-#else
-	if( g_getenv( "VIPS_WARNING" ) )
+		|| g_getenv( "IM_WARNING" )
 #endif
+	)
 		g_log_set_handler( G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, 
 			empty_log_handler, NULL );
 
-	/* Set a minimum stacksize, if we can.
+	/* Block any untrusted operations. This must come after plugin load.
 	 */
-        if( (vips_min_stack_size = g_getenv( "VIPS_MIN_STACK_SIZE" )) )
-		(void) set_stacksize( vips__parse_size( vips_min_stack_size ) );
+	if( g_getenv( "VIPS_BLOCK_UNTRUSTED" ) )
+		vips_block_untrusted_set( TRUE );
+
+	done = TRUE;
 
 	vips__thread_gate_stop( "init: startup" ); 
 
@@ -613,44 +695,6 @@ vips_check_init( void )
 		vips_error_clear();
 }
 
-static void
-vips_leak( void ) 
-{
-	char txt[1024];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
-
-	vips_object_print_all();
-
-	if( vips_tracked_get_allocs() || 
-		vips_tracked_get_mem() ||
-		vips_tracked_get_files() ) {
-		vips_buf_appendf( &buf, "memory: %d allocations, %zd bytes\n",
-			vips_tracked_get_allocs(), vips_tracked_get_mem() );
-		vips_buf_appendf( &buf, "files: %d open\n",
-			vips_tracked_get_files() );
-	}
-
-	vips_buf_appendf( &buf, "memory: high-water mark " );
-	vips_buf_append_size( &buf, vips_tracked_get_mem_highwater() );
-	vips_buf_appends( &buf, "\n" );
-
-	if( strlen( vips_error_buffer() ) > 0 ) 
-		vips_buf_appendf( &buf, "error buffer: %s", 
-			vips_error_buffer() );
-
-	if( vips__n_active_threads > 0 )
-		vips_buf_appendf( &buf, "threads: %d not joined\n", 
-			vips__n_active_threads ); 
-
-	fprintf( stderr, "%s", vips_buf_all( &buf ) );
-
-	vips__print_renders();
-
-#ifdef DEBUG
-	vips_buffer_dump_all();
-#endif /*DEBUG*/
-}
-
 /**
  * vips_thread_shutdown: 
  *
@@ -658,7 +702,7 @@ vips_leak( void )
  *
  * This function needs to be called when a thread that has been using vips
  * exits. It is called for you by vips_shutdown() and for any threads created
- * by vips_g_thread_new(). 
+ * within the #VipsThreadPool. 
  *
  * You will need to call it from threads created in
  * other ways or there will be memory leaks. If you do not call it, vips 
@@ -671,17 +715,24 @@ void
 vips_thread_shutdown( void )
 {
 	vips__thread_profile_detach();
+	vips__buffer_shutdown();
 }
 
 /**
  * vips_shutdown:
  *
- * Call this to drop caches and close plugins. Run with "--vips-leak" to do 
- * a leak check too. 
+ * Call this to drop caches, close plugins, terminate background threads, and
+ * finalize any internal library testing. 
+ *
+ * vips_shutdown() is optional. If you don't call it, your platform will
+ * clean up for you. The only negative consequences are that the leak checker
+ * and the profiler will not work.
  *
  * You may call VIPS_INIT() many times and vips_shutdown() many times, but you 
  * must not call VIPS_INIT() after vips_shutdown(). In other words, you cannot
- * stop and restart vips. 
+ * stop and restart libvips. 
+ *
+ * See also: vips_profile_set(), vips_leak_set().
  */
 void
 vips_shutdown( void )
@@ -698,7 +749,7 @@ vips_shutdown( void )
 
 	/* Mustn't run this more than once. Don't use the VIPS_GATE macro,
 	 * since we don't for gate start.
-	 */
+         */
 {
 	static gboolean done = FALSE;
 
@@ -707,15 +758,22 @@ vips_shutdown( void )
 }
 
 	vips__render_shutdown();
-
 	vips_thread_shutdown();
-
 	vips__thread_profile_stop();
+	vips__threadpool_shutdown();
 
 #ifdef HAVE_GSF
 	gsf_shutdown(); 
 #endif /*HAVE_GSF*/
 
+	/* Don't free vips__global_lock -- we want to be able to use
+	 * vips_error_buffer() after vips_shutdown(), since vips_leak() can
+	 * call it.
+	 */
+	VIPS_FREE( vips__argv0 );
+	VIPS_FREE( vips__prgname );
+	VIPS_FREEF( g_timer_destroy, vips__global_timer );
+
 	/* In dev releases, always show leaks. But not more than once, it's
 	 * annoying.
 	 */
@@ -725,29 +783,13 @@ vips_shutdown( void )
 	{
 		static gboolean done = FALSE;
 
-		if( !done ) 
+		if( !done ) {
+			done = TRUE;
 			vips_leak();
-
-		done = TRUE;
+		}
 	}
 }
 
-const char *
-vips__gettext( const char *msgid )
-{
-	vips_check_init();
-
-	return( dgettext( GETTEXT_PACKAGE, msgid ) );
-}
-
-const char *
-vips__ngettext( const char *msgid, const char *plural, unsigned long int n )
-{
-	vips_check_init();
-
-	return( dngettext( GETTEXT_PACKAGE, msgid, plural, n ) );
-}
-
 static gboolean
 vips_lib_info_cb( const gchar *option_name, const gchar *value, 
 	gpointer data, GError **error )
@@ -788,16 +830,7 @@ static gboolean
 vips_lib_config_cb( const gchar *option_name, const gchar *value, 
 	gpointer data, GError **error )
 {
-	char **split;
-	char *config;
-
-	split = g_strsplit( VIPS_CONFIG, ", ", -1 );
-	config = g_strjoinv( "\n", split );
-
-	printf( "%s\n", config );
-	g_strfreev( split );
-	g_free( config );
-
+	printf( "%s\n", VIPS_CONFIG );
 	vips_shutdown();
 	exit( 0 );
 }
@@ -920,13 +953,10 @@ static char *
 extract_prefix( const char *dir, const char *name )
 {
 	char edir[VIPS_PATH_MAX];
-	char vname[VIPS_PATH_MAX];
+	static char vname[VIPS_PATH_MAX];
 	int i;
 
-#ifdef DEBUG
-	printf( "extract_prefix: trying for dir = \"%s\", name = \"%s\"\n", 
-		dir, name );
-#endif /*DEBUG*/
+	g_info( "trying for dir = \"%s\", name = \"%s\"", dir, name );
 
 	/* Is dir relative? Prefix with cwd.
 	 */
@@ -962,9 +992,7 @@ extract_prefix( const char *dir, const char *name )
 	if( vips_ispostfix( vname, G_DIR_SEPARATOR_S ) )
 		vname[strlen( vname ) - 1] = '\0';
 
-#ifdef DEBUG
-	printf( "extract_prefix: canonicalised path = \"%s\"\n", vname );
-#endif /*DEBUG*/
+	g_info( "canonicalised path = \"%s\"", vname );
 
 	/* Ought to be a "/bin" at the end now.
 	 */
@@ -972,11 +1000,9 @@ extract_prefix( const char *dir, const char *name )
 		return( NULL );
 	vname[strlen( vname ) - strlen( G_DIR_SEPARATOR_S "bin" )] = '\0';
 
-#ifdef DEBUG
-	printf( "extract_prefix: found \"%s\"\n", vname );
-#endif /*DEBUG*/
+	g_info( "found \"%s\"", vname );
 
-	return( vips_strdup( NULL, vname ) );
+	return( vname );
 }
 
 /* Search a path for a file ... we overwrite the PATH string passed in.
@@ -996,10 +1022,8 @@ scan_path( char *path, const char *name )
 		vips_snprintf( str, VIPS_PATH_MAX, 
 			"%s" G_DIR_SEPARATOR_S "%s", p, name );
 
-#ifdef DEBUG
-		printf( "scan_path: looking in \"%s\" for \"%s\"\n", 
+		g_info( "looking in \"%s\" for \"%s\"", 
 			p, name );
-#endif /*DEBUG*/
 
 		if( vips_existsf( "%s", str ) && 
 			(prefix = extract_prefix( str, name )) ) {
@@ -1022,11 +1046,9 @@ find_file( const char *name )
 	if( !path )
 		return( NULL );
 
-#ifdef DEBUG
-	printf( "vips_guess_prefix: g_getenv( \"PATH\" ) == \"%s\"\n", path );
-#endif /*DEBUG*/
+	g_info( "g_getenv( \"PATH\" ) == \"%s\"", path );
 
-#ifdef OS_WIN32
+#ifdef G_OS_WIN32
 {
 	char *dir; 
 
@@ -1037,9 +1059,9 @@ find_file( const char *name )
 		"%s" G_SEARCHPATH_SEPARATOR_S "%s", dir, path );
 	g_free( dir ); 
 }
-#else /*!OS_WIN32*/
+#else /*!G_OS_WIN32*/
 	vips_strncpy( full_path, path, VIPS_PATH_MAX );
-#endif /*OS_WIN32*/
+#endif /*G_OS_WIN32*/
 
 	if( (prefix = scan_path( full_path, name )) ) 
 		return( prefix );
@@ -1054,6 +1076,17 @@ guess_prefix( const char *argv0, const char *name )
 {
 	char *prefix;
 
+	/* We've already checked for VIPSHOME. If the configure-time
+	 * library prefix looks OK, use the configure-time prefix.
+	 */
+	if( vips_existsf( "%s/vips-modules-%d.%d", 
+		VIPS_LIBDIR, VIPS_MAJOR_VERSION, VIPS_MINOR_VERSION ) ) {
+		g_info( "found %s/vips-modules-%d.%d", 
+			VIPS_LIBDIR, VIPS_MAJOR_VERSION, VIPS_MINOR_VERSION );
+		g_info( "using configure-time prefix" );
+		return( VIPS_PREFIX );
+	}
+
 	/* Try to guess from argv0.
 	 */
 	if( argv0 ) {
@@ -1061,10 +1094,7 @@ guess_prefix( const char *argv0, const char *name )
 			/* Must point to our executable.
 			 */
 			if( (prefix = extract_prefix( argv0, name )) ) {
-#ifdef DEBUG
-				printf( "vips_guess_prefix: found \"%s\" from "
-					"argv0\n", prefix );
-#endif /*DEBUG*/
+				g_info( "found \"%s\" from argv0", prefix );
 				return( prefix );
 			} 
 		}
@@ -1072,10 +1102,7 @@ guess_prefix( const char *argv0, const char *name )
 		/* Look along path for name.
 		 */
 		if( (prefix = find_file( name )) ) {
-#ifdef DEBUG
-			printf( "vips_guess_prefix: found \"%s\" from "
-				"PATH\n", prefix );
-#endif /*DEBUG*/
+			g_info( "found \"%s\" from PATH", prefix );
 			return( prefix );
 		}
         }
@@ -1098,10 +1125,7 @@ guess_prefix( const char *argv0, const char *name )
 			g_free( resolved );
 
 			if( prefix ) { 
-#ifdef DEBUG
-				printf( "vips_guess_prefix: found \"%s\" "
-					"from cwd\n", prefix );
-#endif /*DEBUG*/
+				g_info( "found \"%s\" from cwd", prefix );
 				return( prefix );
 			}
 		}
@@ -1138,17 +1162,12 @@ vips_guess_prefix( const char *argv0, const char *env_name )
 
 	/* Already set?
 	 */
-        if( (prefix = g_getenv( env_name )) ) {
-#ifdef DEBUG
-		printf( "vips_guess_prefix: found \"%s\" in environment\n", 
-			prefix );
-#endif /*DEBUG*/
+	if( (prefix = g_getenv( env_name )) )
                 return( prefix );
-	}
 
-#ifdef OS_WIN32
+#ifdef G_OS_WIN32
 	prefix = vips__windows_prefix();
-#else
+#else /*!G_OS_WIN32*/
 {
         char *basename;
 
@@ -1156,7 +1175,7 @@ vips_guess_prefix( const char *argv0, const char *env_name )
 	prefix = guess_prefix( argv0, basename );
 	g_free( basename ); 
 }
-#endif
+#endif /*G_OS_WIN32*/
 
 	g_setenv( env_name, prefix, TRUE );
 
@@ -1187,26 +1206,28 @@ const char *
 vips_guess_libdir( const char *argv0, const char *env_name )
 {
 	const char *prefix = vips_guess_prefix( argv0, env_name );
-        static char *libdir = NULL;
+	static char *libdir = NULL;
+
+	char *suffix;
 
 	if( libdir )
 		return( libdir );
 
 	/* Have we been moved since configure? If not, use the configure-time
-	 * libdir.
+	 * libdir. 
+	 *
+	 * The lib directory name can be eg. "lib", "lib64" etc. depending on
+	 * the platform, so copy that from the configure-time libdir if we can.
+	 * The configure-time LIBDIR is generated by autotools and always uses
+	 * '/', even on Windows.
 	 */
 	if( strcmp( prefix, VIPS_PREFIX ) == 0 ) 
 		libdir = VIPS_LIBDIR;
+	else if( (suffix = strrchr( VIPS_LIBDIR, '/' )) )
+		libdir = g_strdup_printf( "%s%s", prefix, suffix );
 	else
 		libdir = g_strdup_printf( "%s/lib", prefix );
 
-#ifdef DEBUG
-	printf( "vips_guess_libdir: VIPS_PREFIX = %s\n", VIPS_PREFIX );
-	printf( "vips_guess_libdir: VIPS_LIBDIR = %s\n", VIPS_LIBDIR );
-	printf( "vips_guess_libdir: prefix = %s\n", prefix );
-	printf( "vips_guess_libdir: libdir = %s\n", libdir );
-#endif /*DEBUG*/
-
 	return( libdir );
 }
 
@@ -1231,7 +1252,7 @@ vips_version_string( void )
  * Get the major, minor or micro library version, with @flag values 0, 1 and
  * 2.
  *
- * Get the ABI current, revision and age (as used by libtool) with @flag 
+ * Get the ABI current, revision and age (as used by Meson) with @flag 
  * values 3, 4, 5. 
  *
  * Returns: library version number
@@ -1279,11 +1300,39 @@ vips_leak_set( gboolean leak )
 	vips__leak = leak; 
 }
 
-/* Deprecated.
- */
-size_t
-vips__get_sizeof_vipsobject( void )
+static void *
+vips_block_untrusted_set_operation( VipsOperationClass *class, gboolean *state )
 {
-	return( sizeof( VipsObject ) ); 
+	g_assert( VIPS_IS_OPERATION_CLASS( class ) );
+
+	if( class->flags & VIPS_OPERATION_UNTRUSTED )
+		vips_operation_block_set( G_OBJECT_CLASS_NAME( class ), 
+			*state );
+
+	return( NULL );
 }
 
+/** 
+ * vips_block_untrusted_set:
+ * @state: the block state to set
+ *
+ * Set the block state on all untrusted operations. 
+ *
+ * |[
+ * vips_block_untrusted_set( TRUE );
+ * ]|
+ *
+ * Will block all untrusted operations from running.
+ *
+ * Use `vips -l` at the command-line to see the class hierarchy and which 
+ * operations are marked as untrusted.
+ *
+ * Set the environment variable `VIPS_BLOCK_UNTRUSTED` to block all untrusted
+ * operations on vips_init().
+ */
+void
+vips_block_untrusted_set( gboolean state )
+{
+	vips_class_map_all( g_type_from_name( "VipsOperation" ),
+		(VipsClassMapFn) vips_block_untrusted_set_operation, &state );
+}
