@@ -58,7 +58,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -79,6 +79,7 @@
 #endif /*HAVE_UNISTD_H*/
 
 #include <vips/vips.h>
+#include <vips/internal.h>
 
 #ifdef G_OS_WIN32
 #ifndef S_ISREG
@@ -89,6 +90,7 @@
 #endif /*G_OS_WIN32*/
 
 /* Does this fd support mmap. Pipes won't, for example.
+ * FIXME unused internal function
  */
 gboolean
 vips__mmap_supported( int fd )
