@@ -42,7 +42,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -430,12 +430,12 @@ vips_object_dump( VipsObject *object, VipsBuf *buf )
 }
 
 void
-vips_object_print_summary_class( VipsObjectClass *class )
+vips_object_print_summary_class( VipsObjectClass *klass )
 {
 	char str[2048];
 	VipsBuf buf = VIPS_BUF_STATIC( str );
 
-	vips_object_summary_class( class, &buf );
+	vips_object_summary_class( klass, &buf );
 	printf( "%s\n", vips_buf_all( &buf ) );
 }
 
@@ -784,6 +784,7 @@ vips_object_get_argument( VipsObject *object, const char *name,
 			_( "no vips argument named `%s'" ), name );
 		return( -1 );
 	}
+
 	if( !(*argument_instance = vips__argument_get_instance( 
 		*argument_class, object )) ) {
 		vips_error( class->nickname, 
@@ -1562,6 +1563,10 @@ vips_object_class_init( VipsObjectClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 
+	/* We must have threads set up before we can process.
+	 */
+	vips_check_init();
+
 	if( !vips__object_all ) {
 		vips__object_all = g_hash_table_new( 
 			g_direct_hash, g_direct_equal );
@@ -2078,11 +2083,8 @@ vips_object_set_argument_from_string( VipsObject *object,
 			return( -1 );
 		}
 
-		if( sscanf( value, "%lg", &d ) != 1 ) {
-			vips_error( class->nickname,
-				_( "'%s' is not a double" ), value );
+		if( vips_strtod( value, &d ) )
 			return( -1 );
-		}
 
 		g_value_init( &gvalue, G_TYPE_DOUBLE );
 		g_value_set_double( &gvalue, d );
