@@ -38,7 +38,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -171,6 +171,8 @@
  * @VIPS_OPERATION_SEQUENTIAL: can work sequentially with a small buffer
  * @VIPS_OPERATION_NOCACHE: must not be cached
  * @VIPS_OPERATION_DEPRECATED: a compatibility thing
+ * @VIPS_OPERATION_UNTRUSTED: not hardened for untrusted input
+ * @VIPS_OPERATION_BLOCKED: prevent this operation from running
  *
  * Flags we associate with an operation.
  *
@@ -193,6 +195,14 @@
  *
  * @VIPS_OPERATION_DEPRECATED means this is an old operation kept in vips for
  * compatibility only and should be hidden from users.
+ *
+ * @VIPS_OPERATION_UNTRUSTED means the operation depends on external libraries 
+ * which have not been hardened against attack. It should probably not be used 
+ * on untrusted input. Use vips_block_untrusted_set() to block all 
+ * untrusted operations.
+ *
+ * @VIPS_OPERATION_BLOCKED means the operation is prevented from executing. Use
+ * vips_operation_block_set() to enable and disable groups of operations.
  */
 
 /* Abstract base class for operations.
@@ -350,7 +360,6 @@ vips_operation_pspec_usage( VipsBuf *buf, GParamSpec *pspec )
 		vips_buf_appendf( buf, "%s", _( "max" ) );
 		vips_buf_appendf( buf, ": %d\n", pspec_int->maximum ); 
 	}
-
 }
 
 static void *
@@ -531,6 +540,30 @@ vips_operation_vips_operation_print_summary_arg( VipsObject *object,
 	return( NULL );
 }
 
+static int
+vips_operation_build( VipsObject *object )
+{
+	VipsOperationClass *class = VIPS_OPERATION_GET_CLASS( object );
+
+#ifdef VIPS_DEBUG
+	printf( "vips_operation_build: " ); 
+	vips_object_print_name( object );
+	printf( "\n" );
+#endif /*VIPS_DEBUG*/
+
+	if( class->flags & VIPS_OPERATION_BLOCKED ) {
+		vips_error( VIPS_OBJECT_CLASS( class )->nickname, 
+			"%s", _( "operation is blocked" ) ); 
+		return( -1 );
+	}
+
+	if( VIPS_OBJECT_CLASS( vips_operation_parent_class )->
+		build( object ) ) 
+		return( -1 );
+
+	return( 0 );
+}
+
 static void
 vips_operation_summary( VipsObject *object, VipsBuf *buf )
 {
@@ -564,10 +597,11 @@ vips_operation_class_init( VipsOperationClass *class )
 	gobject_class->finalize = vips_operation_finalize;
 	gobject_class->dispose = vips_operation_dispose;
 
-	vobject_class->nickname = "operation";
-	vobject_class->description = _( "operations" );
+	vobject_class->build = vips_operation_build;
 	vobject_class->summary = vips_operation_summary;
 	vobject_class->dump = vips_operation_dump;
+	vobject_class->nickname = "operation";
+	vobject_class->description = _( "operations" );
 
 	class->usage = vips_operation_usage;
 	class->get_flags = vips_operation_real_get_flags;
@@ -611,7 +645,7 @@ vips_operation_get_flags( VipsOperation *operation )
 void
 vips_operation_class_print_usage( VipsOperationClass *operation_class )
 {
-	char str[2048];
+	char str[4096];
 	VipsBuf buf = VIPS_BUF_STATIC( str );
 
 	operation_class->usage( operation_class, &buf );
@@ -636,9 +670,8 @@ vips_operation_invalidate( VipsOperation *operation )
  * Return a new #VipsOperation with the specified nickname. Useful for
  * language bindings. 
  *
- * You'll need to set
- * any arguments and build the operation before you can use it. See
- * vips_call() for a higher-level way to make new operations. 
+ * You'll need to set any arguments and build the operation before you can use 
+ * it. See vips_call() for a higher-level way to make new operations. 
  *
  * Returns: (transfer full): the new operation. 
  */
@@ -1393,3 +1426,58 @@ vips_call_argv( VipsOperation *operation, int argc, char **argv )
 
 	return( 0 );
 }
+
+static void *
+vips_operation_block_set_operation( VipsOperationClass *class, gboolean *state )
+{
+	g_assert( VIPS_IS_OPERATION_CLASS( class ) );
+
+#ifdef VIPS_DEBUG
+	if( ((class->flags & VIPS_OPERATION_BLOCKED) != 0) != *state )
+		VIPS_DEBUG_MSG( "vips_operation_block_set_operation: "
+			"setting block state on %s = %d\n", 
+			VIPS_OBJECT_CLASS( class )->nickname, *state );
+#endif
+
+	if( *state )
+		class->flags |= VIPS_OPERATION_BLOCKED;
+	else
+		class->flags &= ~VIPS_OPERATION_BLOCKED;
+
+	return( NULL );
+}
+
+/** 
+ * vips_operation_block_set:
+ * @name: set block state at this point and below 
+ * @state: the block state to set
+ *
+ * Set the block state on all operations in the libvips class hierarchy at 
+ * @name and below.
+ *
+ * For example:
+ *
+ * |[
+ * vips_operation_block_set( "VipsForeignLoad", TRUE );
+ * vips_operation_block_set( "VipsForeignLoadJpeg", FALSE );
+ * ]|
+ *
+ * Will block all load operations, except JPEG.
+ *
+ * Use `vips -l` at the command-line to see the class hierarchy.
+ *
+ * This call does nothing if the named operation is not found.
+ *
+ * See also: vips_block_untrusted_set().
+ */
+void
+vips_operation_block_set( const char *name, gboolean state )
+{
+	GType base;
+
+	if( (base = g_type_from_name( name )) &&
+		g_type_is_a( base, VIPS_TYPE_OPERATION ) )
+		vips_class_map_all( base, 
+			(VipsClassMapFn) vips_operation_block_set_operation, 
+			&state );
+}
