@@ -45,7 +45,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -535,5 +535,7 @@ vips_sink_disc( VipsImage *im, VipsRegionWrite write_fn, void *a )
 
 	write_free( &write );
 
+	vips_image_minimise_all( im );
+
 	return( result );
 }
