@@ -42,7 +42,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,7 +54,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
 
 #include <vips/vips.h>
 
@@ -65,7 +64,10 @@
 #include <vips/debug.h>
 #include <vips/internal.h>
 
-#define MODE_WRITE CLOEXEC (BINARYIZE (O_WRONLY | O_CREAT | O_TRUNC))
+/* libtiff needs to be able to seek and read back output files, unfortunately,
+ * so we must open read-write.
+ */
+#define MODE_READWRITE CLOEXEC (BINARYIZE (O_RDWR | O_CREAT | O_TRUNC))
 
 G_DEFINE_TYPE( VipsTarget, vips_target, VIPS_TYPE_CONNECTION );
 
@@ -76,12 +78,22 @@ vips_target_finalize( GObject *gobject )
 
 	VIPS_DEBUG_MSG( "vips_target_finalize:\n" );
 
-	VIPS_FREEF( g_byte_array_unref, target->memory_buffer ); 
+	if( target->memory_buffer ) {
+		g_string_free( target->memory_buffer, TRUE ); 
+		target->memory_buffer = NULL;
+	}
+
 	if( target->blob ) { 
 		vips_area_unref( VIPS_AREA( target->blob ) ); 
 		target->blob = NULL;
 	}
 
+	if( target->delete_on_close &&
+		target->delete_on_close_filename )
+		g_unlink( target->delete_on_close_filename );
+
+	VIPS_FREE( target->delete_on_close_filename );
+
 	G_OBJECT_CLASS( vips_target_parent_class )->finalize( gobject );
 }
 
@@ -111,7 +123,7 @@ vips_target_build( VipsObject *object )
 		/* 0644 is rw user, r group and other.
 		 */
 		if( (fd = vips_tracked_open( filename, 
-			MODE_WRITE, 0644 )) == -1 ) {
+			MODE_READWRITE, 0644 )) == -1 ) {
 			vips_error_system( errno, 
 				vips_connection_nick( connection ), 
 				"%s", _( "unable to open for write" ) ); 
@@ -133,7 +145,8 @@ vips_target_build( VipsObject *object )
 #endif /*G_OS_WIN32*/
 	}
 	else if( target->memory ) 
-		target->memory_buffer = g_byte_array_new();
+		target->memory_buffer = 
+			g_string_sized_new( VIPS_TARGET_BUFFER_SIZE );
 
 	return( 0 );
 }
@@ -143,9 +156,105 @@ vips_target_write_real( VipsTarget *target, const void *data, size_t length )
 {
 	VipsConnection *connection = VIPS_CONNECTION( target );
 
+	gint64 result;
+
 	VIPS_DEBUG_MSG( "vips_target_write_real: %zd bytes\n", length );
 
-	return( write( connection->descriptor, data, length ) );
+	if( target->memory_buffer ) {
+		VIPS_DEBUG_MSG( "vips_target_write_real: to position %zd\n", 
+			target->position );
+
+		g_string_overwrite_len( target->memory_buffer, target->position,
+			data, length );
+		target->position += length;
+		result = length;
+	}
+	else
+		result = write( connection->descriptor, data, length );
+
+	return( result );
+}
+
+static off_t 
+vips_target_seek_real( VipsTarget *target, off_t offset, int whence )
+{
+	VipsConnection *connection = VIPS_CONNECTION( target );
+	const char *nick = vips_connection_nick( connection );
+
+	off_t new_position;
+
+	VIPS_DEBUG_MSG( "vips_target_seek_real: offset = %ld, whence = %d\n", 
+		offset, whence );
+
+	if( target->memory_buffer ) {
+		switch( whence ) {
+		case SEEK_SET:
+			new_position = offset;
+			break;
+
+		case SEEK_CUR:
+			new_position = target->position + offset;
+			break;
+
+		case SEEK_END:
+			new_position = target->memory_buffer->len + offset;
+			break;
+
+		default:
+			vips_error( nick, "%s", _( "bad 'whence'" ) );
+			return( -1 );
+		}
+
+		if( new_position > target->memory_buffer->len )
+			g_string_set_size( target->memory_buffer, 
+				new_position );
+
+		target->position = new_position;
+	}
+	else
+		new_position = lseek( connection->descriptor, offset, whence );
+
+	return( new_position );
+}
+
+static gint64 
+vips_target_read_real( VipsTarget *target, void *data, size_t length )
+{
+	gint64 bytes_read;
+
+	VIPS_DEBUG_MSG( "vips_target_read_real: %zd bytes\n", length );
+
+	if( target->memory_buffer ) {
+		bytes_read = VIPS_MIN( length, 
+			target->memory_buffer->len - target->position );
+
+		VIPS_DEBUG_MSG( "    %zd bytes from memory\n", bytes_read );
+		memcpy( data, 
+			target->memory_buffer->str + 
+				target->position, 
+			bytes_read );
+		target->position += bytes_read;
+	}
+	else {
+		VipsConnection *connection = VIPS_CONNECTION( target );
+		int fd = connection->descriptor;
+
+		do { 
+			bytes_read = read( fd, data, length );
+		} while( bytes_read < 0 && errno == EINTR );
+	}
+
+	VIPS_DEBUG_MSG( "  read %zd bytes\n", bytes_read );
+
+	return( bytes_read );
+}
+
+static int
+vips_target_end_real( VipsTarget *target ) 
+{
+	VIPS_DEBUG_MSG( "vips_target_finish_real:\n" );
+
+	return( 0 );
 }
 
 static void
@@ -170,6 +279,9 @@ vips_target_class_init( VipsTargetClass *class )
 	object_class->build = vips_target_build;
 
 	class->write = vips_target_write_real;
+	class->read = vips_target_read_real;
+	class->seek = vips_target_seek_real;
+	class->end = vips_target_end_real;
 	class->finish = vips_target_finish_real;
 
 	VIPS_ARG_BOOL( class, "memory", 3, 
@@ -286,6 +398,50 @@ vips_target_new_to_memory( void )
 	return( target ); 
 }
 
+/**
+ * vips_target_new_temp:
+ * @based_on: base the temporary target on this target
+ *
+ * Create a temporary target -- either a temporary file on disc, or an area in
+ * memory, depending on what sort of target @based_on is.
+ *
+ * See also: vips_target_new_to_file().
+ *
+ * Returns: a new target.
+ */
+VipsTarget *
+vips_target_new_temp( VipsTarget *based_on )
+{
+	VipsTarget *target;
+
+	VIPS_DEBUG_MSG( "vips_target_new_temp: %p\n", based_on );
+
+	if( vips_connection_filename( VIPS_CONNECTION( based_on ) ) ) {
+		int descriptor;
+		char *filename;
+
+		if( !(filename = vips__temp_name( "%s.target" )) )
+			return( NULL );
+		if( (descriptor = 
+			vips__open_image_write( filename, TRUE )) < 0 ) {
+			g_free( filename );
+			return( NULL );
+		}
+		if( !(target = vips_target_new_to_descriptor( descriptor )) ) {
+			g_free( filename );
+			vips_tracked_close( descriptor );
+			return( NULL );
+		}
+		vips_tracked_close( descriptor );
+		target->delete_on_close = TRUE;
+		target->delete_on_close_filename = filename;
+	}
+	else
+		target = vips_target_new_to_memory();
+
+	return( target ); 
+}
+
 static int
 vips_target_write_unbuffered( VipsTarget *target, 
 	const void *data, size_t length )
@@ -294,32 +450,29 @@ vips_target_write_unbuffered( VipsTarget *target,
 
 	VIPS_DEBUG_MSG( "vips_target_write_unbuffered:\n" );
 
-	if( target->finished )
+	if( target->ended )
 		return( 0 );
 
-	if( target->memory_buffer ) 
-		g_byte_array_append( target->memory_buffer, data, length );
-	else 
-		while( length > 0 ) { 
-			gint64 bytes_written;
+	while( length > 0 ) { 
+		gint64 bytes_written;
 
-			bytes_written = class->write( target, data, length );
+		bytes_written = class->write( target, data, length );
 
-			/* n == 0 isn't strictly an error, but we treat it as 
-			 * one to make sure we don't get stuck in this loop.
-			 */
-			if( bytes_written <= 0 ) {
-				vips_error_system( errno, 
-					vips_connection_nick( 
-						VIPS_CONNECTION( target ) ),
-					"%s", _( "write error" ) ); 
-				return( -1 ); 
-			}
-
-			length -= bytes_written;
-			data += bytes_written;
+		/* n == 0 isn't strictly an error, but we treat it as 
+		 * one to make sure we don't get stuck in this loop.
+		 */
+		if( bytes_written <= 0 ) {
+			vips_error_system( errno, 
+				vips_connection_nick( 
+					VIPS_CONNECTION( target ) ),
+				"%s", _( "write error" ) ); 
+			return( -1 ); 
 		}
 
+		length -= bytes_written;
+		data += bytes_written;
+	}
+
 	return( 0 );
 }
 
@@ -376,7 +529,71 @@ vips_target_write( VipsTarget *target, const void *buffer, size_t length )
 }
 
 /**
- * vips_target_finish:
+ * vips_target_read:
+ * @target: target to operate on
+ * @buffer: store bytes here
+ * @length: length of @buffer in bytes
+ *
+ * Read up to @length bytes from @target and store the bytes in @buffer.
+ * Return the number of bytes actually read. If all bytes have been read from 
+ * the file, return 0.
+ *
+ * Arguments exactly as read(2).
+ *
+ * Reading froma  target sounds weird, but libtiff needs this for
+ * multi-page writes. This method will fail for targets like pipes.
+ *
+ * Returns: the number of bytes read, 0 on end of file, -1 on error.
+ */
+gint64
+vips_target_read( VipsTarget *target, void *buffer, size_t length )
+{
+	VipsTargetClass *class = VIPS_TARGET_GET_CLASS( target );
+
+	VIPS_DEBUG_MSG( "vips_target_read: %zd bytes\n", length );
+
+	if( vips_target_flush( target ) )
+		return( -1 );
+
+	return( class->read( target, buffer, length ) );
+}
+
+/**
+ * vips_target_seek:
+ * @target: target to operate on
+ * @position: position to seek to
+ * @whence: seek relative to beginning, offset, or end
+ *
+ * Seek the target. This behaves exactly as lseek(2).
+ *
+ * Seeking a target sounds weird, but libtiff needs this. This method will 
+ * fail for targets like pipes.
+ *
+ * Returns: the new seek position, -1 on error.
+ */
+off_t
+vips_target_seek( VipsTarget *target, off_t position, int whence )
+{
+	VipsTargetClass *class = VIPS_TARGET_GET_CLASS( target );
+
+	off_t new_position;
+
+	VIPS_DEBUG_MSG( "vips_target_seek: pos = %ld, whence = %d\n", 
+		position, whence );
+
+	if( vips_target_flush( target ) )
+		return( -1 );
+
+	new_position = class->seek( target, position, whence );
+
+	VIPS_DEBUG_MSG( "vips_target_seek: new_position = %ld\n", 
+		new_position );
+
+	return( new_position );
+}
+
+/**
+ * vips_target_end:
  * @target: target to operate on
  * @buffer: bytes to write
  * @length: length of @buffer in bytes
@@ -385,35 +602,56 @@ vips_target_write( VipsTarget *target, const void *buffer, size_t length )
  * can call it many times. 
  *
  * After a target has been finished, further writes will do nothing.
+ *
+ * Returns: 0 on success, -1 on error.
  */
-void
-vips_target_finish( VipsTarget *target )
+int
+vips_target_end( VipsTarget *target )
 {
 	VipsTargetClass *class = VIPS_TARGET_GET_CLASS( target );
 
-	VIPS_DEBUG_MSG( "vips_target_finish:\n" );
+	VIPS_DEBUG_MSG( "vips_target_end:\n" );
 
-	if( target->finished )
-		return;
+	if( target->ended )
+		return( 0 );
 
-	(void) vips_target_flush( target );
+	if( vips_target_flush( target ) )
+		return( -1 );
 
 	/* Move the target buffer into the blob so it can be read out.
 	 */
 	if( target->memory_buffer ) {
-		unsigned char *data;
+		const char *data;
 		size_t length;
 
 		length = target->memory_buffer->len;
-		data = g_byte_array_free( target->memory_buffer, FALSE );
+		data = g_string_free( target->memory_buffer, FALSE );
 		target->memory_buffer = NULL;
 		vips_blob_set( target->blob,
 			(VipsCallbackFn) vips_area_free_cb, data, length );
 	}
-	else
-		class->finish( target );
+	else {
+		if( class->end( target ) )
+			return( -1 );
+	}
 
-	target->finished = TRUE;
+	target->ended = TRUE;
+
+	return( 0 );
+}
+
+/**
+ * vips_target_finish:
+ * @target: target to operate on
+ * @buffer: bytes to write
+ * @length: length of @buffer in bytes
+ *
+ * Deprecated in favour of vips_target_end().
+ */
+void
+vips_target_finish( VipsTarget *target )
+{
+	(void) vips_target_end( target );
 }
 
 /**
@@ -435,12 +673,12 @@ vips_target_finish( VipsTarget *target )
 unsigned char *
 vips_target_steal( VipsTarget *target, size_t *length )
 {
-	unsigned char *data;
+	const char *data;
 
 	(void) vips_target_flush( target );
 
 	if( !target->memory_buffer ||
-		target->finished ) {
+		target->ended ) {
 		if( length )
 			*length = target->memory_buffer->len;
 
@@ -449,16 +687,17 @@ vips_target_steal( VipsTarget *target, size_t *length )
 
 	if( length )
 		*length = target->memory_buffer->len;
-	data = g_byte_array_free( target->memory_buffer, FALSE );
+	data = g_string_free( target->memory_buffer, FALSE );
 	target->memory_buffer = NULL;
 
-	/* We must have a valid byte array or finish will fail.
+	/* We must have a valid byte array or end will fail.
 	 */
-	target->memory_buffer = g_byte_array_new();
+	target->memory_buffer = g_string_sized_new( 0 );
 
-	vips_target_finish( target );
+	if( vips_target_end( target ) )
+		return( NULL );
 
-	return( data );
+	return( (unsigned char *) data );
 }
 
 /**
