@@ -35,7 +35,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -586,6 +586,12 @@ vips__set_create_time( int fd )
 	SYSTEMTIME st;
 	FILETIME ft;
 
+	/* Create time cannot be set on invalid or stream
+	 * (stdin, stdout, stderr) file descriptors.
+	 */
+	if( fd < 3 )
+		return;
+
 	if( (handle = (HANDLE) _get_osfhandle( fd )) == INVALID_HANDLE_VALUE )
 		return;
 	GetSystemTime( &st );
@@ -612,7 +618,8 @@ vips__open( const char *filename, int flags, int mode )
 	fd = g_open( filename, flags, mode );
 
 #ifdef G_OS_WIN32
-	if( mode & O_CREAT )
+	if( fd != -1 &&
+		(mode & O_CREAT) )
 		vips__set_create_time( fd ); 
 #endif /*G_OS_WIN32*/
 
@@ -635,7 +642,8 @@ vips__fopen( const char *filename, const char *mode )
 	fp = g_fopen( filename, mode );
 
 #ifdef G_OS_WIN32
-	if( mode[0] == 'w' )
+	if( fp && 
+		mode[0] == 'w' )
 		vips__set_create_time( _fileno( fp ) ); 
 #endif /*G_OS_WIN32*/
 
@@ -1910,18 +1918,28 @@ vips_realpath( const char *path )
 /* A very simple random number generator. See:
  * http://isthe.com/chongo/tech/comp/fnv/#FNV-source
  */
+guint32 
+vips__random_add( guint32 hash, int value )
+{
+#define FNV_ADD( HASH, VALUE8 ) (((HASH) ^ (VALUE8)) * 16777619u)
+
+	hash = FNV_ADD( hash, value & 0xff );
+	hash = FNV_ADD( hash, (value >> 8) & 0xff );
+	hash = FNV_ADD( hash, (value >> 16) & 0xff );
+	hash = FNV_ADD( hash, (value >> 24) & 0xff );
+
+	return( hash ); 
+}
+
 guint32
 vips__random( guint32 seed )
 {
-	return( 1103515245u * seed + 12345 );
-}
+	guint32 hash;
 
-guint32 
-vips__random_add( guint32 seed, int value )
-{
-	seed = ((2166136261u ^ seed) * 16777619u) ^ value;
+	hash = 2166136261u;
+	hash = vips__random_add( hash, seed );
 
-	return( vips__random( seed ) ); 
+	return( hash );
 }
 
 static void *
@@ -2006,7 +2024,7 @@ vips__get_iso8601( void )
 {
 	char *date;
 
-#ifdef HAVE_DATE_TIME_FORMAT_ISO8601
+#if GLIB_CHECK_VERSION( 2, 62, 0 )
 {
 	GDateTime *now;
 
@@ -2014,14 +2032,14 @@ vips__get_iso8601( void )
 	date = g_date_time_format_iso8601( now );
 	g_date_time_unref( now );
 }
-#else /*!HAVE_DATE_TIME_FORMAT_ISO8601*/
+#else /*!GLIB_CHECK_VERSION( 2, 62, 0 )*/
 {
 	GTimeVal now;
 
 	g_get_current_time( &now );
 	date = g_time_val_to_iso8601( &now ); 
 }
-#endif /*HAVE_DATE_TIME_FORMAT_ISO8601*/
+#endif /*GLIB_CHECK_VERSION( 2, 62, 0 )*/
 
 	return( date );
 }
