@@ -55,7 +55,7 @@
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
 
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdlib.h>
 
@@ -69,7 +69,7 @@
  */
 gboolean vips__vector_enabled = TRUE;
 
-void
+static void
 vips_vector_error( VipsVector *vector )
 {
 #ifdef HAVE_ORC
@@ -394,24 +394,24 @@ vips_vector_full( VipsVector *vector )
 	/* We can need a max of 2 constants plus one source per
 	 * coefficient, so stop if we're sure we don't have enough.
 	 */
-	if( vector->n_constant + 2 > 8 )
+	if( vector->n_constant + 2 > 8 /*ORC_MAX_CONST_VARS*/ )
 		return( TRUE );
 
 	/* You can have 8 source, and d1 counts as one of them, so +1
 	 * there.
 	 */
-	if( vector->n_source + vector->n_scanline + 1 > 7 )
+	if( vector->n_source + vector->n_scanline + 1 > 7 /*ORC_MAX_SRC_VARS - 1*/ )
 		return( TRUE );
 
 	/* Need to leave some space, so 1 spare. 
 	 */
-	if( vector->n_parameter > 7 )
+	if( vector->n_parameter > 7 /*ORC_MAX_PARAM_VARS - 1*/ )
 		return( TRUE );
 
 	/* After signalling full, some operations will add up to 4 more 
 	 * instructions as they finish up. Leave a margin.
 	 */
-	if( vector->n_instruction + 10 > 50 )
+	if( vector->n_instruction + 10 > 50 /*ORC_N_INSNS / 2*/ )
 		return( TRUE );
 
 	return( FALSE );
