@@ -41,7 +41,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -196,7 +196,7 @@ vips_getpagesize( void )
 
 		pagesize = si.dwAllocationGranularity;
 #else /*!G_OS_WIN32*/
-		pagesize = getpagesize();
+		pagesize = sysconf(_SC_PAGESIZE);
 #endif /*G_OS_WIN32*/
 
 #ifdef DEBUG_TOTAL
@@ -277,7 +277,7 @@ vips_window_new( VipsImage *im, int top, int height )
 	if( !(window = VIPS_NEW( NULL, VipsWindow )) )
 		return( NULL );
 
-	window->ref_count = 1;
+	window->ref_count = 0;
 	window->im = im;
 	window->top = 0;
 	window->height = 0;
@@ -290,6 +290,7 @@ vips_window_new( VipsImage *im, int top, int height )
 		vips_window_free( window );
 		return( NULL );
 	}
+	window->ref_count = 1;
 
 #ifdef DEBUG
 	printf( "** vips_window_new: window top = %d, height = %d (%p)\n",
@@ -342,14 +343,6 @@ vips_window_find( VipsImage *im, int top, int height )
 	return( window );
 }
 
-/* Old API. Just a compat stub now.
- */
-VipsWindow *
-vips_window_ref( VipsImage *im, int top, int height )
-{
-	return( NULL );
-}
-
 /* Update a window to make it enclose top/height. 
  */
 VipsWindow *
