@@ -17,6 +17,12 @@
  * 	- add @centre option
  * 7/3/17
  * 	- add a seq line cache
+ * 6/6/20 kleisauke
+ * 	- deprecate @centre option, it's now always on
+ * 	- fix pixel shift
+ * 	- speed up the mask construction for uchar/ushort images
+ * 22/4/22 kleisauke
+ * 	- add @gap option
  */
 
 /*
@@ -55,7 +61,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -98,20 +104,21 @@ typedef struct {
 typedef struct _VipsReducev {
 	VipsResample parent_instance;
 
-	double vshrink;		/* Shrink factor */
+	double vshrink;		/* Reduce factor */
+	double gap;		/* Reduce gap */
 
 	/* The thing we use to make the kernel.
 	 */
 	VipsKernel kernel;
 
-	/* Use centre rather than corner sampling convention.
-	 */
-	gboolean centre;
-
 	/* Number of points in kernel.
 	 */
 	int n_point;
 
+	/* Vertical displacement.
+	 */
+	double voffset;
+
 	/* Precalculated interpolation matrices. int (used for pel
 	 * sizes up to short), and double (for all others). We go to
 	 * scale + 1 so we can round-to-nearest safely.
@@ -128,6 +135,10 @@ typedef struct _VipsReducev {
 	int n_pass;	
 	Pass pass[MAX_PASS];
 
+	/* Deprecated.
+	 */
+	gboolean centre;
+
 } VipsReducev;
 
 typedef VipsResampleClass VipsReducevClass;
@@ -200,10 +211,10 @@ vips_reducev_compile_section( VipsReducev *reducev, Pass *pass, gboolean first )
 	 * of the previous pass. 
 	 */
 	if( first ) {
-		char c0[256];
+		char c32[256];
 
-		CONST( c0, 0, 2 );
-		ASM2( "loadpw", "sum", c0 );
+		CONST( c32, 32, 2 );
+		ASM2( "loadpw", "sum", c32 );
 	}
 	else 
 		ASM2( "loadw", "sum", "r" );
@@ -231,7 +242,8 @@ vips_reducev_compile_section( VipsReducev *reducev, Pass *pass, gboolean first )
 		 * of the image and coefficient are interesting, so we can take
 		 * the bottom bits of a 16x16->32 multiply. 
 		 *
-		 * We accumulate the signed 16-bit result in sum.
+		 * We accumulate the signed 16-bit result in sum. Saturated
+		 * add.
 		 */
 		ASM2( "convubw", "value", source );
 		ASM3( "mullw", "value", "value", coeff );
@@ -255,26 +267,12 @@ vips_reducev_compile_section( VipsReducev *reducev, Pass *pass, gboolean first )
 	 * image, otherwise write the 16-bit intermediate to our temp buffer. 
 	 */
 	if( pass->last >= reducev->n_point - 1 ) {
-		char c32[256];
 		char c6[256];
-		char c0[256];
-		char c255[256];
 
-		CONST( c32, 32, 2 );
-		ASM3( "addw", "sum", "sum", c32 );
 		CONST( c6, 6, 2 );
 		ASM3( "shrsw", "sum", "sum", c6 );
 
-		/* You'd think "convsuswb", convert signed 16-bit to unsigned
-		 * 8-bit with saturation, would be quicker, but it's a lot
-		 * slower.
-		 */
-		CONST( c0, 0, 2 );
-		ASM3( "maxsw", "sum", c0, "sum" ); 
-		CONST( c255, 255, 2 );
-		ASM3( "minsw", "sum", c255, "sum" ); 
-
-		ASM2( "convwb", "d1", "sum" );
+		ASM2( "convsuswb", "d1", "sum" );
 	}
 	else 
 		ASM2( "copyw", "d2", "sum" );
@@ -503,8 +501,12 @@ reducev_notab( VipsReducev *reducev,
 
 	vips_reduce_make_mask( cy, reducev->kernel, reducev->vshrink, y ); 
 
-	for( int z = 0; z < ne; z++ ) 
-		out[z] = reduce_sum<T, double>( in + z, l1, cy, n );
+	for( int z = 0; z < ne; z++ ) {
+		double sum;
+		sum = reduce_sum<T, double>( in + z, l1, cy, n );
+
+		out[z] = VIPS_ROUND_UINT( sum );
+	}
 }
 
 static int
@@ -531,22 +533,22 @@ vips_reducev_gen( VipsRegion *out_region, void *vseq,
 #endif /*DEBUG*/
 
 	s.left = r->left;
-	s.top = r->top * reducev->vshrink;
+	s.top = r->top * reducev->vshrink - reducev->voffset;
 	s.width = r->width;
 	s.height = r->height * reducev->vshrink + reducev->n_point;
-	if( reducev->centre )
-		s.height += 1;
 	if( vips_region_prepare( ir, &s ) )
 		return( -1 );
 
 	VIPS_GATE_START( "vips_reducev_gen: work" ); 
 
-	for( int y = 0; y < r->height; y ++ ) { 
+	double Y = (r->top + 0.5) * reducev->vshrink - 0.5 - 
+		reducev->voffset;
+
+	for( int y = 0; y < r->height; y++ ) { 
 		VipsPel *q = 
 			VIPS_REGION_ADDR( out_region, r->left, r->top + y );
-		const double Y = (r->top + y) * reducev->vshrink + 
-			(reducev->centre ? 0.5 : 0.0); 
-		VipsPel *p = VIPS_REGION_ADDR( ir, r->left, (int) Y ); 
+		const int py = (int) Y;
+		VipsPel *p = VIPS_REGION_ADDR( ir, r->left, py );
 		const int sy = Y * VIPS_TRANSFORM_SCALE * 2;
 		const int siy = sy & (VIPS_TRANSFORM_SCALE * 2 - 1);
 		const int ty = (siy + 1) >> 1;
@@ -606,13 +608,15 @@ vips_reducev_gen( VipsRegion *out_region, void *vseq,
 		case VIPS_FORMAT_DPCOMPLEX:
 		case VIPS_FORMAT_DOUBLE:
 			reducev_notab<double>( reducev,
-				q, p, ne, lskip, Y - (int) Y );
+				q, p, ne, lskip, Y - py );
 			break;
 
 		default:
 			g_assert_not_reached();
 			break;
 		}
+
+		Y += reducev->vshrink;
 	}
 
 	VIPS_GATE_STOP( "vips_reducev_gen: work" ); 
@@ -644,11 +648,9 @@ vips_reducev_vector_gen( VipsRegion *out_region, void *vseq,
 #endif /*DEBUG_PIXELS*/
 
 	s.left = r->left;
-	s.top = r->top * reducev->vshrink;
+	s.top = r->top * reducev->vshrink - reducev->voffset;
 	s.width = r->width;
 	s.height = r->height * reducev->vshrink + reducev->n_point;
-	if( reducev->centre )
-		s.height += 1;
 	if( vips_region_prepare( ir, &s ) )
 		return( -1 );
 
@@ -663,12 +665,13 @@ vips_reducev_vector_gen( VipsRegion *out_region, void *vseq,
 
 	VIPS_GATE_START( "vips_reducev_vector_gen: work" ); 
 
-	for( int y = 0; y < r->height; y ++ ) { 
+	double Y = (r->top + 0.5) * reducev->vshrink - 0.5 - 
+		reducev->voffset;
+
+	for( int y = 0; y < r->height; y++ ) { 
 		VipsPel *q = 
 			VIPS_REGION_ADDR( out_region, r->left, r->top + y );
-		const double Y = (r->top + y) * reducev->vshrink + 
-			(reducev->centre ? 0.5 : 0.0); 
-		const int py = (int) Y; 
+		const int py = (int) Y;
 		const int sy = Y * VIPS_TRANSFORM_SCALE * 2;
 		const int siy = sy & (VIPS_TRANSFORM_SCALE * 2 - 1);
 		const int ty = (siy + 1) >> 1;
@@ -682,7 +685,7 @@ vips_reducev_vector_gen( VipsRegion *out_region, void *vseq,
 		printf( "first column of pixel values:\n" ); 
 		for( int i = 0; i < reducev->n_point; i++ ) 
 			printf( "\t%d - %d\n", i, 
-				*VIPS_REGION_ADDR( ir, r->left, r->top + y + i ) ); 
+				*VIPS_REGION_ADDR( ir, r->left, py ) ); 
 #endif /*DEBUG_PIXELS*/
 
 		/* We run our n passes to generate this scanline.
@@ -709,6 +712,8 @@ vips_reducev_vector_gen( VipsRegion *out_region, void *vseq,
 		printf( "pixel result:\n" );
 		printf( "\t%d\n", *q ); 
 #endif /*DEBUG_PIXELS*/
+
+		Y += reducev->vshrink;
 	}
 
 	VIPS_GATE_STOP( "vips_reducev_vector_gen: work" ); 
@@ -719,48 +724,14 @@ vips_reducev_vector_gen( VipsRegion *out_region, void *vseq,
 }
 
 static int
-vips_reducev_raw( VipsReducev *reducev, VipsImage *in, VipsImage **out ) 
+vips_reducev_raw( VipsReducev *reducev, VipsImage *in, int height, 
+		VipsImage **out ) 
 {
 	VipsObjectClass *object_class = VIPS_OBJECT_GET_CLASS( reducev );
-	VipsResample *resample = VIPS_RESAMPLE( reducev );
 
 	VipsGenerateFn generate;
 
-	/* Build masks.
-	 */
-	for( int y = 0; y < VIPS_TRANSFORM_SCALE + 1; y++ ) {
-		reducev->matrixf[y] = 
-			VIPS_ARRAY( NULL, reducev->n_point, double ); 
-		if( !reducev->matrixf[y] )
-			return( -1 ); 
-
-		vips_reduce_make_mask( reducev->matrixf[y],
-			reducev->kernel, reducev->vshrink, 
-			(float) y / VIPS_TRANSFORM_SCALE ); 
-
-#ifdef DEBUG
-		printf( "%6.2g", (double) y / VIPS_TRANSFORM_SCALE ); 
-		for( int i = 0; i < reducev->n_point; i++ ) 
-			printf( ", %6.2g", reducev->matrixf[y][i] );
-		printf( "\n" ); 
-#endif /*DEBUG*/
-	}
-
-	/* uchar and ushort need an int version of the masks.
-	 */
-	if( VIPS_IMAGE_SIZEOF_ELEMENT( in ) <= 2 ) 
-		for( int y = 0; y < VIPS_TRANSFORM_SCALE + 1; y++ ) {
-			reducev->matrixi[y] = 
-				VIPS_ARRAY( NULL, reducev->n_point, int ); 
-			if( !reducev->matrixi[y] )
-				return( -1 ); 
-
-			vips_vector_to_fixed_point( 
-				reducev->matrixf[y], reducev->matrixi[y], 
-				reducev->n_point, VIPS_INTERPOLATE_SCALE );
-		}
-
-	/* And we need an 2.6 version if we will use the vector path.
+	/* We need an 2.6 version if we will use the vector path.
 	 */
 	if( in->BandFmt == VIPS_FORMAT_UCHAR &&
 		vips_vector_isenabled() ) 
@@ -787,18 +758,14 @@ vips_reducev_raw( VipsReducev *reducev, VipsImage *in, VipsImage **out )
 
 	*out = vips_image_new();
 	if( vips_image_pipelinev( *out, 
-		VIPS_DEMAND_STYLE_FATSTRIP, in, (void *) NULL ) )
+		VIPS_DEMAND_STYLE_THINSTRIP, in, (void *) NULL ) )
 		return( -1 );
 
-	/* Size output. We need to always round to nearest, so round(), not
-	 * rint().
-	 *
-	 * Don't change xres/yres, leave that to the application layer. For
+	/* Don't change xres/yres, leave that to the application layer. For
 	 * example, vipsthumbnail knows the true reduce factor (including the
 	 * fractional part), we just see the integer part here.
 	 */
-	(*out)->Ysize = VIPS_ROUND_UINT( 
-		resample->in->Ysize / reducev->vshrink );
+	(*out)->Ysize = height;
 	if( (*out)->Ysize <= 0 ) { 
 		vips_error( object_class->nickname, 
 			"%s", _( "image has shrunk to nothing" ) );
@@ -827,23 +794,62 @@ vips_reducev_build( VipsObject *object )
 	VipsObjectClass *object_class = VIPS_OBJECT_GET_CLASS( object );
 	VipsResample *resample = VIPS_RESAMPLE( object );
 	VipsReducev *reducev = (VipsReducev *) object;
-	VipsImage **t = (VipsImage **) vips_object_local_array( object, 4 );
+	VipsImage **t = (VipsImage **) vips_object_local_array( object, 5 );
 
 	VipsImage *in;
 	int height;
+	int int_vshrink;
+	double extra_pixels;
 
 	if( VIPS_OBJECT_CLASS( vips_reducev_parent_class )->build( object ) )
 		return( -1 );
 
 	in = resample->in; 
 
-	if( reducev->vshrink < 1 ) { 
+	if( reducev->vshrink < 1.0 ) { 
 		vips_error( object_class->nickname, 
-			"%s", _( "reduce factor should be >= 1" ) );
+			"%s", _( "reduce factor should be >= 1.0" ) );
 		return( -1 );
 	}
 
-	if( reducev->vshrink == 1 ) 
+	/* Output size. We need to always round to nearest, so round(), not
+	 * rint().
+	 */
+	height = VIPS_ROUND_UINT( 
+		(double) in->Ysize / reducev->vshrink );
+
+	/* How many pixels we are inventing in the input, -ve for
+	 * discarding.
+	 */
+	extra_pixels = height * reducev->vshrink - in->Ysize;
+
+	if( reducev->gap > 0.0 &&
+		reducev->kernel != VIPS_KERNEL_NEAREST ) {
+		if( reducev->gap < 1.0 ) {
+			vips_error( object_class->nickname,
+				"%s", _( "reduce gap should be >= 1.0" ) );
+			return( -1 );
+		}
+
+		/* The int part of our reduce.
+		 */
+		int_vshrink = VIPS_MAX( 1, VIPS_FLOOR( 
+			(double) in->Ysize / height / reducev->gap ) );
+
+		if( int_vshrink > 1 ) {
+			g_info( "shrinkv by %d", int_vshrink );
+			if( vips_shrinkv( in, &t[0], int_vshrink,
+				"ceil", TRUE,
+				NULL ) )
+				return( -1 );
+			in = t[0];
+
+			reducev->vshrink /= int_vshrink;
+			extra_pixels /= int_vshrink;
+		}
+	}
+
+	if( reducev->vshrink == 1.0 ) 
 		return( vips_image_write( in, resample->out ) );
 
 	reducev->n_point = 
@@ -855,29 +861,60 @@ vips_reducev_build( VipsObject *object )
 		return( -1 );
 	}
 
+	/* If we are rounding down, we are not using some input
+	 * pixels. We need to move the origin *inside* the input image
+	 * by half that distance so that we discard pixels equally
+	 * from left and right. 
+	 */
+	reducev->voffset = (1 + extra_pixels) / 2.0 - 1;
+
+	/* Build the tables of pre-computed coefficients.
+	 */
+	for( int y = 0; y < VIPS_TRANSFORM_SCALE + 1; y++ ) {
+		reducev->matrixf[y] = 
+			VIPS_ARRAY( NULL, reducev->n_point, double ); 
+		reducev->matrixi[y] = 
+			VIPS_ARRAY( NULL, reducev->n_point, int ); 
+		if( !reducev->matrixf[y] ||
+			!reducev->matrixi[y] )
+			return( -1 ); 
+
+		vips_reduce_make_mask( reducev->matrixf[y],
+			reducev->kernel, reducev->vshrink, 
+			(float) y / VIPS_TRANSFORM_SCALE ); 
+
+		for( int i = 0; i < reducev->n_point; i++ )
+			reducev->matrixi[y][i] = reducev->matrixf[y][i] *
+				VIPS_INTERPOLATE_SCALE;
+
+#ifdef DEBUG
+		printf( "vips_reducev_build: mask %d\n    ", y );
+		for( int i = 0; i < reducev->n_point; i++ ) 
+			printf( "%d ", reducev->matrixi[y][i] );
+		printf( "\n" ); 
+#endif /*DEBUG*/
+	}
+
 	/* Unpack for processing.
 	 */
-	if( vips_image_decode( in, &t[0] ) )
-		return( -1 );
-	in = t[0];
-
-	/* Add new pixels around the input so we can interpolate at the edges.
-	 */
-	height = in->Ysize + reducev->n_point - 1;
-	if( reducev->centre )
-		height += 1;
-	if( vips_embed( in, &t[1], 
-		0, reducev->n_point / 2 - 1, 
-		in->Xsize, height, 
-		"extend", VIPS_EXTEND_COPY,
-		(void *) NULL ) )
+	if( vips_image_decode( in, &t[1] ) )
 		return( -1 );
 	in = t[1];
 
-	if( vips_reducev_raw( reducev, in, &t[2] ) )
+	/* Add new pixels around the input so we can interpolate at the edges.
+	 */
+	if( vips_embed( in, &t[2], 
+		0, VIPS_CEIL( reducev->n_point / 2.0 ) - 1, 
+		in->Xsize, in->Ysize + reducev->n_point, 
+		"extend", VIPS_EXTEND_COPY,
+		(void *) NULL ) )
 		return( -1 );
 	in = t[2];
 
+	if( vips_reducev_raw( reducev, in, height, &t[3] ) )
+		return( -1 );
+	in = t[3];
+
 	/* Large reducev will throw off sequential mode. Suppose thread1 is
 	 * generating tile (0, 0), but stalls. thread2 generates tile
 	 * (0, 1), 128 lines further down the output. After it has done,
@@ -891,12 +928,12 @@ vips_reducev_build( VipsObject *object )
 	if( vips_image_get_typeof( in, VIPS_META_SEQUENTIAL ) ) { 
 		g_info( "reducev sequential line cache" ); 
 
-		if( vips_sequential( in, &t[3], 
+		if( vips_sequential( in, &t[4], 
 			"tile_height", 10,
 			// "trace", TRUE,
 			(void *) NULL ) )
 			return( -1 );
-		in = t[3];
+		in = t[4];
 	}
 
 	if( vips_image_write( in, resample->out ) )
@@ -930,7 +967,7 @@ vips_reducev_class_init( VipsReducevClass *reducev_class )
 		_( "Vertical shrink factor" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsReducev, vshrink ),
-		1, 1000000, 1 );
+		1.0, 1000000.0, 1.0 );
 
 	VIPS_ARG_ENUM( reducev_class, "kernel", 4, 
 		_( "Kernel" ), 
@@ -939,12 +976,12 @@ vips_reducev_class_init( VipsReducevClass *reducev_class )
 		G_STRUCT_OFFSET( VipsReducev, kernel ),
 		VIPS_TYPE_KERNEL, VIPS_KERNEL_LANCZOS3 );
 
-	VIPS_ARG_BOOL( reducev_class, "centre", 7, 
-		_( "Centre" ), 
-		_( "Use centre sampling convention" ),
+	VIPS_ARG_DOUBLE( reducev_class, "gap", 5, 
+		_( "Gap" ), 
+		_( "Reducing gap" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
-		G_STRUCT_OFFSET( VipsReducev, centre ),
-		FALSE );
+		G_STRUCT_OFFSET( VipsReducev, gap ),
+		0.0, 1000000.0, 0.0 );
 
 	/* Old name.
 	 */
@@ -953,13 +990,23 @@ vips_reducev_class_init( VipsReducevClass *reducev_class )
 		_( "Vertical shrink factor" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsReducev, vshrink ),
-		1, 1000000, 1 );
+		1.0, 1000000.0, 1.0 );
+
+	/* We used to let people pick centre or corner, but it's automatic now.
+	 */
+	VIPS_ARG_BOOL( reducev_class, "centre", 7,
+		_( "Centre" ),
+		_( "Use centre sampling convention" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
+		G_STRUCT_OFFSET( VipsReducev, centre ),
+		FALSE );
 
 }
 
 static void
 vips_reducev_init( VipsReducev *reducev )
 {
+	reducev->gap = 0.0;
 	reducev->kernel = VIPS_KERNEL_LANCZOS3;
 }
 
