@@ -7,7 +7,7 @@ import pytest
 
 import pyvips
 from helpers import \
-    JPEG_FILE, SRGB_FILE, MATLAB_FILE, PNG_FILE, TIF_FILE, OME_FILE, \
+    IMAGES, JPEG_FILE, SRGB_FILE, MATLAB_FILE, PNG_FILE, TIF_FILE, OME_FILE, \
     ANALYZE_FILE, GIF_FILE, WEBP_FILE, EXR_FILE, FITS_FILE, OPENSLIDE_FILE, \
     PDF_FILE, SVG_FILE, SVGZ_FILE, SVG_GZ_FILE, GIF_ANIM_FILE, DICOM_FILE, \
     BMP_FILE, NIFTI_FILE, ICO_FILE, TGA_FILE, SGI_FILE, AVIF_FILE, \
@@ -347,25 +347,63 @@ class TestForeign:
     @skip_if_no("jpegsave")
     @pytest.mark.xfail(raises=pyvips.error.Error, reason="requires libexif >= 0.6.22")
     def test_jpegsave_exif_2_3_ascii(self):
-        def exif_valid(im):
-            assert im.get("exif-ifd2-CameraOwnerName").find("ASCII, 14 components, 14 bytes") != -1
-            assert im.get("exif-ifd2-BodySerialNumber").find("ASCII, 14 components, 14 bytes") != -1
-            assert im.get("exif-ifd2-LensMake").find("ASCII, 14 components, 14 bytes") != -1
-            assert im.get("exif-ifd2-LensModel").find("ASCII, 14 components, 14 bytes") != -1
-            assert im.get("exif-ifd2-LensSerialNumber").find("ASCII, 14 components, 14 bytes") != -1
+        def exif_valid(exif_tags, im):
+            for exif_tag in exif_tags:
+                assert im.get(exif_tag).find("ASCII, 14 components, 14 bytes") != -1
 
         # first make sure we have exif support
         im = pyvips.Image.new_from_file(JPEG_FILE)
         if im.get_typeof("exif-ifd0-Orientation") != 0:
             x = im.copy()
-            x.set_type(pyvips.GValue.gstr_type, "exif-ifd2-CameraOwnerName", "hello ( there")
-            x.set_type(pyvips.GValue.gstr_type, "exif-ifd2-BodySerialNumber", "hello ( there")
-            x.set_type(pyvips.GValue.gstr_type, "exif-ifd2-LensMake", "hello ( there")
-            x.set_type(pyvips.GValue.gstr_type, "exif-ifd2-LensModel", "hello ( there")
-            x.set_type(pyvips.GValue.gstr_type, "exif-ifd2-LensSerialNumber", "hello ( there")
+            exif_tags = [
+                "exif-ifd2-CameraOwnerName",
+                "exif-ifd2-BodySerialNumber",
+                "exif-ifd2-LensMake",
+                "exif-ifd2-LensModel",
+                "exif-ifd2-LensSerialNumber",
+            ]
+            for exif_tag in exif_tags:
+                x.set_type(pyvips.GValue.gstr_type, exif_tag, "hello ( there")
             buf = x.jpegsave_buffer()
             y = pyvips.Image.new_from_buffer(buf, "")
-            exif_valid(y)
+            exif_valid(exif_tags, y)
+
+    @skip_if_no("jpegsave")
+    @pytest.mark.xfail(raises=pyvips.error.Error, reason="requires libexif >= 0.6.23")
+    def test_jpegsave_exif_2_3_ascii_2(self):
+        def exif_valid(exif_tags, im):
+            for exif_tag in exif_tags:
+                assert im.get(exif_tag).find("ASCII, 14 components, 14 bytes") != -1
+
+        # first make sure we have exif support
+        im = pyvips.Image.new_from_file(JPEG_FILE)
+        if im.get_typeof("exif-ifd0-Orientation") != 0:
+            x = im.copy()
+            exif_tags = [
+                "exif-ifd2-OffsetTime",
+                "exif-ifd2-OffsetTimeOriginal",
+                "exif-ifd2-OffsetTimeDigitized",
+                "exif-ifd3-GPSLatitudeRef",
+                "exif-ifd3-GPSLongitudeRef",
+                "exif-ifd3-GPSSatellites",
+                "exif-ifd3-GPSStatus",
+                "exif-ifd3-GPSMeasureMode",
+                "exif-ifd3-GPSSpeedRef",
+                "exif-ifd3-GPSTrackRef",
+                "exif-ifd3-GPSImgDirectionRef",
+                "exif-ifd3-GPSMapDatum",
+                "exif-ifd3-GPSDestLatitudeRef",
+                "exif-ifd3-GPSDestLongitudeRef",
+                "exif-ifd3-GPSDestBearingRef",
+                "exif-ifd3-GPSDestDistanceRef",
+                "exif-ifd3-GPSDateStamp",
+            ]
+            for exif_tag in exif_tags:
+                x.set_type(pyvips.GValue.gstr_type, exif_tag, "hello ( there")
+
+            buf = x.jpegsave_buffer()
+            y = pyvips.Image.new_from_buffer(buf, "")
+            exif_valid(exif_tags, y)
 
     @skip_if_no("jpegload")
     def test_truncated(self):
@@ -410,9 +448,34 @@ class TestForeign:
         len_mono1 = len(self.mono.write_to_buffer(".png", bitdepth=1))
         assert( len_mono1 < len_mono2 )
 
+        # take a 1-bit image to png and back
+        onebit = self.mono > 128
+        data = onebit.write_to_buffer(".png", bitdepth=1)
+        after = pyvips.Image.new_from_buffer(data, "")
+        assert( (onebit - after).abs().max() == 0 )
+
         # we can't test palette save since we can't be sure libimagequant is
         # available and there's no easy test for its presence
 
+        # see if we have exif parsing: our test jpg image has this field
+        x = pyvips.Image.new_from_file(JPEG_FILE)
+        if x.get_typeof("exif-ifd0-Orientation") != 0:
+            # we need a copy of the image to set the new metadata on
+            # otherwise we get caching problems
+
+            # can set, save and load new orientation
+            x = pyvips.Image.new_from_file(JPEG_FILE)
+            x = x.copy()
+
+            x.set("orientation", 2)
+
+            filename = temp_filename(self.tempdir, '.png')
+            x.write_to_file(filename)
+
+            x = pyvips.Image.new_from_file(filename)
+            y = x.get("orientation")
+            assert y == 2
+
     @skip_if_no("tiffload")
     def test_tiff(self):
         def tiff_valid(im):
@@ -968,6 +1031,32 @@ class TestForeign:
         x2 = pyvips.Image.new_from_file(GIF_ANIM_DISPOSE_PREVIOUS_EXPECTED_PNG_FILE)
         assert (x1 - x2).abs().max() == 0
 
+    @skip_if_no("gifload")
+    def test_gifload_truncated(self):
+        # should load with just a warning
+        truncated_gif = os.path.join(IMAGES, "truncated.gif")
+        im = pyvips.Image.new_from_file(truncated_gif)
+        assert im.width == 575
+
+        # should fail on truncation and warning
+        with pytest.raises(Exception):
+            im = pyvips.Image.new_from_file(truncated_gif, fail_on="warning")
+        with pytest.raises(Exception):
+            im = pyvips.Image.new_from_file(truncated_gif, fail_on="truncated")
+
+    @skip_if_no("gifload")
+    def test_gifload_frame_error(self):
+        # should load with just a warning
+        truncated_gif = os.path.join(IMAGES, "garden.gif")
+        im = pyvips.Image.new_from_file(truncated_gif)
+        assert im.width == 800
+
+        # should fail on warning only
+        im = pyvips.Image.new_from_file(truncated_gif, fail_on="truncated")
+        assert im.width == 800
+        with pytest.raises(Exception):
+            im = pyvips.Image.new_from_file(truncated_gif, fail_on="warning")
+
     @skip_if_no("svgload")
     def test_svgload(self):
         def svg_valid(im):
@@ -1389,14 +1478,12 @@ class TestForeign:
 
         # scrgb mode
         scrgb = self.colour.colourspace("scrgb")
-        no_profile.remove("icc-profile-data")
         self.save_load_buffer("jxlsave_buffer", "jxlload_buffer",
                               scrgb, 120)
 
         # scrgb mode, no profile
         scrgb_no_profile = scrgb.copy()
         scrgb_no_profile.remove("icc-profile-data")
-        no_profile.remove("icc-profile-data")
         self.save_load_buffer("jxlsave_buffer", "jxlload_buffer",
                               scrgb_no_profile, 120)
 
@@ -1434,6 +1521,16 @@ class TestForeign:
         # FIXME ... this requires cgif0.3 or later for fixed loop support
         # assert x1.get("loop") == x2.get("loop")
 
+        # Interlaced write
+        x1 = pyvips.Image.new_from_file(GIF_FILE, n=-1)
+        b1 = x1.gifsave_buffer(interlace=False)
+        b2 = x1.gifsave_buffer(interlace=True)
+        # Interlaced GIFs are usually larger in file size
+        # FIXME ... cgif v0.3 or later required for interlaced write.
+        # If interlaced write is not supported b2 and b1 are expected to be
+        # of the same file size.
+        assert len(b2) >= len(b1)
+
         # Reducing dither will typically reduce file size (and quality)
         little_dither = self.colour.gifsave_buffer(dither=0.1, effort=1)
         large_dither = self.colour.gifsave_buffer(dither=0.9, effort=1)
