@@ -50,14 +50,16 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif /*HAVE_UNISTD_H*/
 #include <locale.h>
 
 #include <vips/vips.h>
@@ -138,7 +140,10 @@ main( int argc, char **argv )
 
 	if( VIPS_INIT( argv[0] ) )
 	        vips_error_exit( "%s", _( "unable to start VIPS" ) );
+
+#ifdef ENABLE_NLS
 	textdomain( GETTEXT_PACKAGE );
+#endif /* ENABLE_NLS */
 	setlocale( LC_ALL, "" );
 
 {
